@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2012 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,14 +19,14 @@
 """
 
 """
-Application module.
-
-Handles application initialization, shutdown, opening projects, autosave and changing
-sequences.
+Handles syncing clips based on their audio data.
 """
 import datetime
-import md5
-import mlt
+import hashlib
+try:
+    import mlt7 as mlt
+except:
+    import mlt
 import os
 import subprocess
 import sys
@@ -44,7 +44,6 @@ import clapperless
 import dialogs
 import dialogutils
 import edit
-import editorpersistance
 import editorstate
 from editorstate import PROJECT
 from editorstate import current_sequence
@@ -55,12 +54,15 @@ import renderconsumer
 import respaths
 import tlinewidgets
 import updater
+import userfolders
 import utils
 
 _tline_sync_data = None # Compound clip and tline clip sync functions can't pass the same data througn clapperless so 
                          # we use this global to save data as needed for tline sync function.
                          # The data flow is a bit haed to follow here, this needs tobe refactored.
 
+_compare_dialog_thread = None
+
 class ClapperlesLaunchThread(threading.Thread):
     def __init__(self, video_file, audio_file, completed_callback):
         threading.Thread.__init__(self)
@@ -73,11 +75,11 @@ class ClapperlesLaunchThread(threading.Thread):
 
 # ------------------------------------------------------- module funcs
 def _write_offsets(video_file_path, audio_file_path, completed_callback):
-    print "Starting clapperless analysis..."
+    print("Starting clapperless analysis...")
     fps = str(int(utils.fps() + 0.5))
     idstr = _get_offset_file_idstr(video_file_path, audio_file_path)
 
-    FLOG = open(utils.get_hidden_user_dir_path() + "log_clapperless", 'w')
+    FLOG = open(userfolders.get_cache_dir() + "log_clapperless", 'w')
     
     # clapperless.py computes offsets and writes them to file clapperless.OFFSETS_DATA_FILE
     p = subprocess.Popen([sys.executable, respaths.LAUNCH_DIR + "flowbladeclapperless", video_file_path, audio_file_path, "--rate", fps, "--idstr", idstr], stdin=FLOG, stdout=FLOG, stderr=FLOG)
@@ -87,12 +89,10 @@ def _write_offsets(video_file_path, audio_file_path, completed_callback):
     GLib.idle_add(completed_callback, (video_file_path, audio_file_path, idstr))
 
 def _get_offset_file_idstr(file_1, file_2):
-    # Create unique file path in hidden render folder
-    folder = editorpersistance.prefs.render_folder
-    return md5.new(file_1 + file_2).hexdigest()
+    return hashlib.md5((file_1 + file_2).encode('utf-8')).hexdigest()
     
 def _read_offsets(idstr):
-    offsets_file = utils.get_hidden_user_dir_path() + clapperless.OFFSETS_DATA_FILE + "_"+ idstr
+    offsets_file = userfolders.get_cache_dir() + clapperless.OFFSETS_DATA_FILE + "_"+ idstr
     with open(offsets_file) as f:
         file_lines = f.readlines()
     file_lines = [x.rstrip("\n") for x in file_lines]
@@ -116,11 +116,11 @@ def init_select_tline_sync_clip(popup_data):
 
     if not (track.clips[clip_index] == clip):
         # This should never happen 
-        print "big fu at init_select_tline_sync_clip(...)"
+        print("big fu at init_select_tline_sync_clip(...)")
         return
 
-    gdk_window = gui.tline_display.get_parent_window();
-    gdk_window.set_cursor(Gdk.Cursor.new(Gdk.CursorType.TCROSS))
+    gdk_window = gui.tline_display.get_parent_window()
+    gdk_window.set_cursor(Gdk.Cursor.new_for_display(Gdk.Display.get_default(), Gdk.CursorType.TCROSS))
     editorstate.edit_mode = editorstate.SELECT_TLINE_SYNC_CLIP
 
     global _tline_sync_data
@@ -138,8 +138,8 @@ def select_sync_clip_mouse_pressed(event, frame):
     
     if utils.is_mlt_xml_file(sync_clip.path) == True:
         # This isn't translated because 1.14 translation window is close, translation coming for 1.16
-        dialogutils.warning_message(_("Cannot Timeline Audio Sync with Compound Clips!"), 
-                                    _("Audio syncing for Compound Clips is not supported."),
+        dialogutils.warning_message(_("Cannot Timeline Audio Sync with MLT XML Container Clips!"), 
+                                    _("Audio syncing for MLT XML Container Clips is not supported."),
                                     gui.editor_window.window,
                                     True)
         return
@@ -157,8 +157,11 @@ def select_sync_clip_mouse_pressed(event, frame):
     
     _tline_sync_data.clip_tline_media_offset = (sync_clip_start_in_tline - sync_clip.clip_in) - (_tline_sync_data.origin_clip_start_in_tline - _tline_sync_data.origin_clip.clip_in)
     
-    gdk_window = gui.tline_display.get_parent_window();
-    gdk_window.set_cursor(Gdk.Cursor.new(Gdk.CursorType.LEFT_PTR))
+    gdk_window = gui.tline_display.get_parent_window()
+    gdk_window.set_cursor(Gdk.Cursor.new_for_display(Gdk.Display.get_default(), Gdk.CursorType.LEFT_PTR))
+    
+    global _compare_dialog_thread
+    _compare_dialog_thread = AudioCompareActiveThread()
     
     # This or GUI freezes, we really can't do Popen.wait() in a Gtk thread
     clapperless_thread = ClapperlesLaunchThread(_tline_sync_data.origin_clip.path, sync_clip.path, _tline_sync_offsets_computed_callback)
@@ -168,7 +171,7 @@ def select_sync_clip_mouse_pressed(event, frame):
     movemodes.clear_selected_clips()
 
     updater.repaint_tline()
-
+    
 def _get_sync_tline_clip(event, frame):
     sync_track = tlinewidgets.get_track(event.y)
 
@@ -176,7 +179,7 @@ def _get_sync_tline_clip(event, frame):
         return None
         
     if sync_track == _tline_sync_data.origin_track:
-        dialogutils.warning_message(_("Audio Sync parent clips must be on differnt tracks "), 
+        dialogutils.warning_message(_("Audio Sync parent clips must be on different tracks "), 
                                 _("Selected audio sync clip is on the sametrack as the sync action origin clip."),
                                 gui.editor_window.window,
                                 True)
@@ -189,7 +192,10 @@ def _get_sync_tline_clip(event, frame):
     return sync_track.clips[sync_clip_index]
     
 def _tline_sync_offsets_computed_callback(clapperless_data):
-    print "Clapperless done for tline sync"
+    print("Clapperless done for tline sync")
+    
+    global _compare_dialog_thread
+    _compare_dialog_thread.compare_done()
     
     file_path_1, file_path_2, idstr = clapperless_data
     files_offsets = _read_offsets(idstr)
@@ -257,19 +263,19 @@ def create_audio_sync_compound_clip():
     video_file = selection[0].media_file
     audio_file = selection[1].media_file
     
-    # Can't sync coumpound clips
+    # Can't sync compound clips
     if utils.is_mlt_xml_file(video_file.path) == True or utils.is_mlt_xml_file(audio_file.path) == True:
-        # This isn't translated because 1.14 translation window is close, translation coming for 1.16
-        dialogutils.warning_message(_("Cannot Create Audio Sync Compound Clip from Compound Clips!"), 
-                                    _("Audio syncing Compound Clips is not supported."),
+
+        dialogutils.warning_message(_("Cannot Create Audio Sync Clip from  MLT XML Container Clips!"), 
+                                    _("Audio syncing MLT XML Container Clips is not supported."),
                                     gui.editor_window.window,
                                     True)
         return
 
     # Can't sync 2 audio clips
     if video_file.type == appconsts.AUDIO and audio_file.type == appconsts.AUDIO:
-        # This isn't translated because 1.14 translation window is close, translation coming for 1.16
-        dialogutils.warning_message(_("Cannot Create Audio Sync Compound Clip from 2 Audio Clips!"), 
+
+        dialogutils.warning_message(_("Cannot Create Audio Sync Container Clip from 2 Audio Clips!"), 
                                     _("One of the media items needs to be a video clip."),
                                     gui.editor_window.window,
                                     True)
@@ -280,15 +286,21 @@ def create_audio_sync_compound_clip():
     elif video_file.type == appconsts.AUDIO and audio_file.type == appconsts.VIDEO:
         video_file, audio_file = audio_file, video_file
     else:
-        print  "2 video files, video audio assignments determined by selection order"
+        print("2 video files, video audio assignments determined by selection order")
 
+    global _compare_dialog_thread
+    _compare_dialog_thread = AudioCompareActiveThread()
+    
     # This or GUI freezes, we really can't do Popen.wait() in a Gtk thread
     clapperless_thread = ClapperlesLaunchThread(video_file.path, audio_file.path, _compound_offsets_complete)
     clapperless_thread.start()
     
 def _compound_offsets_complete(data):
-    print "Clapperless done for compound clip"
+    print("Clapperless done for compound clip")
 
+    global _compare_dialog_thread
+    _compare_dialog_thread.compare_done()
+    
     video_file_path, audio_file_path, idstr = data
     files_offsets = _read_offsets(idstr)
     sync_data = (files_offsets, data)
@@ -310,15 +322,15 @@ def _do_create_sync_compound_clip(dialog, response_id, data):
     dialog.destroy()
     
     # Create unique file path in hidden render folder
-    folder = editorpersistance.prefs.render_folder
-    uuid_str = md5.new(str(os.urandom(32))).hexdigest()
-    write_file = folder + "/"+ uuid_str + ".xml"
+    folder = userfolders.get_render_dir()
+    uuid_str = hashlib.md5(str(os.urandom(32)).encode('utf-8')).hexdigest()
+    write_file = folder + uuid_str + ".xml"
     
     # Create tractor
     tractor = mlt.Tractor()
     multitrack = tractor.multitrack()
-    track_video = mlt.Playlist()
-    track_audio = mlt.Playlist()
+    track_video = mlt.Playlist(PROJECT().profile)
+    track_audio = mlt.Playlist(PROJECT().profile)
     track_audio.set("hide", 1) # video off, audio on as mlt "hide" value
     multitrack.connect(track_audio, 0)
     multitrack.connect(track_video, 0)
@@ -328,8 +340,7 @@ def _do_create_sync_compound_clip(dialog, response_id, data):
     audio_clip = mlt.Producer(PROJECT().profile, str(audio_file))
     
     # Get offset
-    offset = files_offsets[audio_file]
-    print audio_file, offset
+    offset = float(files_offsets[audio_file])
     
     # Add clips
     if offset > 0:
@@ -347,5 +358,38 @@ def _do_create_sync_compound_clip(dialog, response_id, data):
         track_audio.append(audio_clip, 0, audio_clip.get_length() - 1)
 
     # render MLT XML, callback in projectaction.py creates media object
-    render_player = renderconsumer.XMLCompoundRenderPlayer(write_file, media_name, projectaction._xml_compound_render_done_callback, tractor)
+    render_player = renderconsumer.XMLCompoundRenderPlayer(write_file, media_name, projectaction._xml_compound_render_done_callback, tractor, PROJECT())
     render_player.start()
+
+
+# This is not a thread anymore but successive calls to Gdk.threads_add_timeout() 
+# to achieve the same functionality.
+class AudioCompareActiveThread:
+
+    def __init__(self):
+        self.running = True
+        self.dialog = None
+        
+        Gdk.threads_add_timeout(GLib.PRIORITY_HIGH_IDLE, 10, self.start_dialog)
+                
+    def start_dialog(self):
+        self.dialog = dialogs.audio_sync_active_dialog()
+        self.dialog.progress_bar.set_pulse_step(0.2)
+        time.sleep(0.1)
+
+        Gdk.threads_add_timeout(GLib.PRIORITY_HIGH_IDLE, 200, self.run_dialog)
+        
+        return False
+    
+    def run_dialog(self):
+        if self.running:
+            self.dialog.progress_bar.pulse()
+            return True
+        else:
+            PROJECT().update_media_lengths_on_load = False
+            self.dialog.destroy()
+            return False
+
+    def compare_done(self):
+        self.running = False
+
