@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2012 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,6 +22,9 @@
 Handles Box tool functionality.
 """
 
+import appconsts
+import dialogutils
+import gui
 import edit
 import editorstate
 from editorstate import current_sequence
@@ -30,6 +33,10 @@ import updater
 
 box_selection_data = None
 edit_data = None
+entered_from_overwrite = False
+
+# Set at app.py to be movemodes.select_from_box_selection
+set_move_selection_from_box_selection_func = None
 
 def clear_data():
     # These need to cleared when box tool is activated
@@ -38,16 +45,20 @@ def clear_data():
     edit_data = None
      
 def mouse_press(event, frame):
+    mouse_press_with_coords(event.x, event.y, frame)
+
+def mouse_press_with_coords(ex, ey, frame):
     global edit_data, box_selection_data
+
     if box_selection_data == None: # mouse action is to select
-        press_point = (event.x, event.y)
+        press_point = (ex, ey)
         
         edit_data = {"action_on":True,
                      "press_point":press_point,
                      "mouse_point":press_point,
                      "box_selection_data":None}
     else: # mouse action is to move
-        if box_selection_data.is_hit(event.x, event.y) == False:
+        if box_selection_data.is_hit(ex, ey) == False:
             # Back to start state if selection box missed
             edit_data = None
             box_selection_data = None
@@ -64,35 +75,76 @@ def mouse_move(x, y, frame):
     global edit_data
     if edit_data == None:
         return
+
     if box_selection_data == None: # mouse action is to select
         edit_data["mouse_point"] = (x, y)
-       
     else: # mouse action is to move
-        delta = frame - edit_data["press_frame"]
+        delta = edit_data["snapped_frame"] - edit_data["press_frame"]
         edit_data["delta"] = delta
 
     tlinewidgets.set_edit_mode_data(edit_data)
     updater.repaint_tline()
     
 def mouse_release(x, y, frame):
-    global box_selection_data, edit_data
+    global box_selection_data, edit_data, entered_from_overwrite
     if edit_data == None:
+        if entered_from_overwrite == True:
+            _exit_to_overwrite()
         return
         
     if box_selection_data == None: # mouse action is to select
         box_selection_data = BoxMoveData(edit_data["press_point"], (x, y))
+        
+        locked_track = box_selection_data.get_possible_locked_track()
+        if locked_track != None:
+            dialogutils.track_lock_check_and_user_info(locked_track)
+            edit_data = None
+            box_selection_data = None
+            tlinewidgets.set_edit_mode_data(edit_data)
+            updater.repaint_tline()
+            # Exit box mode if entered from overwrite
+            if entered_from_overwrite == True:
+                _exit_to_overwrite()
+            return 
+              
         if box_selection_data.is_empty() == False:
-            edit_data = {"action_on":True,
-                         "press_frame":frame,
-                         "delta":0,
-                         "box_selection_data":box_selection_data}
+            if len(box_selection_data.track_selections) == 1 and entered_from_overwrite == True:
+                # If selection only contains a single track we treat it as a movemode selection range
+                # because that provides the added functionality of moving clips to another track.
+                _exit_to_overwrite()
+                set_move_selection_from_box_selection_func(box_selection_data.track_selections[0])
+                return
+            else:
+                edit_data = {"action_on":True,
+                             "press_frame":frame,
+                             "delta":0,
+                             "box_selection_data":box_selection_data}
         else:
             box_selection_data = None
             edit_data = {"action_on":False,
                          "press_frame":-1,
                          "delta":0,
                          "box_selection_data":box_selection_data}
+            # Exit box mode if entered from overwrite  with empty selection
+            if entered_from_overwrite == True:
+                _exit_to_overwrite()
+                return 
+    
     else: # mouse action is to move
+        # Exit if selection contains locked track
+        locked_track = box_selection_data.get_possible_locked_track()
+        if locked_track != None:
+            dialogutils.track_lock_check_and_user_info(locked_track)
+            edit_data = None
+            box_selection_data = None
+            tlinewidgets.set_edit_mode_data(edit_data)
+            updater.repaint_tline()
+            # Exit box mode if entered from overwrite
+            if entered_from_overwrite == True:
+                _exit_to_overwrite()
+            return 
+            
+        # If we lock track after 
         delta = frame - edit_data["press_frame"]
         edit_data["delta"] = delta
 
@@ -105,9 +157,24 @@ def mouse_release(x, y, frame):
         # Back to start state
         edit_data = None
         box_selection_data = None
+        
+        # Exit box mode if entered from overwrite with empty selection
+        if entered_from_overwrite == True:
+            _exit_to_overwrite()
+            return 
     
     tlinewidgets.set_edit_mode_data(edit_data)
     updater.repaint_tline()
+                
+def _exit_to_overwrite():
+    # If we entered box mode from overwrite mode empty click, this is used to enter back into overwrite mode.
+    global entered_from_overwrite
+    entered_from_overwrite = False
+    editorstate.overwrite_mode_box = False
+    tlinewidgets.set_edit_mode_data(None)
+    gui.editor_window.tline_cursor_manager.set_cursor_to_mode() # This gets set wrong in editevent.tline_canvas_mouse_released() and were putting it back here, 
+                                           # this could be investigated for better solution, this could cause a cursor flash, but on dev system we're not getting it.
+    updater.repaint_tline()
 
 
 class BoxMoveData:
@@ -135,7 +202,10 @@ class BoxMoveData:
         
         start_frame = tlinewidgets.get_frame(x1)
         end_frame = tlinewidgets.get_frame(x2) 
-        
+
+        if start_frame < 0:
+            start_frame = 0
+
         track_top_index = self.get_bounding_track_index(y1, tlinewidgets.get_track(y1))
         track_bottom_index = self.get_bounding_track_index(y2, tlinewidgets.get_track(y2))
 
@@ -211,6 +281,13 @@ class BoxMoveData:
         return False
 
 
+    def get_possible_locked_track(self):
+        for selection in self.track_selections:
+            if current_sequence().tracks[selection.track_id].edit_freedom == appconsts.LOCKED:
+                return current_sequence().tracks[selection.track_id]
+        
+        return None
+
 class BoxTrackSelection:
     """
     This class collects data on track's Box selected clips.
