@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2012 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@ Module handles clip effects editing logic and gui
 import cairo
 import copy
 from gi.repository import GLib
-from gi.repository import Gtk, Gdk
+from gi.repository import Gtk
 import pickle
 import threading
 import time
@@ -33,20 +33,20 @@ import appconsts
 import atomicfile
 import dialogs
 import dialogutils
-import dnd
 import edit
 import editorlayout
 import editorpersistance
 import editorstate
 from editorstate import PROJECT
+from editorstate import PLAYER
 import gui
 import guicomponents
+import guipopover
 import guiutils
 import mltfilters
 import propertyedit
 import propertyeditorbuilder
 import respaths
-import tlinerender
 import translations
 import updater
 import utils
@@ -63,8 +63,8 @@ _block_stack_update = False # Used to block full stack update when adding new fi
 # Property change polling.
 # We didn't put a layer of indirection to look for and launch events on filter property edits
 # so now we detect filter edits by polling. This has no performance impect, n is so small.
-_edit_polling_thread = None
-filter_changed_since_last_save = False
+#_edit_polling_thread = None
+# filter_changed_since_last_save = False
 
 # This is updated when filter panel is displayed and cleared when removed.
 # Used to update kfeditors with external tline frame position changes
@@ -91,36 +91,61 @@ class FilterFooterRow:
         self.filter_object = filter_object
         self.filter_stack = filter_stack
         
-        surface = cairo.ImageSurface.create_from_png(respaths.IMAGE_PATH + "filter_save.png")
-        save_button = guicomponents.PressLaunch(self.save_pressed, surface, w=22, h=22)
+        surface = guiutils.get_cairo_image("filter_save")
+        if editorpersistance.prefs.double_track_hights  == False:
+            save_button = guicomponents.PressLaunch(self.save_pressed, surface, w=22, h=22)
+        else:
+            save_button = guicomponents.PressLaunch(self.save_pressed, surface, w=44, h=44)
         save_button.widget.set_tooltip_markup(_("Save effect values"))
         
-        surface = cairo.ImageSurface.create_from_png(respaths.IMAGE_PATH + "filter_load.png")
-        load_button = guicomponents.PressLaunch(self.load_pressed, surface, w=22, h=22)
+        surface = guiutils.get_cairo_image("filter_load")
+        if editorpersistance.prefs.double_track_hights  == False:
+            load_button = guicomponents.PressLaunch(self.load_pressed, surface, w=22, h=22)
+        else:
+            load_button = guicomponents.PressLaunch(self.load_pressed, surface, w=44, h=44)
         load_button.widget.set_tooltip_markup(_("Load effect values"))
 
-        surface = cairo.ImageSurface.create_from_png(respaths.IMAGE_PATH + "filter_reset.png")
-        reset_button = guicomponents.PressLaunch(self.reset_pressed, surface, w=22, h=22)
+        surface = guiutils.get_cairo_image("filter_reset")
+        if editorpersistance.prefs.double_track_hights  == False:
+            reset_button = guicomponents.PressLaunch(self.reset_pressed, surface, w=22, h=22)
+        else:
+            reset_button = guicomponents.PressLaunch(self.reset_pressed, surface, w=44, h=44)
         reset_button.widget.set_tooltip_markup(_("Reset effect values"))
         
         surface = guiutils.get_cairo_image("filters_mask_add")
-        mask_button = guicomponents.PressLaunch(self.add_mask_pressed, surface, w=22, h=22)
+        if editorpersistance.prefs.double_track_hights  == False:
+            mask_button = guicomponents.PressLaunch(self.add_mask_pressed, surface, w=22, h=22)
+        else:
+            mask_button = guicomponents.PressLaunch(self.add_mask_pressed, surface, w=44, h=44)
         mask_button.widget.set_tooltip_markup(_("Add Filter Mask"))
-
+        self.mask_button = mask_button
+        
         surface = guiutils.get_cairo_image("filters_move_up")
-        move_up_button = guicomponents.PressLaunch(self.move_up_pressed, surface, w=22, h=22)
+        if editorpersistance.prefs.double_track_hights  == False:
+            move_up_button = guicomponents.PressLaunch(self.move_up_pressed, surface, w=22, h=22)
+        else:
+            move_up_button = guicomponents.PressLaunch(self.move_up_pressed, surface, w=44, h=44)
         move_up_button.widget.set_tooltip_markup(_("Move Filter Up"))
 
         surface = guiutils.get_cairo_image("filters_move_down")
-        move_down_button = guicomponents.PressLaunch(self.move_down_pressed, surface, w=22, h=22)
+        if editorpersistance.prefs.double_track_hights  == False:
+            move_down_button = guicomponents.PressLaunch(self.move_down_pressed, surface, w=22, h=22)
+        else:
+            move_down_button = guicomponents.PressLaunch(self.move_down_pressed, surface, w=44, h=44)
         move_down_button.widget.set_tooltip_markup(_("Move Filter Down"))
 
         surface = guiutils.get_cairo_image("filters_move_top")
-        move_top_button = guicomponents.PressLaunch(self.move_top_pressed, surface, w=22, h=22)
+        if editorpersistance.prefs.double_track_hights  == False:
+            move_top_button = guicomponents.PressLaunch(self.move_top_pressed, surface, w=22, h=22)
+        else:
+            move_top_button = guicomponents.PressLaunch(self.move_top_pressed, surface, w=44, h=44)
         move_top_button.widget.set_tooltip_markup(_("Move Filter To Top"))
 
         surface = guiutils.get_cairo_image("filters_move_bottom")
-        move_bottom_button = guicomponents.PressLaunch(self.move_bottom_pressed, surface, w=22, h=22)
+        if editorpersistance.prefs.double_track_hights  == False:
+            move_bottom_button = guicomponents.PressLaunch(self.move_bottom_pressed, surface, w=22, h=22)
+        else:
+            move_bottom_button = guicomponents.PressLaunch(self.move_bottom_pressed, surface, w=44, h=44)
         move_bottom_button.widget.set_tooltip_markup(_("Move Filter To Bottom"))
         
         self.widget = Gtk.HBox(False, 0)
@@ -138,20 +163,21 @@ class FilterFooterRow:
         self.widget.pack_start(load_button.widget, False, False, 0)
 
         self.widget.pack_start(Gtk.Label(), True, True, 0)
-        
+        self.widget.set_name("lighter-bg-widget")
+            
     def save_pressed(self, w, e):
         default_name = self.filter_object.info.name + _("_effect_values") + ".data"
         dialogs.save_effects_compositors_values(_save_effect_values_dialog_callback, default_name, True, self.filter_object)
 
     def load_pressed(self, w, e):
         dialogs.load_effects_compositors_values_dialog(_load_effect_values_dialog_callback, True, self.filter_object)
-        
+
     def reset_pressed(self, w, e):
         _reset_filter_values(self.filter_object)
 
     def add_mask_pressed(self, w, e):
         filter_index = self.filter_stack.get_filter_index(self.filter_object)
-        _filter_mask_launch_pressed(w, e, filter_index)
+        _filter_mask_launch_pressed(self.mask_button, w, filter_index)
 
     def move_up_pressed(self, w, e):
         from_index = self.filter_stack.get_filter_index(self.filter_object)
@@ -203,7 +229,6 @@ class FilterHeaderRow:
         hbox.pack_start(self.icon, False, False, 0)
         hbox.pack_start(self.filter_name_label, False, False, 0)
         hbox.pack_start(Gtk.Label(), True, True, 0)
-
         self.widget = hbox
 
 
@@ -214,9 +239,8 @@ class FilterStackItem:
         self.filter_header_row = FilterHeaderRow(filter_object)
 
         self.edit_panel = edit_panel
-        self.edit_panel_frame = Gtk.Frame()
+        self.edit_panel_frame = Gtk.Box.new(Gtk.Orientation.VERTICAL, 0)
         self.edit_panel_frame.add(edit_panel)
-        self.edit_panel_frame.set_shadow_type(Gtk.ShadowType.NONE)
         
         self.filter_stack = filter_stack
         self.expander = Gtk.Expander()
@@ -224,9 +248,8 @@ class FilterStackItem:
         self.expander.add(self.edit_panel_frame)
         self.expander.set_label_fill(True)
 
-        self.expander_frame = Gtk.Frame()
+        self.expander_frame = Gtk.Box.new(Gtk.Orientation.VERTICAL, 0)
         self.expander_frame.add(self.expander)
-        self.expander_frame.set_shadow_type(Gtk.ShadowType.NONE)
         guiutils.set_margins(self.expander_frame, 2, 0, 0, 0)
         
         self.active_check = Gtk.CheckButton()
@@ -303,7 +326,7 @@ class ClipFilterStack:
             for child in children:
                 self.widget.remove(child)
                 
-            # Remove old stack item for reseted filter.
+            # Remove old stack item for reset filter.
             self.filter_stack.pop(stack_index)
             self.clear_kf_editors_from_update_list(filter_object)
 
@@ -384,6 +407,7 @@ def get_clip_effects_editor_info_row():
 
     info_row = Gtk.HBox(False, 2)
     info_row.pack_start(widgets.hamburger_launcher.widget, False, False, 0)
+    info_row.pack_start(widgets.filter_add_launch.widget, True, True, 0)
     info_row.pack_start(Gtk.Label(), True, True, 0)
     info_row.pack_start(widgets.clip_info, False, False, 0)
     info_row.pack_start(Gtk.Label(), True, True, 0)
@@ -398,13 +422,18 @@ def _create_widgets():
     
     widgets.value_edit_box = Gtk.VBox()
     widgets.value_edit_frame = Gtk.Frame()
-    widgets.value_edit_frame.set_shadow_type(Gtk.ShadowType.NONE)
     widgets.value_edit_frame.add(widgets.value_edit_box)
     
     widgets.hamburger_launcher = guicomponents.HamburgerPressLaunch(_hamburger_launch_pressed)
+    widgets.hamburger_launcher.do_popover_callback = True
     guiutils.set_margins(widgets.hamburger_launcher.widget, 6, 8, 1, 0)
 
-
+    surface_active = guiutils.get_cairo_image("filter_add")
+    surface_not_active = guiutils.get_cairo_image("filter_add_not_active")
+    surfaces = [surface_active, surface_not_active]
+    widgets.filter_add_launch = guicomponents.HamburgerPressLaunch(lambda w,e:_filter_add_menu_launch_pressed(w, e), surfaces)
+    guiutils.set_margins(widgets.filter_add_launch.widget, 6, 8, 1, 0)
+    
 # ------------------------------------------------------------------- interface
 def set_clip(clip, track, clip_index, show_tab=True):
     """
@@ -417,9 +446,15 @@ def set_clip(clip, track, clip_index, show_tab=True):
     global keyframe_editor_widgets
     keyframe_editor_widgets = []
 
+    if _filter_stack != None and clip != _filter_stack.clip:
+        clip_start_frame = track.clip_start(clip_index)
+        PLAYER().seek_frame(clip_start_frame)
+    
     widgets.clip_info.display_clip_info(clip, track, clip_index)
     set_enabled(True)
     update_stack(clip, track, clip_index)
+    if len(clip.filters) > 0:
+        set_filter_item_expanded(len(clip.filters) - 1)
 
     if len(clip.filters) > 0:
         pass # remove if nothing needed here.
@@ -427,15 +462,17 @@ def set_clip(clip, track, clip_index, show_tab=True):
         show_text_in_edit_area(_("Clip Has No Filters"))
 
     if show_tab:
-        editorlayout.show_panel(appconsts.PANEL_FILTERS)
+        editorlayout.show_panel(appconsts.PANEL_MULTI_EDIT)
 
-    global _edit_polling_thread
+    gui.editor_window.edit_multi.set_visible_child_name(appconsts.EDIT_MULTI_FILTERS)
+
+    #global _edit_polling_thread
     # Close old polling
-    if _edit_polling_thread != None:
-        _edit_polling_thread.shutdown()
+    #if _edit_polling_thread != None:
+    #    _edit_polling_thread.shutdown()
     # Start new polling
-    _edit_polling_thread = PropertyChangePollingThread()
-    _edit_polling_thread.start()
+    #_edit_polling_thread = PropertyChangePollingThread()
+    #_edit_polling_thread.start()
 
 def refresh_clip():
     if _filter_stack == None:
@@ -448,6 +485,12 @@ def refresh_clip():
 
     _filter_stack.set_expanded(expanded_panels)
 
+def get_clip_editor_clip_data():
+    if _filter_stack == None:
+        return None
+    else:
+        return _filter_stack.get_clip_data()
+
 def clip_is_being_edited(clip):
     if _filter_stack == None:
         return False
@@ -487,6 +530,17 @@ def add_currently_selected_effect():
     group_index = gui.effect_select_combo_box.get_active()
 
     _add_filter_from_effect_select_panel(row_index, group_index)
+
+def get_currently_selected_filter_info():
+    # Currently selected in effect select panel, not here.
+    treeselection = gui.effect_select_list_view.treeview.get_selection()
+    (model, rows) = treeselection.get_selected_rows()    
+    row = rows[0]
+    row_index = max(row)
+    group_index = gui.effect_select_combo_box.get_active()
+    group_name, filters_array = mltfilters.groups[group_index]
+    filter_info = filters_array[row_index]
+    return filter_info
     
 def _add_filter_from_effect_select_panel(row_index, group_index):
     # Add filter
@@ -507,9 +561,6 @@ def _add_filter_from_effect_select_panel(row_index, group_index):
 
     updater.repaint_tline()
 
-def _quit_editing_clip_clicked(): # this is a button callback
-    clear_clip()
-
 def clear_clip():
     """
     Removes clip from effects editing gui.
@@ -520,7 +571,10 @@ def clear_clip():
     show_text_in_edit_area(_("No Clip"))
 
     set_enabled(False)
-    shutdown_polling()
+    # shutdown_polling()
+
+    global keyframe_editor_widgets
+    keyframe_editor_widgets = []
 
 def _set_no_clip_info():
     widgets.clip_info.set_no_clip_info()
@@ -529,6 +583,8 @@ def set_enabled(value):
     widgets.clip_info.set_enabled(value)
     widgets.hamburger_launcher.set_sensitive(value)
     widgets.hamburger_launcher.widget.queue_draw()
+    widgets.filter_add_launch.set_sensitive(value)
+    widgets.filter_add_launch.widget.queue_draw()
 
 def set_stack_update_blocked():
     global _block_stack_update
@@ -544,7 +600,7 @@ def update_stack(clip, track, clip_index):
     _filter_stack = new_stack
 
     scroll_window = Gtk.ScrolledWindow()
-    scroll_window.add_with_viewport(_filter_stack.widget)
+    scroll_window.add(_filter_stack.widget)
     scroll_window.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
     scroll_window.show_all()
 
@@ -553,12 +609,14 @@ def update_stack(clip, track, clip_index):
     widgets.value_edit_frame.add(scroll_window)
 
     widgets.value_edit_box = scroll_window
-    
+
+"""
 def update_stack_changed_blocked():
     global _block_changed_update
     _block_changed_update = True
     update_stack()
     _block_changed_update = False
+"""
 
 def _alpha_filter_add_maybe_info(filter_info):
     if editorpersistance.prefs.show_alpha_info_message == True and \
@@ -630,7 +688,14 @@ def delete_effect_pressed(clip, filter_index):
     set_clip(clip, track, clip_index)
 
     updater.repaint_tline()
-    
+
+def _save_stack_pressed():
+    default_name = _("unnamed_stack_values") + ".data"
+    dialogs.save_effects_compositors_values(_save_effect_stack_values_dialog_callback, default_name, True, None, True)
+
+def _load_stack_pressed():
+    dialogs.load_effects_compositors_values_dialog(_load_effect_stack_values_dialog_callback, True, None, True)
+        
 def _toggle_all_pressed():
     if _filter_stack == None:
         return False
@@ -672,10 +737,6 @@ def _get_filter_panel(clip, filter_object, filter_index, track, clip_index):
 
     # Get editors and set them displayed
     vbox = Gtk.VBox(False, 0)
-    try:
-        filter_name = translations.filter_names[filter_object.info.name]
-    except KeyError:
-        filter_name = filter_object.info.name
 
     filter_keyframe_editor_widgets = []
 
@@ -687,7 +748,7 @@ def _get_filter_panel(clip, filter_object, filter_index, track, clip_index):
             editor_row = propertyeditorbuilder.get_editor_row(ep)
             if editor_row == None:
                 continue
-
+            editor_row.set_name("editor-row-widget")
             # Set keyframe editor widget to be updated for frame changes if such is created 
             try:
                 editor_type = ep.args[propertyeditorbuilder.EDITOR]
@@ -751,7 +812,7 @@ def show_text_in_edit_area(text):
     vbox.show_all()
 
     scroll_window = Gtk.ScrolledWindow()
-    scroll_window.add_with_viewport(vbox)
+    scroll_window.add(vbox)
     scroll_window.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
     scroll_window.show_all()
 
@@ -761,10 +822,7 @@ def show_text_in_edit_area(text):
     widgets.value_edit_box = scroll_window
 
 def clear_effects_edit_panel():
-    widgets.value_edit_frame.remove(widgets.value_edit_box)
-    label = Gtk.Label()
-    widgets.value_edit_frame.add(label)
-    widgets.value_edit_box = label
+    show_text_in_edit_area(_("Clip Has No Filters"))
 
 def filter_edit_done_stack_update(edited_clip, index=-1):
     """
@@ -777,16 +835,21 @@ def filter_edit_done_stack_update(edited_clip, index=-1):
     if edited_clip != get_edited_clip(): # This gets called by all undos/redos, we only want to update if clip being edited here is affected
         return
 
+    track = _filter_stack.track
+    clip_index = _filter_stack.clip_index
+        
     global _block_changed_update
     _block_changed_update = True
-    update_stack()
+    update_stack(edited_clip, track, clip_index)
     _block_changed_update = False
 
-    # Select row in effect stack view and to display corresponding effect editor panel.
-    if not(index < 0):
-        widgets.effect_stack_view.treeview.get_selection().select_path(str(index))
-    else: # no effects after edit, clear effect editor panel
+    if len(_filter_stack.clip.filters) == 0:
         clear_effects_edit_panel()
+    
+def filter_edit_multi_done_stack_update(clips):
+    for clip in clips:
+        if clip == get_edited_clip():
+             clear_clip()
 
 def display_kfeditors_tline_frame(frame):
     for kf_widget in keyframe_editor_widgets:
@@ -805,11 +868,11 @@ def update_kfeditors_positions():
 
 
 # ------------------------------------------------ FILTER MASK 
-def _filter_mask_launch_pressed(widget, event, filter_index):
+def _filter_mask_launch_pressed(launcher, widget, filter_index):
     filter_names, filter_msgs = mltfilters.get_filter_mask_start_filters_data()
-    guicomponents.get_filter_mask_menu(event, _filter_mask_item_activated, filter_names, filter_msgs, filter_index)
+    guipopover.filter_mask_popover_show(launcher, widget, _filter_mask_item_activated, filter_names, filter_msgs, filter_index)
 
-def _filter_mask_item_activated(widget, data):
+def _filter_mask_item_activated(action, variant, data):
     if _filter_stack == None:
         return False
     
@@ -845,21 +908,14 @@ def _filter_mask_item_activated(widget, data):
     set_clip(clip, track, clip_index)
     _filter_stack.set_filter_item_expanded(current_filter_index + 1)
 
-def _clip_has_filter_mask_filter():
-    if clip == None:
-        return False
-    
-    for f in clip.filters:
-        if f.info.filter_mask_filter != "":
-            return True
-          
-    return False
 
 # ------------------------------------------------ SAVE, LOAD etc. from hamburger menu
-def _hamburger_launch_pressed(widget, event):
-    guicomponents.get_clip_effects_editor_hamburger_menu(event, _clip_hamburger_item_activated)
+def _hamburger_launch_pressed(launcher, widget, event, data):
+    guipopover.effects_editor_hamburger_popover_show(launcher, widget, _clip_hamburger_item_activated)
+    
+    #guicomponents.get_clip_effects_editor_hamburger_menu(event, _clip_hamburger_item_activated)
 
-def _clip_hamburger_item_activated(widget, msg):
+def _clip_hamburger_item_activated(action, event, msg):
     if msg == "fade_length":
         dialogs.set_fade_length_default_dialog(_set_fade_length_dialog_callback, PROJECT().get_project_property(appconsts.P_PROP_DEFAULT_FADE_LENGTH))
 
@@ -868,13 +924,41 @@ def _clip_hamburger_item_activated(widget, msg):
     
     if msg == "close":
         clear_clip()
+        gui.editor_window.edit_multi.set_visible_child_name(appconsts.EDIT_MULTI_EMPTY)
     elif  msg == "expanded":
         _filter_stack.set_all_filters_expanded_state(True)
     elif  msg == "unexpanded":
         _filter_stack.set_all_filters_expanded_state(False)
     elif  msg == "toggle":
         _toggle_all_pressed()
+    elif  msg == "save_stack":
+        _save_stack_pressed()
+    elif  msg == "load_stack":
+        _load_stack_pressed()
 
+def _filter_add_menu_launch_pressed(w, event):
+    if _filter_stack != None:
+        clip = _filter_stack.clip
+        track = _filter_stack.track 
+        guipopover.filter_add_popover_show(widgets.filter_add_launch, w, clip, track, event.x, mltfilters.groups, _filter_popover_callback)
+
+def _filter_popover_callback(action, variant, data):
+    _filter_menu_callback(None, data)
+
+def _filter_menu_callback(w, data):
+    clip, track, item_id, item_data = data
+    x, filter_info = item_data
+
+    action = get_filter_add_action(filter_info, clip)
+    set_stack_update_blocked() # We update stack on set_clip below
+    action.do_edit()
+    set_stack_update_unblocked()
+
+    # (re)open clip in editor.
+    index = track.clips.index(clip)
+    set_clip(clip, track, index)
+    set_filter_item_expanded(len(clip.filters) - 1)
+    
 def _save_effect_values_dialog_callback(dialog, response_id, filter_object):
     if response_id == Gtk.ResponseType.ACCEPT:
         save_path = dialog.get_filenames()[0]
@@ -895,12 +979,47 @@ def _load_effect_values_dialog_callback(dialog, response_id, filter_object):
             # Info window
             saved_effect_name = effect_data.info.name
             current_effect_name = filter_object.info.name
-            primary_txt = _("Saved Filter data not applicaple for this Filter!")
+            primary_txt = _("Saved Filter data not applicable for this Filter!")
             secondary_txt = _("Saved data is for ") + saved_effect_name + " Filter,\n" + _("current edited Filter is ") + current_effect_name + "."
             dialogutils.warning_message(primary_txt, secondary_txt, gui.editor_window.window)
     
     dialog.destroy()
 
+def _save_effect_stack_values_dialog_callback(dialog, response_id):
+    if response_id == Gtk.ResponseType.ACCEPT:
+        save_path = dialog.get_filenames()[0]
+        stack_data = EffectStackSaveData()
+        stack_data.save(save_path)
+    dialog.destroy()
+
+def _load_effect_stack_values_dialog_callback(dialog, response_id):
+    if response_id == Gtk.ResponseType.ACCEPT:
+        load_path = dialog.get_filenames()[0]
+        stack_data = utils.unpickle(load_path)
+
+        for effect_data in stack_data.effects_data:
+            filter_info, properties, non_mlt_properties = effect_data
+      
+            data = {"clip":_filter_stack.clip, 
+                    "filter_info":filter_info,
+                    "filter_edit_done_func":filter_edit_done_stack_update}
+            action = edit.add_filter_action(data)
+
+            set_stack_update_blocked()
+            action.do_edit()
+            set_stack_update_unblocked()
+    
+            filters = _filter_stack.get_filters()
+            filter_object = filters[len(filters) - 1]
+    
+            filter_object.properties = copy.deepcopy(properties)
+            filter_object.non_mlt_properties = copy.deepcopy(non_mlt_properties)
+            filter_object.update_mlt_filter_properties_all()
+
+            _filter_stack.reinit_stack_item(filter_object)
+                    
+    dialog.destroy()
+
 def _reset_filter_values(filter_object):
         filter_object.properties = copy.deepcopy(filter_object.info.properties)
         filter_object.non_mlt_properties = copy.deepcopy(filter_object.info.non_mlt_properties)
@@ -920,7 +1039,7 @@ class EffectValuesSaveData:
     
     def __init__(self, filter_object):
         self.info = filter_object.info
-        self.multipart_filter = self.info.multipart_filter
+        self.multipart_filter = self.info.multipart_filter # DEPRECATED
 
         # Values of these are edited by the user.
         self.properties = copy.deepcopy(filter_object.properties)
@@ -929,7 +1048,7 @@ class EffectValuesSaveData:
         except:
             self.non_mlt_properties = [] # Versions prior 0.14 do not have non_mlt_properties and fail here on load
 
-        if self.multipart_filter == True:
+        if self.multipart_filter == True: # DEPRECATED
             self.value = filter_object.value
         else:
             self.value = None
@@ -945,66 +1064,26 @@ class EffectValuesSaveData:
         return False
 
     def set_effect_values(self, filter_object):
-        if self.multipart_filter == True:
+        if self.multipart_filter == True: # DEPRECATED
             filter_object.value = self.value
          
         filter_object.properties = copy.deepcopy(self.properties)
         filter_object.non_mlt_properties = copy.deepcopy(self.non_mlt_properties)
         filter_object.update_mlt_filter_properties_all()
 
-
-# ------------------------------------------------------- CHANGE POLLING
-def shutdown_polling():
-    global _edit_polling_thread
-    if _edit_polling_thread != None:
-        _edit_polling_thread.shutdown()
-        _edit_polling_thread = None
-
-
-class PropertyChangePollingThread(threading.Thread):
-    
+class EffectStackSaveData:
     def __init__(self):
-        threading.Thread.__init__(self)
-        self.last_properties = None
-        
-    def run(self):
-
-        self.running = True
-        while self.running:
-            
-            if _filter_stack == None:
-                self.shutdown()
-            else:
-                if self.last_properties == None:
-                    self.last_properties = self.get_clip_filters_properties()
-                
-                new_properties = self.get_clip_filters_properties()
-                
-                changed = False
-                for new_filt_props, old_filt_props in zip(new_properties, self.last_properties):
-                        for new_prop, old_prop in zip(new_filt_props, old_filt_props):
-                            if new_prop != old_prop:
-                                changed = True
-
-                if changed:
-                    global filter_changed_since_last_save
-                    filter_changed_since_last_save = True
-                    tlinerender.get_renderer().timeline_changed()
-
-                self.last_properties = new_properties
-                
-                time.sleep(1.0)
-
-    def get_clip_filters_properties(self):
-        filters_properties = []
-        for filt in _filter_stack.get_filters():
-            filt_props = []
-            for prop in filt.properties:
-                filt_props.append(copy.deepcopy(prop))
-
-            filters_properties.append(filt_props)
-        
-        return filters_properties
-        
-    def shutdown(self):
-        self.running = False
+        self.effects_data = []
+        self.empty = True
+        filters = _filter_stack.get_filters()
+        if len(filters) > 0:
+            self.empty = False
+            for f in filters:
+                self.effects_data.append((f.info,
+                                          copy.deepcopy(f.properties),
+                                          copy.deepcopy(f.non_mlt_properties)))
+                                      
+    def save(self, save_path):
+        with atomicfile.AtomicFileWriter(save_path, "wb") as afw:
+            write_file = afw.get_file()
+            pickle.dump(self, write_file)
