@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2014 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -27,13 +27,15 @@ from PIL import Image
 from gi.repository import GLib
 from gi.repository import Gtk
 
-import mlt
+try:
+    import mlt7 as mlt
+except:
+    import mlt
 from operator import itemgetter
 import os
 import shutil
 import time
 
-import audiowaveform
 import audiosync
 import appconsts
 import clipeffectseditor
@@ -44,15 +46,16 @@ import dialogutils
 import gui
 import guicomponents
 import edit
-import editevent
 from editorstate import current_sequence
 from editorstate import get_track
 from editorstate import PROJECT
 from editorstate import PLAYER
 import kftoolmode
+import mediaplugin
 import mlttransitions
 import modesetting
 import movemodes
+import projectaction
 import syncsplitevent
 import tlinewidgets
 import tlineaction
@@ -60,7 +63,6 @@ import updater
 import userfolders
 import utils
 
-_match_frame_writer = None
 
 # ---------------------------------- clip menu
 def display_clip_menu(y, event, frame):
@@ -76,16 +78,23 @@ def display_clip_menu(y, event, frame):
         return False
     
     # Display popup
+    not_multi_selection = (movemodes.selected_range_in == -1 or (movemodes.selected_range_out - movemodes.selected_range_in) == 0)
     gui.tline_canvas.drag_on = False
     pressed_clip = track.clips[clip_index]
     if pressed_clip.is_blanck_clip == False:
-        movemodes.select_clip(track.id, clip_index)
+        if not_multi_selection == True:
+            movemodes.select_clip(track.id, clip_index)
     else:
         movemodes.select_blank_range(track, pressed_clip)
 
     if track.type == appconsts.VIDEO:
-        guicomponents.display_clip_popup_menu(event, pressed_clip, \
+        if not_multi_selection == True:
+            guicomponents.display_clip_popup_menu(event, pressed_clip, \
                                               track, _clip_menu_item_activated)
+        else:
+            guicomponents.display_multi_clip_popup_menu(event, pressed_clip, \
+                                                track, _clip_menu_item_activated)
+                                              
     elif track.type == appconsts.AUDIO:
         guicomponents.display_audio_clip_popup_menu(event, pressed_clip, \
                                                     track, _clip_menu_item_activated)
@@ -120,7 +129,19 @@ def _delete_compositors(data):
         data = {"compositor":compositor}
         action = edit.delete_compositor_action(data)
         action.do_edit()
-    
+        
+def _multi_delete_compositors(data):
+    clip, track, item_id, x = data
+    for clip_index in range(movemodes.selected_range_in, movemodes.selected_range_out + 1):
+        composited_clip = track.clips[clip_index]
+        if composited_clip.is_blanck_clip == True:
+            continue
+        compositors = current_sequence().get_clip_compositors(composited_clip)
+        for compositor in compositors:
+            data = {"compositor":compositor}
+            action = edit.delete_compositor_action(data)
+            action.do_edit()
+
 def _open_clip_in_effects_editor(data):
     updater.open_clip_in_effects_editor(data)
     
@@ -128,6 +149,9 @@ def _open_clip_in_clip_monitor(data):
     clip, track, item_id, x = data
     
     media_file = PROJECT().get_media_file_for_path(clip.path)
+    if media_file == None and clip.container_data != None:
+        media_file = PROJECT().get_media_file_for_container_data(clip.container_data)
+
     media_file.mark_in = clip.clip_in
     media_file.mark_out = clip.clip_out
     updater.set_and_display_monitor_media_file(media_file)
@@ -221,25 +245,67 @@ def _add_filter(data):
     clipeffectseditor.set_clip(clip, track, index)
     clipeffectseditor.set_filter_item_expanded(len(clip.filters) - 1)
 
+def _add_filter_multi(data):
+    clip, track, item_id, item_data = data
+    x, filter_info = item_data
+    
+    clipeffectseditor.set_stack_update_blocked() # We update stack on set_clip below
+    
+    range_in = movemodes.selected_range_in
+    range_out = movemodes.selected_range_out
+    clips = track.clips[range_in:range_out +1]
+    data = {"clips":clips, 
+            "filter_info":filter_info,
+            "filter_edit_done_func":clipeffectseditor.filter_edit_multi_done_stack_update}
+    action = edit.add_filter_multi_action(data)
+    action.do_edit()
+    
+    clipeffectseditor.set_stack_update_unblocked()
+
+    # (re)open clip in editor
+    frame = tlinewidgets.get_frame(x)
+    index = track.get_clip_index_at(frame)
+    clipeffectseditor.set_clip(clip, track, index)
+    clipeffectseditor.set_filter_item_expanded(len(clip.filters) - 1)
+
 def _add_compositor(data):
     clip, track, item_id, item_data = data
-    x, compositor_type = item_data
-
-    frame = tlinewidgets.get_frame(x)
-    clip_index = track.get_clip_index_at(frame)
+    x, compositor_type, add_compositors_is_multi_selection = item_data
 
     target_track_index = track.id - 1
 
-    if current_sequence().compositing_mode == appconsts.COMPOSITING_MODE_STANDARD_AUTO_FOLLOW:
-        target_track_index = current_sequence().first_video_index
-
-    compositor_in = current_sequence().tracks[track.id].clip_start(clip_index)
-    clip_length = clip.clip_out - clip.clip_in
-    compositor_out = compositor_in + clip_length
-
     a_track = target_track_index
     b_track = track.id
 
+    if add_compositors_is_multi_selection == True:
+        for clip_index in range(movemodes.selected_range_in, movemodes.selected_range_out + 1):
+            composited_clip = track.clips[clip_index]
+            if composited_clip.is_blanck_clip == True:
+                continue
+            compositor_in = current_sequence().tracks[track.id].clip_start(clip_index)
+            clip_length = composited_clip.clip_out - composited_clip.clip_in
+            compositor_out = compositor_in + clip_length
+            
+            edit_data = {"origin_clip_id":composited_clip.id,
+                        "in_frame":compositor_in,
+                        "out_frame":compositor_out,
+                        "a_track":a_track,
+                        "b_track":b_track,
+                        "compositor_type":compositor_type,
+                        "clip":composited_clip}
+            action = edit.add_compositor_action(edit_data)
+            action.do_edit()
+        
+        updater.repaint_tline()
+        return
+
+    frame = tlinewidgets.get_frame(x)
+    clip_index = track.get_clip_index_at(frame)
+    
+    compositor_in = current_sequence().tracks[track.id].clip_start(clip_index)
+    clip_length = clip.clip_out - clip.clip_in
+    compositor_out = compositor_in + clip_length
+    
     edit_data = {"origin_clip_id":clip.id,
                 "in_frame":compositor_in,
                 "out_frame":compositor_out,
@@ -249,20 +315,48 @@ def _add_compositor(data):
                 "clip":clip}
     action = edit.add_compositor_action(edit_data)
     action.do_edit()
-    
+        
     updater.repaint_tline()
 
 def _add_autofade(data):
-    # NOTE: These stay synced only in "Top Down Auto Follow" mode, see: edit.get_full_compositor_sync_data()
+    # NOTE: These stay synhced only in "Top Down Auto Follow" mode, see: edit.get_full_compositor_sync_data()
     
     clip, track, item_id, item_data = data
-    x, compositor_type = item_data
+    x, compositor_type, add_compositors_is_multi_selection = item_data
 
     frame = tlinewidgets.get_frame(x)
     clip_index = track.get_clip_index_at(frame)
 
     target_track_index = track.id - 1
 
+    if add_compositors_is_multi_selection == True:
+        for clip_index in range(movemodes.selected_range_in, movemodes.selected_range_out + 1):
+            composited_clip = track.clips[clip_index]
+            if composited_clip.is_blanck_clip == True:
+                continue
+                
+            clip_length = composited_clip.clip_out - composited_clip.clip_in
+            if compositor_type == "##auto_fade_in":
+                compositor_in = current_sequence().tracks[track.id].clip_start(clip_index)
+                compositor_out = compositor_in + int(utils.fps()) - 1
+            else: # fade out
+                clip_start = current_sequence().tracks[track.id].clip_start(clip_index)
+                compositor_out = clip_start + clip_length
+                compositor_in = compositor_out - int(utils.fps()) + 1
+
+            edit_data = {"origin_clip_id":composited_clip.id,
+                        "in_frame":compositor_in,
+                        "out_frame":compositor_out,
+                        "a_track":target_track_index,
+                        "b_track":track.id,
+                        "compositor_type":compositor_type,
+                        "clip":composited_clip}
+            action = edit.add_compositor_action(edit_data)
+            action.do_edit()
+        
+        updater.repaint_tline()
+        return
+
     clip_length = clip.clip_out - clip.clip_in
     if compositor_type == "##auto_fade_in":
         compositor_in = current_sequence().tracks[track.id].clip_start(clip_index)
@@ -279,6 +373,7 @@ def _add_autofade(data):
                 "b_track":track.id,
                 "compositor_type":compositor_type,
                 "clip":clip}
+    #print("edit_data", edit_data)
     action = edit.add_compositor_action(edit_data)
     action.do_edit()
     
@@ -309,7 +404,7 @@ def _mute_clip(data):
 
 def _delete_clip(data):
     tlineaction.splice_out_button_pressed()
-    
+
 def _lift(data):
     tlineaction.lift_button_pressed()
     
@@ -437,7 +532,8 @@ def clear_filters():
     track = get_track(movemodes.selected_track)
     clips = []
     for i in range(movemodes.selected_range_in, movemodes.selected_range_out + 1):
-        clips.append(track.clips[i])
+        if track.clips[i].is_blanck_clip == False:
+            clips.append(track.clips[i])
 
     data = {"clips":clips}
     action = edit.remove_multiple_filters_action(data)
@@ -446,26 +542,74 @@ def clear_filters():
     movemodes.clear_selected_clips()
     updater.repaint_tline()
 
-def _display_wavefrom(data):
-    audiowaveform.set_waveform_displayer_clip_from_popup(data)
-
-def _clear_waveform(data):
-    audiowaveform.clear_waveform(data)
-
 def  _clone_filters_from_next(data):
-    clip, track, item_id, item_data = data
+    clip, track, item_id, is_multi = data
+    if is_multi == True:
+        end_index = movemodes.selected_range_out
+        if end_index == len(track.clips) - 1:
+            return # clip is last clip
+        clone_index = end_index + 1
+        while track.clips[clone_index].is_blanck_clip == True:
+            clone_index += 1
+            if clone_index == len(track.clips):
+                return # We auto delete blanks after clips so we should not hit this
+
+        clone_clip = track.clips[clone_index]
+                    
+        for clip_index in range(movemodes.selected_range_in, movemodes.selected_range_out + 1):
+            target_clip = track.clips[clip_index]
+            if target_clip.is_blanck_clip == True:
+                continue
+
+            _do_filter_clone(target_clip, clone_clip)
+            
+        return
+        
     index = track.clips.index(clip)
     if index == len(track.clips) - 1:
         return # clip is last clip
-    clone_clip = track.clips[index + 1]
+    clone_index = index + 1
+    while track.clips[clone_index].is_blanck_clip == True:
+        clone_index += 1
+        if clone_index == len(track.clips):
+            return # We auto delete blanks after clips so we should not hit this
+    clone_clip = track.clips[clone_index]
     _do_filter_clone(clip, clone_clip)
 
 def _clone_filters_from_prev(data):
-    clip, track, item_id, item_data = data
+    clip, track, item_id, is_multi = data
+    
+    if is_multi == True:
+        start_index = movemodes.selected_range_in
+        if start_index == 0:
+            return  # clip is first clip
+            
+        clone_index = start_index - 1
+        while track.clips[clone_index].is_blanck_clip == True:
+            clone_index -= 1
+            if clone_index == 0:
+                return
+
+        clone_clip = track.clips[clone_index]
+                    
+        for clip_index in range(movemodes.selected_range_in, movemodes.selected_range_out + 1):
+            target_clip = track.clips[clip_index]
+            if target_clip.is_blanck_clip == True:
+                continue
+
+            _do_filter_clone(target_clip, clone_clip)
+            
+        return
+        
     index = track.clips.index(clip)
     if index == 0:
         return # clip is first clip
-    clone_clip = track.clips[index - 1]
+    clone_index = index - 1
+    while track.clips[clone_index].is_blanck_clip == True:
+        clone_index -= 1
+        if clone_index == 0:
+            return 
+    clone_clip = track.clips[clone_index]
     _do_filter_clone(clip, clone_clip)
 
 def _do_filter_clone(clip, clone_clip):
@@ -479,6 +623,25 @@ def _clear_filters(data):
     clip, track, item_id, item_data = data
     clear_filters()
 
+def _multi_split_audio(data):
+    clip, track, item_id, item_data = data
+    clips = _get_non_blank_selected_clips(track)
+    
+    syncsplitevent.split_audio_from_clips_list(clips, track)
+
+def _multi_split_audio_synched(data):
+    clip, track, item_id, item_data = data
+    clips = _get_non_blank_selected_clips(track)
+    
+    syncsplitevent.split_audio_synched_from_clips_list(clips, track)
+
+def _get_non_blank_selected_clips(track):
+    clips = []
+    for i in range(movemodes.selected_range_in, movemodes.selected_range_out + 1):
+        if track.clips[i].is_blanck_clip == False:
+            clips.append(track.clips[i])
+    return clips
+
 def _select_all_after(data):
     clip, track, item_id, item_data = data
     movemodes._select_multiple_clips(track.id, track.clips.index(clip), len(track.clips) - 1)
@@ -489,95 +652,10 @@ def _select_all_before(data):
     movemodes._select_multiple_clips(track.id, 0, track.clips.index(clip))
     updater.repaint_tline()
 
-def _match_frame_start(data):
-    clip, track, item_id, item_data = data
-    _set_match_frame(clip, clip.clip_in, track, True)
-
-def _match_frame_end(data):
-    clip, track, item_id, item_data = data
-    _set_match_frame(clip, clip.clip_out, track, False)
-
-def _match_frame_start_monitor(data):
-    clip, track, item_id, item_data = data
-    gui.monitor_widget.set_frame_match_view(clip, clip.clip_in)
-
-def _match_frame_end_monitor(data):
-    clip, track, item_id, item_data = data
-    gui.monitor_widget.set_frame_match_view(clip, clip.clip_out)
-     
-def _set_match_frame(clip, frame, track, display_on_right):
-    global _match_frame_writer
-    _match_frame_writer = MatchFrameWriter(clip, frame, track, display_on_right)
-    
-    GLib.idle_add(_write_match_frame)
-
-def _write_match_frame():
-    _match_frame_writer.write_image()
-
-def _match_frame_close(data):
-    tlinewidgets.set_match_frame(-1, -1, True)
-    gui.monitor_widget.set_default_view_force()
-    updater.repaint_tline()
-
-
-class MatchFrameWriter:
-    def __init__(self, clip, clip_frame, track, display_on_right):
-        self.clip = clip
-        self.clip_frame = clip_frame
-        self.track = track
-        self.display_on_right = display_on_right
-        
-    def write_image(self):
-        """
-        Writes thumbnail image from file producer
-        """
-        clip_path = self.clip.path
-
-        # Create consumer
-        matchframe_new_path = userfolders.get_cache_dir() + appconsts.MATCH_FRAME_NEW
-        consumer = mlt.Consumer(PROJECT().profile, "avformat", matchframe_new_path)
-        consumer.set("real_time", 0)
-        consumer.set("vcodec", "png")
-
-        # Create one frame producer
-        producer = mlt.Producer(PROJECT().profile, str(clip_path))
-        producer = producer.cut(int(self.clip_frame), int(self.clip_frame))
-
-        # Delete new match frame
-        try:
-            os.remove(matchframe_new_path)
-        except:
-            # This fails when done first time ever  
-            pass
-
-        # Connect and write image
-        consumer.connect(producer)
-        consumer.run()
-        
-        # Wait until new file exists
-        while os.path.isfile(matchframe_new_path) != True:
-            time.sleep(0.1)
-
-        # Copy to match frame
-        matchframe_path = userfolders.get_cache_dir() + appconsts.MATCH_FRAME
-        shutil.copyfile(matchframe_new_path, matchframe_path)
-
-        # Update timeline data           
-        # Get frame of clip.clip_in_in on timeline.
-        clip_index = self.track.clips.index(self.clip)
-        clip_start_in_tline = self.track.clip_start(clip_index)
-        tline_match_frame = clip_start_in_tline + (self.clip_frame - self.clip.clip_in)
-        tlinewidgets.set_match_frame(tline_match_frame, self.track.id, self.display_on_right)
-
-        # Update view
-        updater.repaint_tline()
-
-
 def _add_clip_marker(data):
     clip, track, item_id, item_data = data
     current_frame = PLAYER().current_frame()
 
-    playhead_on_popup_clip = True
     try:
         current_frame_clip_index = track.get_clip_index_at(current_frame)
         current_frame_clip = track.clips[current_frame_clip_index]
@@ -586,7 +664,6 @@ def _add_clip_marker(data):
     
     if current_frame_clip != clip:
         # Playhead is not on popup clip
-        
         return
 
     clip_start_in_tline = track.clip_start(current_frame_clip_index)
@@ -669,8 +746,10 @@ def _reload_clip_media(data):
     action.do_edit()
     
     
-    
-    
+def _create_container_clip_from_selection(data):
+    GLib.idle_add(projectaction.create_selection_compound_clip)
+
+
 # Functions to handle popup menu selections for strings 
 # set as activation messages in guicomponents.py
 # activation_message -> _handler_func
@@ -684,22 +763,16 @@ POPUP_HANDLERS = {"set_master":syncsplitevent.init_select_master_clip,
                   "split_audio_synched":syncsplitevent.split_audio_synched,
                   "resync":syncsplitevent.resync_clip,
                   "add_filter":_add_filter,
+                  "add_filter_multi":_add_filter_multi,
                   "add_compositor":_add_compositor,
                   "clear_sync_rel":syncsplitevent.clear_sync_relation,
                   "mute_clip":_mute_clip,
-                  "display_waveform":_display_wavefrom,
-                  "clear_waveform":_clear_waveform,
                   "delete_blank":_delete_blank,
                   "cover_with_prev": _cover_blank_from_prev,
                   "cover_with_next": _cover_blank_from_next,
                   "clone_filters_from_next": _clone_filters_from_next,
                   "clone_filters_from_prev": _clone_filters_from_prev,
                   "clear_filters": _clear_filters,
-                  "match_frame_close":_match_frame_close,
-                  "match_frame_start":_match_frame_start,
-                  "match_frame_end":_match_frame_end,
-                  "match_frame_start_monitor":_match_frame_start_monitor,
-                  "match_frame_end_monitor":_match_frame_end_monitor,
                   "select_all_after": _select_all_after,
                   "select_all_before":_select_all_before,
                   "delete":_delete_clip,
@@ -717,9 +790,14 @@ POPUP_HANDLERS = {"set_master":syncsplitevent.init_select_master_clip,
                   "volumekf":_volume_keyframes,
                   "brightnesskf":_brightness_keyframes,
                   "delete_compositors":_delete_compositors,
+                  "multi_delete_compositors": _multi_delete_compositors,
                   "reload_media":_reload_clip_media,
+                  "create_multi_compound":_create_container_clip_from_selection,
+                  "multi_split_audio":_multi_split_audio,
+                  "multi_split_audio_synched":_multi_split_audio_synched,
                   "cc_render_full_media":containerclip.render_full_media,
                   "cc_render_clip":containerclip.render_clip_length,
                   "cc_go_to_underdered":containerclip.switch_to_unrendered_media,
                   "cc_render_settings":containerclip.set_render_settings,
-                  "cc_edit_program":containerclip.edit_program}
+                  "cc_edit_program":containerclip.edit_program,
+                  "cc_clone_generator":mediaplugin.add_media_plugin_clone}
