@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2012 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,10 +20,10 @@
 import appconsts
 import dialogutils
 import gui
-import editorstate
 from editorstate import current_sequence
 import propertyedit
 import propertyparse
+import propertyeditorbuilder
 
 """
 This module handles adding fade-ins and fade-outs to compositors and filters.
@@ -103,19 +103,21 @@ def set_auto_fade_out_keyframes(compositor):
     keyframes.append((0, 100))
     keyframes.append((compositor.get_length() - 1, 0))
 
+    print("before write out keyframes")
     keyframe_property.write_out_keyframes(keyframes)
-    
+    print("after write out keyframes")
+        
 # ---------------------------------------------------------------------- module functions
 def _get_kfproperty_klass_and_keyframes(compositor, clip):
     # We create a SECOND SET of EditableProperties from compositor properties.
     # These are not the same EditableProperties that are edited in GUI in "Compositor" panel.
-    # This approach seems necessery because Affine Blend requires creating a new property.
+    # This approach seems necessary because Affine Blend requires creating a new property.
     t_editable_properties = propertyedit.get_transition_editable_properties(compositor)
 
     # Find keyframe property, its class and create keyframes list
     if compositor.transition.info.mlt_service_id == "frei0r.cairoaffineblend": # Affine Blend
         # Because of frei0r's forced value 0.0-1.0 range "Affine Blend" is handled in a more complex way compared to other compositors
-        keyframe_property = propertyparse.create_editable_property_for_affine_blend(clip, t_editable_properties)
+        keyframe_property = propertyeditorbuilder.create_editable_property_for_affine_blend(clip, t_editable_properties)
         keyframes = propertyparse.rotating_geom_keyframes_value_string_to_geom_kf_array(keyframe_property.value, keyframe_property.get_in_value)
         property_klass = keyframe_property.__class__.__name__
         return (keyframe_property, property_klass, keyframes)
@@ -127,6 +129,7 @@ def _get_kfproperty_klass_and_keyframes(compositor, clip):
             property_klass = ep.__class__.__name__
             if property_klass == "OpacityInGeomKeyframeProperty": # Dissolve
                 keyframe_property = ep
+                print("HERE  OpacityInGeomKeyframeProperty")
                 keyframes = propertyparse.geom_keyframes_value_string_to_opacity_kf_array(keyframe_property.value, keyframe_property.get_in_value)
                 break
             if property_klass == "KeyFrameHCSTransitionProperty" and compositor.transition.info.mlt_service_id != "affine": # Blend, and we exclude Transform
@@ -155,27 +158,27 @@ def _get_compositor_clip(compositor):
 
 def _add_default_fade_in(keyframe_property, property_klass, keyframes, fade_in_length):
     if property_klass in _dissolve_property_klasses:
-        frame, opacity  = keyframes.pop(0)
-        keyframes.append((frame, 0))
-        keyframes.append((frame + fade_in_length, 100))
+        frame, opacity, kf_type = keyframes.pop(0)
+        keyframes.append((frame, 0, appconsts.KEYFRAME_LINEAR))
+        keyframes.append((frame + fade_in_length, 100, appconsts.KEYFRAME_LINEAR))
         return keyframes
     else:
         # (0, [0, 0, 1280, 720], 100.0) or (0, [640.0, 360.0, 1.0, 1.0, 0.0], 100.0) e.g.
-        frame, geom, opacity = keyframes.pop(0)
-        keyframes.append((frame, geom, 0))
-        keyframes.append((frame + fade_in_length, geom, 100))
+        frame, geom, opacity, kf_type = keyframes.pop(0)
+        keyframes.append((frame, geom, 0, appconsts.KEYFRAME_LINEAR))
+        keyframes.append((frame + fade_in_length, geom, 100, appconsts.KEYFRAME_LINEAR))
         return keyframes
 
 def _add_default_fade_out(keyframe_property, property_klass, keyframes, fade_out_length, clip, kf_before_fade_out_index=0):
     if property_klass in _dissolve_property_klasses:
-        keyframes.append((clip.clip_length() - fade_out_length - 1, 100))
-        keyframes.append((clip.clip_length() - 1, 0))
+        keyframes.append((clip.clip_length() - fade_out_length - 1, 100, appconsts.KEYFRAME_LINEAR))
+        keyframes.append((clip.clip_length() - 1, 0, appconsts.KEYFRAME_LINEAR))
         return keyframes
     else:
         # (0, [0, 0, 1280, 720], 100.0) or (0, [640.0, 360.0, 1.0, 1.0, 0.0], 100.0) e.g.
-        frame, geom, opacity = keyframes[kf_before_fade_out_index]
-        keyframes.append((clip.clip_length() - fade_out_length - 1, geom, 100))
-        keyframes.append((clip.clip_length() - 1, geom, 0))
+        frame, geom, opacity, kf_type = keyframes[kf_before_fade_out_index]
+        keyframes.append((clip.clip_length() - fade_out_length - 1, geom, 100, appconsts.KEYFRAME_LINEAR))
+        keyframes.append((clip.clip_length() - 1, geom, 0, appconsts.KEYFRAME_LINEAR))
         return keyframes
 
 def _do_user_add_fade_in(keyframe_property, property_klass, keyframes, fade_in_length):
@@ -185,9 +188,9 @@ def _do_user_add_fade_in(keyframe_property, property_klass, keyframes, fade_in_l
     for i in range (0, len(keyframes)):
         kf = keyframes[i]
         if property_klass in _dissolve_property_klasses:
-            frame, opacity  = kf
+            frame, opacity, kf_type = kf
         else:
-            frame, geom, opacity = kf
+            frame, geom, opacity, kf_type = kf
         
         if frame > fade_in_length:
             kf_after_fade_in_index = i
@@ -206,14 +209,14 @@ def _do_user_add_fade_in(keyframe_property, property_klass, keyframes, fade_in_l
         for i in range(0, kf_after_fade_in_index - 1):
             keyframes.pop(1)
         if property_klass in _dissolve_property_klasses:
-            frame, opacity  = keyframes.pop(0)
-            keyframes.insert(0, (frame, 0))
-            keyframes.insert(1,(frame + fade_in_length, 100))
+            frame, opacity, kf_type = keyframes.pop(0)
+            keyframes.insert(0, (frame, 0, appconsts.KEYFRAME_LINEAR))
+            keyframes.insert(1,(frame + fade_in_length, 100, appconsts.KEYFRAME_LINEAR))
         else:
             # (0, [0, 0, 1280, 720], 100.0) or (0, [640.0, 360.0, 1.0, 1.0, 0.0], 100.0) e.g.
-            frame, geom, opacity = keyframes.pop(0)
-            keyframes.insert(0, (frame, geom, 0))
-            keyframes.insert(1, (frame + fade_in_length, geom, 100))
+            frame, geom, opacity, kf_type = keyframes.pop(0)
+            keyframes.insert(0, (frame, geom, 0, appconsts.KEYFRAME_LINEAR))
+            keyframes.insert(1, (frame + fade_in_length, geom, 100, appconsts.KEYFRAME_LINEAR))
 
     # Because we created a SECOND SET of EditableProperties this only updates data structures (py and MLT)
     # but not EditableProperties wrappers that are edited in GUI in "Compositor" panel.
@@ -229,9 +232,9 @@ def _do_user_add_fade_out(keyframe_property, property_klass, keyframes, fade_out
     for i in range (0, len(keyframes)):
         kf = keyframes[i]
         if property_klass in _dissolve_property_klasses:
-            frame, opacity  = kf
+            frame, opacity, kf_type = kf
         else:
-            frame, geom, opacity = kf
+            frame, geom, opacity, kf_type = kf
         
         if frame >= fade_out_frame:
             kf_after_fade_out_index = i
@@ -261,7 +264,7 @@ def _do_user_add_fade_in_filter(keyframe_property, keyframes, fade_in_length, cl
     kf_after_fade_in_index = -1
     for i in range (0, len(keyframes)):
         kf = keyframes[i]
-        frame, opacity  = kf
+        frame, opacity, kf_type = kf
         
         if frame > clip_in + fade_in_length:
             kf_after_fade_in_index = i
@@ -275,8 +278,8 @@ def _do_user_add_fade_in_filter(keyframe_property, keyframes, fade_in_length, cl
 
         frame, opacity  = keyframes.pop(0)
 
-        keyframes.append((clip_in, 0))
-        keyframes.append((clip_in + fade_in_length, 100))
+        keyframes.append((clip_in, 0, appconsts.KEYFRAME_LINEAR))
+        keyframes.append((clip_in + fade_in_length, 100, appconsts.KEYFRAME_LINEAR))
     # Case keyframes exists after fade in length
     else:
         # Remove keyframes in range 0 - kf_after_fade_in_index
@@ -301,7 +304,7 @@ def _do_user_add_fade_out_filter(keyframe_property, keyframes, fade_out_length,
     kf_after_fade_out_index = -1
     for i in range (0, len(keyframes)):
         kf = keyframes[i]
-        frame, opacity  = kf
+        frame, opacity, kf_type  = kf
 
         if frame >= fade_out_frame:
             kf_after_fade_out_index = i
@@ -309,8 +312,8 @@ def _do_user_add_fade_out_filter(keyframe_property, keyframes, fade_out_length,
 
     # Case no keyframes after fade out start
     if kf_after_fade_out_index == -1:
-        keyframes.append((clip.clip_out - fade_out_length - 1, 100))
-        keyframes.append((clip.clip_out - 1, 0))
+        keyframes.append((clip.clip_out - fade_out_length - 1, 100, appconsts.KEYFRAME_LINEAR))
+        keyframes.append((clip.clip_out - 1, 0, appconsts.KEYFRAME_LINEAR))
 
     # Case keyframes exists after  fade out start
     else:
@@ -318,8 +321,8 @@ def _do_user_add_fade_out_filter(keyframe_property, keyframes, fade_out_length,
         for i in range(kf_after_fade_out_index, len(keyframes)):
             keyframes.pop(-1) # pop last
             
-        keyframes.append((clip.clip_out - fade_out_length - 1, 100))
-        keyframes.append((clip.clip_out - 1, 0))
+        keyframes.append((clip.clip_out - fade_out_length - 1, 100, appconsts.KEYFRAME_LINEAR))
+        keyframes.append((clip.clip_out - 1, 0, appconsts.KEYFRAME_LINEAR))
 
     # Because we created a SECOND SET of EditableProperties this only updates data structures (py and MLT)
     # but not EditableProperties wrappers that are edited in GUI in "Compositor" panel.
