@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2017 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
     along with Flowblade Movie Editor.  If not, see <http://www.gnu.org/licenses/>.
 """
 
-from gi.repository import Gtk, Gdk
+from gi.repository import Gtk, Gdk, GLib
 
 from os import listdir
 from os.path import isfile, join
@@ -37,8 +37,13 @@ RECREATE_WARNING = 1
 PROJECT_DATA_WARNING = 2
 
 _panels = None
+_legacy_disk_data_exists = True
 
 
+def legacy_disk_data_exists():
+    return _legacy_disk_data_exists
+    
+
 class DiskFolderManagementPanel:
     
     def __init__(self, xdg_folder, folder, info_text, warning_level, recursive=False):
@@ -47,7 +52,7 @@ class DiskFolderManagementPanel:
         self.warning_level = warning_level
         self.recursive = recursive
         
-        self.destroy_button = Gtk.Button(_("Destroy data"))
+        self.destroy_button = Gtk.Button(label=_("Destroy data"))
         self.destroy_button.connect("clicked", self.destroy_pressed)
         self.destroy_guard_check = Gtk.CheckButton()
         self.destroy_guard_check.set_active(False)
@@ -66,7 +71,7 @@ class DiskFolderManagementPanel:
             self.destroy_button.set_sensitive(False)
         button_area.pack_start(self.destroy_button, True, True, 0)
         if self.warning_level == PROJECT_DATA_WARNING:
-            warning_icon = Gtk.Image.new_from_stock(Gtk.STOCK_DIALOG_WARNING, Gtk.IconSize.SMALL_TOOLBAR)
+            warning_icon = Gtk.Image.new_from_icon_name(Gtk.STOCK_DIALOG_WARNING, Gtk.IconSize.SMALL_TOOLBAR)
             warning_icon.set_tooltip_text( _("Destroying this data may change contents of existing\nprojects and make some projects unopenable."))
             button_area.pack_start(warning_icon, False, False, 0)
         else:
@@ -118,34 +123,13 @@ class DiskFolderManagementPanel:
             return str(int(size)) + " B"
 
     def destroy_pressed(self, widget):
-        if self.warning_level == NO_WARNING:
-            # Delete data
-            self.destroy_data()
-            return
-            
-        primaty_text = _("Confirm Destroying Cached Data!")
-        if self.warning_level == PROJECT_DATA_WARNING:
-            secondary_text = _("Destroying this data may <b>change contents</b> of existing\nprojects or <b>make some projects unopenable!</b>")
-            secondary_text += "\n\n"
-            secondary_text += _("You can use 'File->Save Backup Snapshot...' functionality to backup projects\nso that they can be opened later before destroying this data.")
-        else:
-            secondary_text = _("Destroying this data may require parts of it to be recreated later.")
-            
-        dialogutils.warning_confirmation(self.warning_confirmation, primaty_text, secondary_text, gui.editor_window.window, None, False, True)
+        self.destroy_data()
 
     def destroy_guard_toggled(self, check_button):
         if check_button.get_active() == True:
             self.destroy_button.set_sensitive(True)
         else:
             self.destroy_button.set_sensitive(False)
-         
-    def warning_confirmation(self, dialog, response_id):
-        dialog.destroy()
-
-        if response_id != Gtk.ResponseType.ACCEPT:
-            return
-    
-        self.destroy_data()
     
     def destroy_data(self):
         print("deleting", self.folder)
@@ -161,7 +145,10 @@ class DiskFolderManagementPanel:
                     os.rmdir(file_path)
             else:
                 os.remove(file_path)
+    
+        GLib.idle_add(self._update_view)
 
+    def _update_view(self):
         self.size_info.set_text(self.get_folder_size_str())
         self.size_info.queue_draw()
 
@@ -173,9 +160,16 @@ def show_disk_management_dialog():
     global _panels
     _panels = _get_disk_dir_panels()
 
-    pane = Gtk.VBox(True, 2)
+    lagacy_label_text =  _("This dialog handles legacy Projects data created using versions before 2.12.\nUse <b>Data Store Manager</b> to inspect data created with versions 2.12 or higher\n\n")
+    lagacy_label_text += _("Destroying data using this dialog may <b>change contents</b> of existing\nprojects or <b>make some projects unopenable!</b>")
+    lagacy_label = Gtk.Label()
+    lagacy_label.set_markup("<small>" + lagacy_label_text + "</small>")
+    lagacy_label.set_margin_bottom(8)
+
+    pane = Gtk.VBox(False, 2)
+    pane.pack_start(lagacy_label, False, False, 0)
     for panel in _panels:
-        pane.pack_start(panel.vbox, True, True, 0)
+        pane.pack_start(panel.vbox, False, False, 0)
 
     guiutils.set_margins(pane, 12, 24, 12, 12)
 
@@ -193,20 +187,13 @@ def check_disk_cache_size():
     if check_level == 0:
         return
 
-    check_thread = DiskCacheWarningThread()
-    check_thread.start()
+    Gdk.threads_add_timeout(GLib.PRIORITY_HIGH_IDLE, 10, _check_cache_size)
 
-
-class DiskCacheWarningThread(threading.Thread):
-    
-    def __init__(self):
-        threading.Thread.__init__(self)
-
-    def run(self):
+def _check_cache_size():
+    try:
+        # We are running this check in GUI thread, but this should be fast enough to not block app GUI noticeably.
         check_level = editorpersistance.prefs.disk_space_warning
         
-        Gdk.threads_enter()
-        
         # Get disk cache size
         panels = _get_disk_dir_panels()
         used_disk_cache_size = 0
@@ -217,30 +204,36 @@ class DiskCacheWarningThread(threading.Thread):
 
         # check levels [off, 500 MB,1 GB, 2 GB], see preferenceswindow.py
         if check_level == 1 and used_disk_cache_size > 1000000 * 500:
-            self.show_warning(size_str)
+            _show_warning(size_str)
         elif check_level == 2 and used_disk_cache_size > 1000000 * 1000:
-            self.show_warning(size_str)
+            _show_warning(size_str)
         elif check_level == 3 and used_disk_cache_size > 1000000 * 2000:
-            self.show_warning(size_str)
+            _show_warning(size_str)
+    except Exception as e:
+        #print(str(e))
+        # No legacy data for layout prior to 2.12 exists.
+        global _legacy_disk_data_exists
+        _legacy_disk_data_exists = False
 
-        Gdk.threads_leave()
-
-    def show_warning(self, size_str):
-        primary_txt = _("Disk Cache Size Exceeds Current Warning Level!")
-        secondary_txt = _("Flowblade currently uses ") + size_str + _(" of disk space.") + "\n\n" + \
-                        _("You can either delete saved data using dialog opened with <b>Edit->Disk Cache</b> or") + "\n" + \
-                        _("change warning level in <b>Edit->Preferences 'General Options'</b> panel.") 
-        dialogutils.warning_message(primary_txt, secondary_txt, gui.editor_window.window, is_info=False)
+    return False
 
+def _show_warning(size_str):
+    primary_txt = _("Disk Cache Size Exceeds Current Warning Level!")
+    secondary_txt = _("Flowblade currently uses ") + size_str + _(" of disk space.") + "\n\n" + \
+                    _("You can either delete saved data using dialog opened with <b>Edit->Disk Cache</b> or") + "\n" + \
+                    _("change warning level in <b>Edit->Preferences 'General Options'</b> panel.") 
+    dialogutils.warning_message(primary_txt, secondary_txt, gui.editor_window.window, is_info=False)
 
 def _get_disk_dir_panels():
     panels = []
     panels.append(DiskFolderManagementPanel(userfolders.get_cache_dir(), appconsts.AUDIO_LEVELS_DIR, _("Audio Levels Data"), RECREATE_WARNING))
-    panels.append(DiskFolderManagementPanel(userfolders.get_cache_dir(), appconsts.GMIC_DIR, _("G'Mic Tool Session Data"), NO_WARNING))
+    # This is too small amount of data to make deletable.
+    #panels.append(DiskFolderManagementPanel(userfolders.get_cache_dir(), appconsts.GMIC_DIR, _("G'Mic Tool Session Data"), NO_WARNING))
     panels.append(DiskFolderManagementPanel(userfolders.get_data_dir(), appconsts.RENDERED_CLIPS_DIR, _("Rendered Files"), PROJECT_DATA_WARNING))
-    panels.append(DiskFolderManagementPanel(userfolders.get_render_dir(), "/" + appconsts.PROXIES_DIR, _("Proxy Files"), PROJECT_DATA_WARNING))
+    panels.append(DiskFolderManagementPanel(userfolders.get_render_dir(True), "/" + appconsts.PROXIES_DIR, _("Proxy Files"), PROJECT_DATA_WARNING))
     panels.append(DiskFolderManagementPanel(userfolders.get_data_dir(), appconsts.CONTAINER_CLIPS_DIR, _("Container Clips"), PROJECT_DATA_WARNING, True))
     panels.append(DiskFolderManagementPanel(userfolders.get_cache_dir(), appconsts.THUMBNAILS_DIR, _("Thumbnails"), RECREATE_WARNING))
-    panels.append(DiskFolderManagementPanel(userfolders.get_data_dir(), appconsts.USER_PROFILES_DIR_NO_SLASH, _("User Created Custom Profiles"), PROJECT_DATA_WARNING))
+    # This is too small amount of data to make deletable.
+    #panels.append(DiskFolderManagementPanel(userfolders.get_data_dir(), appconsts.USER_PROFILES_DIR_NO_SLASH, _("User Created Custom Profiles"), PROJECT_DATA_WARNING))
 
     return panels
