@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2012 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,9 +23,6 @@ Handles or passes on mouse edit events from timeline.
 
 Handles edit mode setting.
 """
-
-import os
-
 from gi.repository import Gdk
 
 import copy
@@ -36,9 +33,11 @@ import clipeffectseditor
 import clipenddragmode
 import compositeeditor
 import compositormodes
+import containeractions
 import cutmode
 import dialogs
 import dialogutils
+import dnd
 import edit
 import editorstate
 from editorstate import current_sequence
@@ -58,7 +57,6 @@ import syncsplitevent
 import tlinewidgets
 import trimmodes
 import updater
-import utils
 
 
 # functions are monkeypatched in at app.py 
@@ -232,18 +230,20 @@ def tline_canvas_mouse_pressed(event, frame):
     """
     Mouse event callback from timeline canvas widget
     """
-    editorstate.timeline_mouse_disabled = False # This is used to disable "move and "release" events when they would get bad data.
+    editorstate.timeline_mouse_disabled = False # This is used to disable "move" and "release" events when they would get bad data.
+    
+    dnd.clear_tline_out_drag_context() # This exits if tline out drag failed.
     
     if PLAYER().looping():
         return
     elif PLAYER().is_playing():
         PLAYER().stop_playback()
     
-    # Double click handled separately
+    # Double click handled separately.
     if event.type == Gdk.EventType._2BUTTON_PRESS:
         return
 
-    # Handle and exit parent clip selecting
+    # Handle and exit parent clip selecting.
     if EDIT_MODE() == editorstate.SELECT_PARENT_CLIP:
         syncsplitevent.select_sync_parent_mouse_pressed(event, frame)
         editorstate.timeline_mouse_disabled = True
@@ -251,7 +251,7 @@ def tline_canvas_mouse_pressed(event, frame):
         modesetting.set_default_edit_mode()  
         return
 
-    # Handle and exit tline sync clip selecting
+    # Handle and exit tline sync clip selecting.
     if EDIT_MODE() == editorstate.SELECT_TLINE_SYNC_CLIP:
         audiosync.select_sync_clip_mouse_pressed(event, frame)
         editorstate.timeline_mouse_disabled = True
@@ -264,12 +264,12 @@ def tline_canvas_mouse_pressed(event, frame):
     if not timeline_visible():
         updater.display_sequence_in_monitor()
         if (event.button == 1):
-            # Now that we have correct edit mode we'll reenter
-            # this method to get e.g. a select action
+            # Now that we have correct edit mode we'll re-enter
+            # this method to get e.g. a select action.
             tline_canvas_mouse_pressed(event, frame)
             return
         if (event.button == 3):
-            # Right mouse + CTRL displays clip menu if we hit clip
+            # Right mouse + CTRL displays clip menu if we hit clip.
             if (event.get_state() & Gdk.ModifierType.CONTROL_MASK):
                 PLAYER().seek_frame(frame)
             # Right mouse on timeline seeks frame
@@ -279,20 +279,13 @@ def tline_canvas_mouse_pressed(event, frame):
                     PLAYER().seek_frame(frame)
         return
 
-    # If clip end drag mode is for some reason still active, exit to default edit mode
+    # If clip end drag mode is for some reason still active, exit to default edit mode.
     if EDIT_MODE() == editorstate.CLIP_END_DRAG:
         modesetting.set_default_edit_mode()
         # This shouldn't happen unless for some reason mouse release didn't hit clipenddragmode listener.
         print("EDIT_MODE() == editorstate.CLIP_END_DRAG at mouse press!")
-
-    #  Check if match frame close is hit
-    if editorstate.current_is_move_mode() and timeline_visible():
-        if tlinewidgets.match_frame_close_hit(event.x, event.y) == True:
-            tlinewidgets.set_match_frame(-1, -1, True)
-            updater.repaint_tline()
-            return
-
-    #  Check if compositor is hit and if so, handle compositor editing
+    
+    #  Check if compositor is hit and if so, handle compositor editing.
     if editorstate.current_is_move_mode() and timeline_visible():
         hit_compositor = tlinewidgets.compositor_hit(frame, event.x, event.y, current_sequence().compositors)
         if hit_compositor != None:
@@ -321,7 +314,7 @@ def tline_canvas_mouse_pressed(event, frame):
 
     compositormodes.clear_compositor_selection()
 
-    # Check if we should enter clip end drag mode
+    # Check if we should enter clip end drag mode.
     if (event.button == 3 and editorstate.current_is_move_mode()
         and timeline_visible() and (event.get_state() & Gdk.ModifierType.CONTROL_MASK)):
         # with CTRL right mouse
@@ -333,7 +326,7 @@ def tline_canvas_mouse_pressed(event, frame):
 
     # Handle mouse button presses depending which button was pressed and
     # editor state.
-    # RIGHT BUTTON: seek frame or display clip menu if not dragging clip end
+    # RIGHT BUTTON: seek frame or display clip menu if not dragging clip end.
     if (event.button == 3 and EDIT_MODE() != editorstate.CLIP_END_DRAG and EDIT_MODE() != editorstate.KF_TOOL):
         if ((not editorstate.current_is_active_trim_mode()) and timeline_visible()):
             if not(event.get_state() & Gdk.ModifierType.CONTROL_MASK):
@@ -341,7 +334,7 @@ def tline_canvas_mouse_pressed(event, frame):
                 if not success:
                     PLAYER().seek_frame(frame)
         else:
-            # For trim modes set <X>_NO_EDIT edit mode and seek frame. and seek frame
+            # For trim modes set <X>_NO_EDIT edit mode and seek frame. and seek frame.
             trimmodes.set_no_edit_trim_mode()
             PLAYER().seek_frame(frame)
         return
@@ -355,13 +348,13 @@ def tline_canvas_mouse_pressed(event, frame):
         if not success:
             modesetting.set_default_edit_mode(True)	
             return	
-            	
+
         if trimmodes.edit_data["to_side_being_edited"] == True:	
             pointer_context = appconsts.POINTER_CONTEXT_TRIM_LEFT	
         else:	
             pointer_context = appconsts.POINTER_CONTEXT_TRIM_RIGHT	
-        gui.editor_window.set_tline_cursor_to_context(pointer_context)	
-        gui.editor_window.set_tool_selector_to_mode()	
+        gui.editor_window.tline_cursor_manager.set_tline_cursor_to_context(pointer_context)	
+        gui.editor_window.tline_cursor_manager.set_tool_selector_to_mode()	
         if not editorpersistance.prefs.quick_enter_trims:	
             editorstate.timeline_mouse_disabled = True	
         else:	
@@ -402,10 +395,10 @@ def tline_canvas_mouse_released(x, y, frame, button, state):
     """
     Mouse event callback from timeline canvas widget
     """
-    gui.editor_window.set_cursor_to_mode() # we need this for box move at least, probably trims too
+    gui.editor_window.tline_cursor_manager.set_cursor_to_mode() # we need this for box move at least, probably trims too
      
     if editorstate.timeline_mouse_disabled == True:
-        gui.editor_window.set_cursor_to_mode() # we only need this update when mode change (to active trim mode) disables mouse, so we'll only do this then
+        gui.editor_window.tline_cursor_manager.set_cursor_to_mode() # we only need this update when mode change (to active trim mode) disables mouse, so we'll only do this then
         tlinewidgets.trim_mode_in_non_active_state = False # we only need this update when mode change (to active trim mode) disables mouse, so we'll only do this then
         gui.tline_canvas.widget.queue_draw()
         editorstate.timeline_mouse_disabled = False
@@ -472,13 +465,43 @@ def tline_effect_drop(x, y):
     if dialogutils.track_lock_check_and_user_info(track):
         modesetting.set_default_edit_mode()
         return
-        
-    if clip != clipeffectseditor.clip:
-        clipeffectseditor.set_clip(clip, track, index)
-    
-    clipeffectseditor.add_currently_selected_effect() # drag start selects the dragged effect
 
-def tline_media_drop(media_file, x, y, use_marks=False):
+    filter_info = clipeffectseditor.get_currently_selected_filter_info()
+                
+    selected_track_before = movemodes.selected_track
+    selected_in_before = movemodes.selected_range_in
+    selected_out_before = movemodes.selected_range_out
+    
+    # Effect dropped on selected range, add to all in range.
+    if selected_in_before != -1 and selected_track_before == track.id and ((selected_in_before <= index) and (selected_out_before >= index)):
+        actions = []
+        for add_index in range(selected_in_before, selected_out_before + 1):
+            add_clip = track.clips[add_index]
+            if add_clip.is_blanck_clip == True:
+                continue
+
+            data = {"clip":add_clip, 
+                    "filter_info":filter_info,
+                    "filter_edit_done_func":clipeffectseditor.filter_edit_done_stack_update}
+            action = edit.add_filter_action(data)
+            actions.append(action)
+    
+        if len(actions) > 0:
+            c_action = edit.ConsolidatedEditAction(actions)
+            c_action.do_consolidated_edit()
+    else:
+        # Effect dropped on single clip.
+        data = {"clip":clip, 
+                "filter_info":filter_info,
+                "filter_edit_done_func":clipeffectseditor.filter_edit_done_stack_update}
+        action = edit.add_filter_action(data)
+        action.do_edit()
+
+    clipeffectseditor.set_clip(clip, track, index)
+    clipeffectseditor.set_filter_item_expanded(len(clip.filters) - 1)
+    
+def tline_media_drop(drag_data, x, y, use_marks=False):
+    # drag_data not used unless we which later to enable dropping multiple media items.
     track = tlinewidgets.get_track(y)
     if track == None:
         return
@@ -494,6 +517,11 @@ def tline_media_drop(media_file, x, y, use_marks=False):
         kftoolmode.exit_tool()
 
     frame = tlinewidgets.get_frame(x)
+
+    if dnd.drag_source == dnd.SOURCE_MONITOR_WIDGET:
+        media_file = dnd.drag_data
+    else:
+        media_file = gui.media_list_view.last_pressed.media_file
     
     # Create new clip.
     if media_file.type != appconsts.PATTERN_PRODUCER:
@@ -515,50 +543,57 @@ def tline_media_drop(media_file, x, y, use_marks=False):
 
         if media_file.type == appconsts.IMAGE_SEQUENCE:
             new_clip.mark_out = media_file.length
-    else:
+    else:    
+        # Media types that do not have length determined by content.
         if new_clip.media_type == appconsts.IMAGE or new_clip.media_type == appconsts.PATTERN_PRODUCER:
-            # Give IMAGE and PATTERN_PRODUCER media types default mark in and mark out if not already set.
-            # This makes them reasonably short and trimmable in both directions.
-            # NOTE: WE SHOULD BE DOING THIS AT CREATION TIME, WE'RE DOING THE SAME THING IN updater.display_clip_in_monitor() ?
-            #       ...but then we would need to patch persistance.py...maybe keep this even if not too smart.
-            # TODO: Make default length user settable or use graphics value
-            if (hasattr(new_clip, 'mark_in') == False) or (new_clip.mark_in == -1 and new_clip.mark_out == -1):
-                center_frame = new_clip.get_length() // 2
-                default_length_half = 75
-                mark_in = center_frame - default_length_half
-                mark_out = center_frame + default_length_half - 1
-                new_clip.mark_in = mark_in
-                new_clip.mark_out = mark_out
+
+            # If no marks use default length.
+            # This is different from media that has length determined by content.
+            if (hasattr(media_file, 'mark_in') == False) or (media_file.mark_in == -1 and media_file.mark_out == -1):
+                in_fr, out_fr, l = editorpersistance.get_graphics_default_in_out_length()
+                new_clip.mark_in = in_fr
+                new_clip.mark_out = out_fr
+            else:
+                new_clip.mark_in = media_file.mark_in
+                new_clip.mark_out = media_file.mark_out
+                
+            # Replace single missing mark in 3-point edit style.
+            if new_clip.mark_in == -1:
+                new_clip.mark_in = 0
+            if new_clip.mark_out == -1:
+                new_clip.mark_out = 14999
         else: # All the rest
+
             new_clip.mark_in = media_file.mark_in
             new_clip.mark_out = media_file.mark_out
 
-        if new_clip.mark_in == -1:
-            new_clip.mark_in = 0
-        if new_clip.mark_out == -1:
-            new_clip.mark_out = new_clip.get_length() - 1 # - 1 because out is mark_out inclusive
-            if media_file.type == appconsts.IMAGE_SEQUENCE:
-                new_clip.mark_out = media_file.length
-
-    # Graphics files get added with their default lengths
-    f_name, ext = os.path.splitext(media_file.name)
-    if utils.file_extension_is_graphics_file(ext) and media_file.type != appconsts.IMAGE_SEQUENCE: # image sequences are graphics files but have own length
-        in_fr, out_fr, l = editorpersistance.get_graphics_default_in_out_length()
-        new_clip.mark_in = in_fr
-        new_clip.mark_out = out_fr
+            # Replace single missing mark in 3-point edit style.
+            if new_clip.mark_in == -1:
+                new_clip.mark_in = 0
+            if new_clip.mark_out == -1:
+                new_clip.mark_out = new_clip.get_length() - 1 # - 1 because out is mark_out inclusive
+                if media_file.type == appconsts.IMAGE_SEQUENCE:
+                    new_clip.mark_out = media_file.length
 
     # Non-insert DND actions
     if editorpersistance.prefs.dnd_action == appconsts.DND_OVERWRITE_NON_V1:
         if track.id != current_sequence().first_video_track().id:
             drop_done = _attempt_dnd_overwrite(track, new_clip, frame)
             if drop_done == True:
+                maybe_autorender_plugin(new_clip)
+                gui.media_list_view.clear_selection()
                 return
     elif editorpersistance.prefs.dnd_action == appconsts.DND_ALWAYS_OVERWRITE:
         drop_done = _attempt_dnd_overwrite(track, new_clip, frame)
         if drop_done == True:
+            maybe_autorender_plugin(new_clip)
+            gui.media_list_view.clear_selection()
             return
             
     do_clip_insert(track, new_clip, frame)
+    gui.media_list_view.clear_selection()
+                
+    maybe_autorender_plugin(new_clip)
 
 def tline_range_item_drop(rows, x, y):
     track = tlinewidgets.get_track(y)
@@ -575,7 +610,14 @@ def tline_range_item_drop(rows, x, y):
     modesetting.set_default_edit_mode()
     do_multiple_clip_insert(track, clips, frame)
 
+def maybe_autorender_plugin(clip):
+    if clip.container_data == None:
+        return
 
+    if editorpersistance.prefs.auto_render_media_plugins == True:
+        if clip.container_data.container_type == appconsts.CONTAINER_CLIP_FLUXITY:
+            action_object = containeractions.get_action_object(clip.container_data)
+            action_object.render_full_media(clip)
 
 
 # ------------------------------------ function tables
