@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2012 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,15 +22,21 @@
 This modules handles displaying and moving panels into different positions 
 in application window.
 """
-from gi.repository import Gtk
+from gi.repository import Gtk, GLib
 
 import copy
+import pickle
 
 import appconsts
+import atomicfile
 import editorpersistance
 import editorstate
+import dialogs
+import dialogutils
 import gui
-import middlebar
+import guiutils
+import guipopover
+import utils
 
 # Transforms when adding panels.
 # 0 -> 1    The pre-created Gtk.Frame is filled with added panel.
@@ -40,34 +46,69 @@ import middlebar
 
 # Transforms when removing panels.
 # N -> 2    Panel is removed from notebook.
-# 2 -> 1    Notebook is removed from pre-created Gtk.Frame, panel is removed from noteboook
+# 2 -> 1    Notebook is removed from pre-created Gtk.Frame, panel is removed from notebook
 #           and added to pre-created Gtk.Frame.
 # 1 -> 0    Panel is removed from the pre-created Gtk.Frame.
 
 # Pre-created Gtk.Frames exist all through app life-cycle, notebooks are dynamically created
 # as needed.
 
-# The exception for these transforms here is top row default notebook 
-# that is forced to always exit with at least two panels displayed.
+# The exception for these transforms here is the top row default notebook 
+# that is forced to always exits with at least two panels displayed.
 # This makes easier to add and remove top row panels.
 
+ # This is no longer default. Available as layout option 'Monitor Left'
 DEFAULT_PANEL_POSITIONS = { \
     appconsts.PANEL_MEDIA: appconsts.PANEL_PLACEMENT_TOP_ROW_DEFAULT,
-    appconsts.PANEL_FILTER_SELECT: appconsts.PANEL_PLACEMENT_BOTTOM_ROW_RIGHT, # This is the only different default position.
+    appconsts.PANEL_FILTER_SELECT: appconsts.PANEL_PLACEMENT_BOTTOM_ROW_RIGHT,
     appconsts.PANEL_RANGE_LOG: appconsts.PANEL_PLACEMENT_TOP_ROW_DEFAULT,
-    appconsts.PANEL_FILTERS: appconsts.PANEL_PLACEMENT_TOP_ROW_DEFAULT,
-    appconsts.PANEL_COMPOSITORS: appconsts.PANEL_PLACEMENT_TOP_ROW_DEFAULT,
+    appconsts.PANEL_MULTI_EDIT: appconsts.PANEL_PLACEMENT_TOP_ROW_DEFAULT,
     appconsts.PANEL_JOBS: appconsts.PANEL_PLACEMENT_TOP_ROW_DEFAULT,
-    appconsts.PANEL_PROJECT_SMALL_SCREEN: appconsts.PANEL_PLACEMENT_TOP_ROW_PROJECT_DEFAULT, # default values are for large screen single window layout, these are modified on init if needed.
+    appconsts.PANEL_PROJECT_SMALL_SCREEN: appconsts.PANEL_PLACEMENT_TOP_ROW_PROJECT_DEFAULT,
     appconsts.PANEL_PROJECT: appconsts.PANEL_PLACEMENT_TOP_ROW_PROJECT_DEFAULT,
     appconsts.PANEL_RENDERING: appconsts.PANEL_PLACEMENT_TOP_ROW_DEFAULT,
-    appconsts.PANEL_MEDIA_AND_BINS_SMALL_SCREEN: None # default values are for large screen single window layout, these are modified on startup if needed.
+    appconsts.PANEL_MEDIA_AND_BINS_SMALL_SCREEN: None
+}
+
+MONITOR_CENTER_PANEL_POSITIONS = { \
+    appconsts.PANEL_MEDIA: appconsts.PANEL_PLACEMENT_TOP_ROW_DEFAULT,
+    appconsts.PANEL_FILTER_SELECT: appconsts.PANEL_PLACEMENT_BOTTOM_ROW_RIGHT,
+    appconsts.PANEL_RANGE_LOG: appconsts.PANEL_PLACEMENT_TOP_ROW_DEFAULT,
+    appconsts.PANEL_MULTI_EDIT: appconsts.PANEL_PLACEMENT_TOP_ROW_RIGHT,
+    appconsts.PANEL_JOBS: appconsts.PANEL_PLACEMENT_TOP_ROW_DEFAULT,
+    appconsts.PANEL_PROJECT_SMALL_SCREEN: appconsts.PANEL_PLACEMENT_TOP_ROW_PROJECT_DEFAULT,
+    appconsts.PANEL_PROJECT: appconsts.PANEL_PLACEMENT_BOTTOM_ROW_LEFT,
+    appconsts.PANEL_RENDERING: appconsts.PANEL_PLACEMENT_TOP_ROW_DEFAULT,
+    appconsts.PANEL_MEDIA_AND_BINS_SMALL_SCREEN: None
+}
+
+TOP_ROW_FOUR_POSITIONS = { \
+    appconsts.PANEL_MEDIA: appconsts.PANEL_PLACEMENT_TOP_ROW_DEFAULT,
+    appconsts.PANEL_FILTER_SELECT: appconsts.PANEL_PLACEMENT_BOTTOM_ROW_RIGHT,
+    appconsts.PANEL_RANGE_LOG: appconsts.PANEL_PLACEMENT_TOP_ROW_DEFAULT,
+    appconsts.PANEL_MULTI_EDIT: appconsts.PANEL_PLACEMENT_TOP_ROW_RIGHT,
+    appconsts.PANEL_JOBS: appconsts.PANEL_PLACEMENT_TOP_ROW_DEFAULT,
+    appconsts.PANEL_PROJECT_SMALL_SCREEN: appconsts.PANEL_PLACEMENT_TOP_ROW_PROJECT_DEFAULT,
+    appconsts.PANEL_PROJECT: appconsts.PANEL_PLACEMENT_TOP_ROW_PROJECT_DEFAULT,
+    appconsts.PANEL_RENDERING: appconsts.PANEL_PLACEMENT_TOP_ROW_DEFAULT,
+    appconsts.PANEL_MEDIA_AND_BINS_SMALL_SCREEN: None
+}
+
+MEDIA_PANEL_LEFT_POSITIONS = { \
+    appconsts.PANEL_MEDIA: appconsts.PANEL_PLACEMENT_LEFT_COLUMN,
+    appconsts.PANEL_FILTER_SELECT: appconsts.PANEL_PLACEMENT_NOT_VISIBLE,
+    appconsts.PANEL_RANGE_LOG: appconsts.PANEL_PLACEMENT_TOP_ROW_DEFAULT,
+    appconsts.PANEL_MULTI_EDIT: appconsts.PANEL_PLACEMENT_TOP_ROW_DEFAULT,
+    appconsts.PANEL_JOBS: appconsts.PANEL_PLACEMENT_TOP_ROW_DEFAULT,
+    appconsts.PANEL_PROJECT_SMALL_SCREEN: appconsts.PANEL_PLACEMENT_TOP_ROW_PROJECT_DEFAULT,
+    appconsts.PANEL_PROJECT: appconsts.PANEL_PLACEMENT_BOTTOM_ROW_LEFT,
+    appconsts.PANEL_RENDERING: appconsts.PANEL_PLACEMENT_TOP_ROW_DEFAULT,
+    appconsts.PANEL_MEDIA_AND_BINS_SMALL_SCREEN: None
 }
 
 AVAILABLE_PANEL_POSITIONS_OPTIONS = { \
     appconsts.PANEL_MEDIA: [appconsts.PANEL_PLACEMENT_TOP_ROW_DEFAULT, appconsts.PANEL_PLACEMENT_TOP_ROW_RIGHT, appconsts.PANEL_PLACEMENT_LEFT_COLUMN, appconsts.PANEL_PLACEMENT_BOTTOM_ROW_LEFT],
-    appconsts.PANEL_FILTERS: [appconsts.PANEL_PLACEMENT_TOP_ROW_DEFAULT,  appconsts.PANEL_PLACEMENT_TOP_ROW_RIGHT, appconsts.PANEL_PLACEMENT_LEFT_COLUMN, appconsts.PANEL_PLACEMENT_BOTTOM_ROW_LEFT],
-    appconsts.PANEL_COMPOSITORS: [appconsts.PANEL_PLACEMENT_TOP_ROW_DEFAULT, appconsts.PANEL_PLACEMENT_TOP_ROW_RIGHT, appconsts.PANEL_PLACEMENT_BOTTOM_ROW_LEFT, appconsts.PANEL_PLACEMENT_BOTTOM_ROW_RIGHT],
+    appconsts.PANEL_MULTI_EDIT: [appconsts.PANEL_PLACEMENT_TOP_ROW_DEFAULT,  appconsts.PANEL_PLACEMENT_TOP_ROW_RIGHT, appconsts.PANEL_PLACEMENT_LEFT_COLUMN, appconsts.PANEL_PLACEMENT_BOTTOM_ROW_LEFT],
     appconsts.PANEL_RANGE_LOG: [appconsts.PANEL_PLACEMENT_TOP_ROW_DEFAULT, appconsts.PANEL_PLACEMENT_TOP_ROW_RIGHT, appconsts.PANEL_PLACEMENT_BOTTOM_ROW_LEFT, appconsts.PANEL_PLACEMENT_BOTTOM_ROW_RIGHT],
     appconsts.PANEL_RENDERING: [appconsts.PANEL_PLACEMENT_TOP_ROW_DEFAULT, appconsts.PANEL_PLACEMENT_TOP_ROW_RIGHT, appconsts.PANEL_PLACEMENT_BOTTOM_ROW_LEFT, appconsts.PANEL_PLACEMENT_BOTTOM_ROW_RIGHT],
     appconsts.PANEL_JOBS: [appconsts.PANEL_PLACEMENT_TOP_ROW_DEFAULT, appconsts.PANEL_PLACEMENT_TOP_ROW_RIGHT, appconsts.PANEL_PLACEMENT_BOTTOM_ROW_LEFT, appconsts.PANEL_PLACEMENT_BOTTOM_ROW_RIGHT],
@@ -78,17 +119,15 @@ AVAILABLE_PANEL_POSITIONS_OPTIONS = { \
 }
 
 PANEL_ORDER_IN_NOTEBOOKS = [appconsts.PANEL_MEDIA, appconsts.PANEL_FILTER_SELECT, appconsts.PANEL_RANGE_LOG, 
-                            appconsts.PANEL_FILTERS, appconsts.PANEL_COMPOSITORS,
-                            appconsts.PANEL_JOBS, appconsts.PANEL_PROJECT_SMALL_SCREEN, 
+                            appconsts.PANEL_MULTI_EDIT, appconsts.PANEL_JOBS, appconsts.PANEL_PROJECT_SMALL_SCREEN, 
                             appconsts.PANEL_PROJECT, appconsts.PANEL_RENDERING,
                             appconsts.PANEL_MEDIA_AND_BINS_SMALL_SCREEN]
 
 PANEL_MINIMUM_SIZES = { \
-    appconsts.PANEL_MEDIA: None, 
+    appconsts.PANEL_MEDIA: (appconsts.PANEL_MEDIA_MINIMUM_SIZE, appconsts.TOP_ROW_HEIGHT), 
     appconsts.PANEL_FILTER_SELECT: None,
     appconsts.PANEL_RANGE_LOG: None,
-    appconsts.PANEL_FILTERS: (510, 100), # This has very small default size when empty and needs preferred size set to show properly when moved.
-    appconsts.PANEL_COMPOSITORS: (510, 100), # This has very small default size when empty and needs preferred size set to show properly when moved.
+    appconsts.PANEL_MULTI_EDIT: (appconsts.PANEL_MULTI_EDIT_MINIMUM_SIZE, appconsts.TOP_ROW_HEIGHT),
     appconsts.PANEL_JOBS: None,
     appconsts.PANEL_RENDERING: None,
     appconsts.PANEL_PROJECT: None,
@@ -110,14 +149,13 @@ DEFAULT_TABS_POSITIONS = { \
 # Saved data struct holding panel positions information.
 _panel_positions = None
 
-# Gtk.Notebooks that may or may not exist to containe 2-N panel is layou position
+# Gtk.Notebooks that may or may not exist to contain 2-N panel is layout position
 _position_notebooks = {}
 
 # Dicts for translations.
 _positions_names = {}
 _panels_names = {}
 
-
 def top_level_project_panel():
     if editorpersistance.prefs.top_level_project_panel == True and editorstate.SCREEN_WIDTH > 1440 and editorstate.SCREEN_HEIGHT > 898:
         return True
@@ -129,9 +167,14 @@ def init_layout_data():
     global _panel_positions, _positions_names, _panels_names, _position_notebooks
     _panel_positions = editorpersistance.prefs.panel_positions
 
-    # Use default panels positons if nothing available yet or too small screen
-    if panel_positioning_available() == False or _panel_positions == None:
+    # Use default panels positions if nothing available yet or too small screen, 
+    # or default panel position is empty, layout code makes too many assumptions to make that work.
+    if panel_positioning_available() == False or _panel_positions == None \
+       or not (appconsts.PANEL_PLACEMENT_TOP_ROW_DEFAULT in _panel_positions.values()):
         _panel_positions = copy.deepcopy(DEFAULT_PANEL_POSITIONS)
+        if editorstate.SCREEN_WIDTH < 1439:
+            # App window may become invsible if all buttons shown in midbar and window too wide for screen.
+            _panel_positions[appconsts.PANEL_FILTER_SELECT] = appconsts.PANEL_PLACEMENT_NOT_VISIBLE
         editorpersistance.prefs.panel_positions = _panel_positions
         editorpersistance.save()
 
@@ -139,15 +182,20 @@ def init_layout_data():
         editorpersistance.prefs.positions_tabs = DEFAULT_TABS_POSITIONS
         editorpersistance.save()
 
+    # PANEL_COMPOSITORS was deprecated whenmoving to use multi edit panel.
+    if appconsts.PANEL_COMPOSITORS in _panel_positions:
+        _panel_positions.pop(appconsts.PANEL_COMPOSITORS)
+
     # Force media panel positioning to work with both one and two window modes 
     if editorpersistance.prefs.global_layout == appconsts.SINGLE_WINDOW:
+        # When two windows -> single window put media panel in dewfault pos
         if _panel_positions[appconsts.PANEL_MEDIA] == appconsts.PANEL_PLACEMENT_TWO_WINDOWS_MEDIA_PANEL_POS:
             _panel_positions[appconsts.PANEL_MEDIA] = appconsts.PANEL_PLACEMENT_TOP_ROW_DEFAULT
     else:
-        if _panel_positions[appconsts.PANEL_MEDIA] == appconsts.PANEL_PLACEMENT_TOP_ROW_DEFAULT:
-            _panel_positions[appconsts.PANEL_MEDIA] = appconsts.PANEL_PLACEMENT_TWO_WINDOWS_MEDIA_PANEL_POS
+        # When two windows always force pos  PANEL_PLACEMENT_TWO_WINDOWS_MEDIA_PANEL_POS for media.
+        _panel_positions[appconsts.PANEL_MEDIA] = appconsts.PANEL_PLACEMENT_TWO_WINDOWS_MEDIA_PANEL_POS
 
-    # We are using different media panels for differnt screen sizes,
+    # We are using different media panels for different screen sizes,
     # make sure that are using and displaying the right one here even screen size has changed.
     if top_level_project_panel() == True:
         if appconsts.PANEL_PROJECT_SMALL_SCREEN in _panel_positions:
@@ -171,8 +219,7 @@ def init_layout_data():
     
     _panels_names = { \
         appconsts.PANEL_MEDIA: _("Media"),
-        appconsts.PANEL_FILTERS: _("Filters"),
-        appconsts.PANEL_COMPOSITORS: _("Compositors"),
+        appconsts.PANEL_MULTI_EDIT: _("Edit"),
         appconsts.PANEL_RANGE_LOG: _("Range Log"),
         appconsts.PANEL_RENDERING: _("Render"),
         appconsts.PANEL_JOBS: _("Jobs"),
@@ -223,8 +270,7 @@ def _get_position_panels(position):
 def _get_panels_widgets_dict(editor_window):
     _panels_widgets = { \
         appconsts.PANEL_MEDIA: editor_window.mm_paned,
-        appconsts.PANEL_FILTERS: editor_window.effects_panel,
-        appconsts.PANEL_COMPOSITORS: editor_window.compositors_panel,
+        appconsts.PANEL_MULTI_EDIT: editor_window.edit_multi.widget,
         appconsts.PANEL_RANGE_LOG: editor_window.media_log_panel,
         appconsts.PANEL_RENDERING: editor_window.render_panel,
         appconsts.PANEL_JOBS: editor_window.jobs_pane,
@@ -281,25 +327,16 @@ def _set_min_size(panel_id, widget):
 def panel_positioning_available():
     # This feature now available only for 1680x1050 and larger screens.
     # Maybe some reduced version later for smaller screens.
-    if editorstate.SCREEN_WIDTH > 1678 and  editorstate.SCREEN_HEIGHT > 1048:
+    if editorstate.SCREEN_WIDTH > 1678 and editorstate.SCREEN_HEIGHT > 1048:
         return True
     
     return False
 
-def get_bottom_row_minimum_width():
-    middle_bar_w, dummy = gui.editor_window.edit_buttons_row.get_preferred_width()
-    bottom_left_w, dummy = _get_position_frames_dict()[appconsts.PANEL_PLACEMENT_BOTTOM_ROW_LEFT].get_preferred_width()
-    bottom_right_w, dummy = _get_position_frames_dict()[appconsts.PANEL_PLACEMENT_BOTTOM_ROW_RIGHT].get_preferred_width()
-    left_column_w, dummy = _get_position_frames_dict()[appconsts.PANEL_PLACEMENT_LEFT_COLUMN].get_preferred_width() 
-    combined = middle_bar_w + bottom_left_w + bottom_right_w + left_column_w
-
-    return combined
-
 def set_positions_frames_visibility():
     # if frame has 0 panels in it, hide
     for position in _positions_names:
         if position == appconsts.PANEL_PLACEMENT_NOT_VISIBLE:
-            continue # this not applicaple here
+            continue # this not applicable here
         frame = _get_position_frames_dict()[position]
         panels = _get_position_panels(position)
         if len(panels) > 0:
@@ -356,12 +393,22 @@ def _create_notebook(position, editor_window):
         
     return notebook
     
-# ---------------------------------------------------------- APP MENU
+# ---------------------------------------------------------- APP MENU, TOP BAR BUTTON MENU
 def get_panel_positions_menu_item():
     panel_positions_menu_item = Gtk.MenuItem(_("Panel Placement"))
     panel_positions_menu = Gtk.Menu()
     panel_positions_menu_item.set_submenu(panel_positions_menu)
 
+    # Presets
+    if editorpersistance.prefs.global_layout == appconsts.SINGLE_WINDOW:
+        presets_menu_item = Gtk.MenuItem(_("Preset Layouts"))
+        presets_menu = Gtk.Menu()
+        presets_menu_item.set_submenu(presets_menu)
+        _create_layout_presets_menu(presets_menu)
+        panel_positions_menu.append(presets_menu_item)
+        sep = Gtk.SeparatorMenuItem()
+        panel_positions_menu.append(sep)
+    
     # Project Panel
     project_panel_menu_item = Gtk.MenuItem(_("Project Panel"))
     panel_positions_menu.append(project_panel_menu_item)
@@ -369,7 +416,7 @@ def get_panel_positions_menu_item():
     project_panel_menu = _get_position_selection_menu(appconsts.PANEL_PROJECT)
     project_panel_menu_item.set_submenu(project_panel_menu)
     
-    # Media Panel - we're forcing a position for this on two window mode for time being.
+    # Media Panel - we're forcing a position for this on two window mode for the time being.
     if editorpersistance.prefs.global_layout == appconsts.SINGLE_WINDOW:
         media_panel_menu_item = Gtk.MenuItem(_("Media Panel"))
         panel_positions_menu.append(media_panel_menu_item)
@@ -385,19 +432,12 @@ def get_panel_positions_menu_item():
     range_log_panel_menu_item.set_submenu(range_log_panel_menu)
     
     # Filter Panel
-    filter_panel_menu_item = Gtk.MenuItem(_("Filter Panel"))
+    filter_panel_menu_item = Gtk.MenuItem(_("Edit Panel"))
     panel_positions_menu.append(filter_panel_menu_item)
 
-    filter_panel_menu =  _get_position_selection_menu(appconsts.PANEL_FILTERS)
+    filter_panel_menu =  _get_position_selection_menu(appconsts.PANEL_MULTI_EDIT)
     filter_panel_menu_item.set_submenu(filter_panel_menu)
 
-    # Compositors Panel
-    compositors_panel_menu_item = Gtk.MenuItem(_("Compositors Panel"))
-    panel_positions_menu.append(compositors_panel_menu_item)
-
-    compositors_panel_menu =  _get_position_selection_menu(appconsts.PANEL_COMPOSITORS)
-    compositors_panel_menu_item.set_submenu(compositors_panel_menu)
-
     # Jobs
     jobs_panel_menu_item = Gtk.MenuItem(_("Jobs Panel"))
     panel_positions_menu.append(jobs_panel_menu_item)
@@ -440,7 +480,7 @@ def _get_position_selection_menu(panel_id):
             menu_item = Gtk.RadioMenuItem.new_with_label([first_item], _positions_names[pos_option])
             positions_menu.append(menu_item)
             menu_items.append(menu_item)
-                
+
     selected_index = available_positions.index(current_position)
     menu_items[selected_index].set_active(True)
     
@@ -482,11 +522,98 @@ def get_tabs_menu_item():
         down_item.connect("activate", _change_tabs_pos, position, DOWN)
         
     return tabs_menu_item
-        
+
+def show_layout_press_menu(launcher, widget, event):
+    guipopover.layout_menu_show(launcher, widget, _top_bar_menu_item_activated_popover)
+
+
+def _create_layout_presets_menu(menu):
+    callback = _top_bar_menu_item_activated
+
+    menu_item = guiutils.get_menu_item(_("Layout Monitor Left"), callback, "monitor_left")
+    menu.add(menu_item)
+
+    menu_item = guiutils.get_menu_item(_("Layout Monitor Center"), callback, "monitor_center")
+    menu.add(menu_item)
+
+    menu_item = guiutils.get_menu_item(_("Layout Top Row 4 Panels"), callback, "top_row_four")
+    menu.add(menu_item)
+    if editorstate.SCREEN_WIDTH < 1919:
+        menu_item.set_sensitive(False)
+
+    menu_item = guiutils.get_menu_item(_("Layout Media Panel Left Column"), callback, "media_panel_left")
+    menu.add(menu_item)
+    
+    guiutils.add_separetor(menu)
+    
+    menu_item = guiutils.get_menu_item(_("Save Current Layout..."), callback, "save_layout")
+    menu.add(menu_item)
+
+    menu_item = guiutils.get_menu_item(_("Load Layout..."), callback, "load_layout")
+    menu.add(menu_item)
+
+    menu.show_all()
+
+def _top_bar_menu_item_activated_popover(action, variant, msg):
+    _top_bar_menu_item_activated(None, msg)
+
+def _top_bar_menu_item_activated(widget, msg):
+    if msg == "monitor_center":
+        apply_layout(MONITOR_CENTER_PANEL_POSITIONS)
+    elif msg == "monitor_left":
+         apply_layout(DEFAULT_PANEL_POSITIONS)
+    elif msg == "top_row_four":
+         apply_layout(TOP_ROW_FOUR_POSITIONS)
+    elif msg == "media_panel_left":
+         apply_layout(MEDIA_PANEL_LEFT_POSITIONS)
+    elif msg == "save_layout":
+        data = (editorpersistance.prefs.panel_positions, editorpersistance.prefs.positions_tabs)
+        dialogs.save_layout_data(_save_layout_callback, data)
+    elif msg == "load_layout":
+        dialogs.load_layout_data(_load_layout_callback)
+
+def _save_layout_callback(dialog, response_id, data):
+    if response_id == Gtk.ResponseType.ACCEPT:
+        save_path = dialog.get_filenames()[0]
+        with atomicfile.AtomicFileWriter(save_path, "wb") as afw:
+            write_file = afw.get_file()
+            pickle.dump(data, write_file)
+            
+    dialog.destroy()
+
+def _load_layout_callback(dialog, response_id):
+    if response_id == Gtk.ResponseType.ACCEPT:
+        try:
+            load_path = dialog.get_filenames()[0]
+            layout_data = utils.unpickle(load_path)
+            panel_positions, positions_tabs = layout_data
+            apply_layout(panel_positions)
+            editorpersistance.prefs.panel_positions = panel_positions
+            editorpersistance.prefs.positions_tabs = positions_tabs
+            editorpersistance.save()
+            apply_tabs_positions()
+        except Exception as e:
+            apply_layout(DEFAULT_PANEL_POSITIONS)
+            editorpersistance.save()
+            dialogutils.info_message(_("Loading Layout Filed!"), "Exception message: " + str(e), gui.editor_window.window)
+                    
+    dialog.destroy()
+    
 # ----------------------------------------------- CHANGING POSITIONS
 def _change_panel_position(widget, panel_id, pos_option):
-    if widget.get_active() == False:
+    # We're again getting one event for activated item and one for the de-activated item.
+    if widget != None and widget.get_active() == False:
         return
+    
+    # Refuse to make default notebook empty.
+    panels_in_default_notebook = 0
+    for pos in _panel_positions.values():
+        if pos == appconsts.PANEL_PLACEMENT_TOP_ROW_DEFAULT:
+            panels_in_default_notebook += 1
+    if panels_in_default_notebook == 1 and \
+       _panel_positions[panel_id] == appconsts.PANEL_PLACEMENT_TOP_ROW_DEFAULT:
+       dialogs.refuse_to_empty_default_notebook_dialog()
+       return
 
     # Remove panel if it currently has position in layout.
     if _panel_positions[panel_id] != appconsts.PANEL_PLACEMENT_NOT_VISIBLE:
@@ -498,20 +625,49 @@ def _change_panel_position(widget, panel_id, pos_option):
     else:
         _panel_positions[panel_id] = pos_option
 
-    # If bottom row items do not fit, drop some buttons in middlebar.
+    # Show layout immediately if this called to change single position,
+    # applying layout changes multiple positions.
+    if widget != None:
+        gui.editor_window.window.show_all()
+        set_positions_frames_visibility()
+        GLib.timeout_add(100, _check_dimensions)
+        
+def _check_dimensions():
+    top_row_min_width = get_top_row_minimum_width()
     bottom_row_min_width = get_bottom_row_minimum_width()
-    if bottom_row_min_width > editorstate.SCREEN_WIDTH:
-        editorpersistance.prefs.force_small_midbar = True
-        editorpersistance.save()
-        middlebar.do_layout_after_dock_change(gui.editor_window)
-    else:
-        editorpersistance.prefs.force_small_midbar = False
-        editorpersistance.save()
-        middlebar.do_layout_after_dock_change(gui.editor_window)
-    
-    gui.editor_window.window.show_all()
-    set_positions_frames_visibility()
 
+    if top_row_min_width > editorstate.SCREEN_WIDTH or bottom_row_min_width > editorstate.SCREEN_WIDTH:
+        primary_txt = _("Custom layout does not fit on screen!") 
+        
+        info_text = _("Available screen dimensions: ") + str(editorstate.SCREEN_WIDTH) + " x " +  str(editorstate.SCREEN_HEIGHT) + "\n\n"
+        if top_row_min_width > editorstate.SCREEN_WIDTH:
+            info_text = info_text + _("Top row width:        ") + str(top_row_min_width) + "\n"
+        if bottom_row_min_width > editorstate.SCREEN_WIDTH:
+            info_text = info_text + _("Bottom row width:     ") + str(bottom_row_min_width) + "\n\n"
+        
+        info_text = info_text + _("Change panels positions or remove buttons from midbar\nto make layout fit on screen.")
+
+        dialogutils.info_message(primary_txt, info_text, gui.editor_window.window)
+
+    return  False
+
+def get_top_row_minimum_width():
+    player_buttons_w, h = gui.editor_window.player_buttons_row.get_preferred_width()
+    top_default_w, h = _get_position_frames_dict()[appconsts.PANEL_PLACEMENT_TOP_ROW_DEFAULT].get_preferred_width()
+    top_right_w, h = _get_position_frames_dict()[appconsts.PANEL_PLACEMENT_TOP_ROW_RIGHT].get_preferred_width()
+    left_column_w, h = _get_position_frames_dict()[appconsts.PANEL_PLACEMENT_LEFT_COLUMN].get_preferred_width() 
+    top_left_w, h = _get_position_frames_dict()[appconsts.PANEL_PLACEMENT_TOP_ROW_PROJECT_DEFAULT].get_preferred_width() 
+    combined = top_default_w + top_right_w + left_column_w + top_left_w + player_buttons_w
+    return combined
+
+def get_bottom_row_minimum_width():
+    middle_bar_w, h = gui.editor_window.edit_buttons_row.get_preferred_width()
+    bottom_left_w, h = _get_position_frames_dict()[appconsts.PANEL_PLACEMENT_BOTTOM_ROW_LEFT].get_preferred_width()
+    bottom_right_w, h = _get_position_frames_dict()[appconsts.PANEL_PLACEMENT_BOTTOM_ROW_RIGHT].get_preferred_width()
+    left_column_w, h = _get_position_frames_dict()[appconsts.PANEL_PLACEMENT_LEFT_COLUMN].get_preferred_width() 
+    combined = middle_bar_w + bottom_left_w + bottom_right_w + left_column_w
+    return combined
+    
 def _remove_panel(panel_id):
     current_position = _panel_positions[panel_id]
     panel_widgets = _get_panels_widgets_dict(gui.editor_window)
@@ -588,5 +744,16 @@ def apply_tabs_positions():
             else:
                 notebook.set_tab_pos(Gtk.PositionType.BOTTOM)
 
+def apply_layout(layout_dict):
+    for panel_id, layout_position in layout_dict.items():
+        try:
+            current_position = _panel_positions[panel_id]
+            if current_position != layout_position:
+                _change_panel_position(None, panel_id, layout_position)
+        except KeyError:
+            pass # Not all panels are part of current layout
+            
+    gui.editor_window.window.show_all()
+    set_positions_frames_visibility()
 
 
