@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2012 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -51,6 +51,9 @@ MULTI_TRIM = 15
 SDL_1 = 1
 SDL_2 = 2
 
+# Gtk.Application object for main app. 
+app = None
+
 # Project being edited
 project = None
 
@@ -96,7 +99,7 @@ SCREEN_WIDTH = -1
 # Runtime environment data
 gtk_version = None
 mlt_version = None
-appversion = "2.10.0"
+appversion = "2.12.0"
 RUNNING_FROM_INSTALLATION = 0
 RUNNING_FROM_DEV_VERSION = 1
 RUNNING_FROM_FLATPAK = 2
@@ -139,6 +142,9 @@ steal_frames = True
 # Timeline rendering
 tline_render_mode = appconsts.TLINE_RENDERING_OFF
 
+# Timeline view mode
+tline_view_mode = 0 # sequence.PROGRAM_OUT_MODE
+
 # Trim clips cache for quicker inits, path -> clip
 _trim_clips_cache = {}
 
@@ -175,6 +181,9 @@ def current_tline_frame():
     else:
         return tline_shadow_frame
 
+def APP():
+    return app
+
 def PROJECT():
     return project
     
