@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2014 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -192,6 +192,7 @@ class ColorBox:
         self.saturation = 0.0
         self.draw_saturation_gradient = True
         self.selection_cursor = SELECT_CIRCLE
+        self.motion_on = False
 
     def get_hue_saturation(self):
         return (self.hue, self.saturation)
@@ -216,8 +217,11 @@ class ColorBox:
         self._save_values()
         self.edit_listener()
         self.widget.queue_draw()
+        self.motion_on = True
 
     def _motion_notify_event(self, x, y, state):
+        if self.motion_on == False:
+            return
         self.cursor_x, self.cursor_y = self._get_legal_point(x, y)
         self._save_values()
         self.edit_listener()
@@ -228,7 +232,8 @@ class ColorBox:
         self._save_values()
         self.edit_listener()
         self.widget.queue_draw()
-
+        self.motion_on = False
+        
     def _get_legal_point(self, x, y):
         if x < self.X_PAD:
             x = self.X_PAD
@@ -297,6 +302,7 @@ class ThreeBandColorBox(ColorBox):
         self.hi_x = self.cursor_x
         self.hi_y = self.cursor_y
         self.band_change_listerner = band_change_listerner
+        self.motion_on = False
 
     def set_cursors(self, s_h, s_s, m_h, m_s, h_h, h_s):
         self.shadow_x = self._x_for_hue(s_h)
@@ -315,19 +321,25 @@ class ThreeBandColorBox(ColorBox):
         self._save_values()
         self.edit_listener()
         self.widget.queue_draw()
-
+        self.motion_on = True
+        
     def _motion_notify_event(self, x, y, state):
+        if self.motion_on == False:
+            return
         self.cursor_x, self.cursor_y = self._get_legal_point(x, y)
         self._save_values()
         self.edit_listener()
         self.widget.queue_draw()
 
     def _release_event(self, event):
+        if self.motion_on == False:
+            return
         self.cursor_x, self.cursor_y = self._get_legal_point(event.x, event.y)
         self._save_values()
         self.edit_listener()
         self.widget.queue_draw()
-
+        self.motion_on = False
+        
     def _check_band_hit(self, x, y):
         if self._control_point_hit(x, y, self.shadow_x, self.shadow_y):
             return SHADOW
@@ -415,12 +427,12 @@ class ColorBoxFilterEditor:
         self.SAT_MAX = 0.5
         self.widget = Gtk.VBox()
 
-        self.hue = filter(lambda ep: ep.name == "hue", editable_properties)[0]
-        self.saturation = filter(lambda ep: ep.name == "saturation", editable_properties)[0]
+        self.hue = [ep for ep in editable_properties if ep.name == "hue"][0]
+        self.saturation = [ep for ep in editable_properties if ep.name == "saturation"][0]
 
-        self.R = filter(lambda ep: ep.name == "R", editable_properties)[0]
-        self.G = filter(lambda ep: ep.name == "G", editable_properties)[0]
-        self.B = filter(lambda ep: ep.name == "B", editable_properties)[0]
+        self.R = [ep for ep in editable_properties if ep.name == "R"][0]
+        self.G = [ep for ep in editable_properties if ep.name == "G"][0]
+        self.B = [ep for ep in editable_properties if ep.name == "B"][0]
 
         self.color_box = ColorBox(self.color_box_values_changed)
         self.color_box.set_cursor(self.hue.get_float_value(), self.saturation.get_float_value())
@@ -462,7 +474,7 @@ class ColorBoxFilterEditor:
 
     def _display_values(self, hue, saturation):
         sat_str = str(int(saturation * 100)) + "%"
-        hue_str = unicode(int(360 * hue)) + ColorGrader.DEGREE_CHAR + u' '
+        hue_str = str(int(360 * hue)) + ColorGrader.DEGREE_CHAR + ' '
         self.h_label.set_text(hue_str)
         self.s_label.set_text(sat_str)
 
@@ -474,23 +486,23 @@ class ColorLGGFilterEditor:
         self.widget = Gtk.VBox()
 
         # Get MLT properties
-        self.lift_r = filter(lambda ep: ep.name == "lift_r", editable_properties)[0]
-        self.lift_g = filter(lambda ep: ep.name == "lift_g", editable_properties)[0]
-        self.lift_b = filter(lambda ep: ep.name == "lift_b", editable_properties)[0]
-        self.gamma_r = filter(lambda ep: ep.name == "gamma_r", editable_properties)[0]
-        self.gamma_g = filter(lambda ep: ep.name == "gamma_g", editable_properties)[0]
-        self.gamma_b = filter(lambda ep: ep.name == "gamma_b", editable_properties)[0]
-        self.gain_r = filter(lambda ep: ep.name == "gain_r", editable_properties)[0]
-        self.gain_g = filter(lambda ep: ep.name == "gain_g", editable_properties)[0]
-        self.gain_b = filter(lambda ep: ep.name == "gain_b", editable_properties)[0]
+        self.lift_r = [ep for ep in editable_properties if ep.name == "lift_r"][0]
+        self.lift_g = [ep for ep in editable_properties if ep.name == "lift_g"][0]
+        self.lift_b = [ep for ep in editable_properties if ep.name == "lift_b"][0]
+        self.gamma_r = [ep for ep in editable_properties if ep.name == "gamma_r"][0]
+        self.gamma_g = [ep for ep in editable_properties if ep.name == "gamma_g"][0]
+        self.gamma_b = [ep for ep in editable_properties if ep.name == "gamma_b"][0]
+        self.gain_r = [ep for ep in editable_properties if ep.name == "gain_r"][0]
+        self.gain_g = [ep for ep in editable_properties if ep.name == "gain_g"][0]
+        self.gain_b = [ep for ep in editable_properties if ep.name == "gain_b"][0]
 
         # Get Non-MLT properties
-        self.lift_hue = filter(lambda ep: ep.name == "lift_hue", editable_properties)[0]
-        self.lift_value = filter(lambda ep: ep.name == "lift_value", editable_properties)[0]
-        self.gamma_hue = filter(lambda ep: ep.name == "gamma_hue", editable_properties)[0]
-        self.gamma_value = filter(lambda ep: ep.name == "gamma_value", editable_properties)[0]
-        self.gain_hue = filter(lambda ep: ep.name == "gain_hue", editable_properties)[0]
-        self.gain_value = filter(lambda ep: ep.name == "gain_value", editable_properties)[0]
+        self.lift_hue = [ep for ep in editable_properties if ep.name == "lift_hue"][0]
+        self.lift_value = [ep for ep in editable_properties if ep.name == "lift_value"][0]
+        self.gamma_hue = [ep for ep in editable_properties if ep.name == "gamma_hue"][0]
+        self.gamma_value = [ep for ep in editable_properties if ep.name == "gamma_value"][0]
+        self.gain_hue = [ep for ep in editable_properties if ep.name == "gain_hue"][0]
+        self.gain_value = [ep for ep in editable_properties if ep.name == "gain_value"][0]
 
         # Lift editor
         self.lift_hue_selector = self.get_hue_selector(self.lift_hue_edited)
@@ -600,7 +612,7 @@ class ColorLGGFilterEditor:
         self.gain_adjustment.set_value(val)
 
     def set_hue_label_value(self, hue, label):
-        hue_str = unicode(int(360 * hue)) + ColorGrader.DEGREE_CHAR + u' '
+        hue_str = str(int(360 * hue)) + ColorGrader.DEGREE_CHAR + ' '
         label.set_text(hue_str)
 
     # ------------------------------ color box listeners
@@ -687,7 +699,7 @@ class BoxEditor:
     def __init__(self, pix_size):
         self.value_size = 1.0 # Box editor works in 0-1 normalized space
 
-        self.pix_size = pix_size;
+        self.pix_size = pix_size
         self.pix_per_val = self.value_size / pix_size
         self.off_x = 0.5
         self.off_y = 0.5
@@ -717,11 +729,6 @@ class BoxEditor:
     def draw_box(self, cr, allocation):
         x, y, w, h = allocation
 
-        if editorpersistance.prefs.theme == appconsts.LIGHT_THEME:
-            cr.set_source_rgb(*BOX_BG_COLOR )
-            cr.rectangle(0, 0, self.pix_size + 1, self.pix_size + 1)
-            cr.fill()
-
         # value lines
         cr.set_source_rgb(*BOX_LINE_COLOR)
         step = self.pix_size / 8
@@ -846,7 +853,7 @@ class CatmullRomFilterEditor:
         elif self.current_edit_curve == CatmullRomFilterEditor.G:
             self.cr_filter.g_points_prop.write_property_value(points_str)
 
-        else: # CatmullRomFilterEditor.G
+        else: # CatmullRomFilterEditor.B
             self.cr_filter.b_points_prop.write_property_value(points_str)
 
         self.cr_filter.update_table_property_values()
@@ -871,10 +878,9 @@ class CurvesBoxEditor(BoxEditor):
         self.last_point = None
         self.edit_on = False
 
-        if editorpersistance.prefs.theme != appconsts.LIGHT_THEME:
-            BOX_LINE_COLOR = (0.8, 0.8, 0.8)
-            CURVE_COLOR = (0.8, 0.8, 0.8)
-            self.curve_color = CURVE_COLOR
+        BOX_LINE_COLOR = (0.8, 0.8, 0.8)
+        CURVE_COLOR = (0.8, 0.8, 0.8)
+        self.curve_color = CURVE_COLOR
 
     def set_curve(self, curve, curve_color):
         self.curve = curve
@@ -907,7 +913,7 @@ class CurvesBoxEditor(BoxEditor):
         if self.edit_on == False:
             return
         vx, vy = BoxEditor.get_box_val_point(self, event.x, event.y)
-        p = lutfilter.CurvePoint(int(round(vx * 255)),int(round(vy * 255)))
+        p = lutfilter.CurvePoint(int(round(vx * 255)), int(round(vy * 255)))
         self.curve.remove_range(self.last_point.x, p.x)
         self.curve.set_curve_point(p)
 
@@ -944,7 +950,7 @@ class CurvesBoxEditor(BoxEditor):
 
 class ColorGrader:
 
-    DEGREE_CHAR = u'\u00B0'
+    DEGREE_CHAR = '\u00B0'
 
     def __init__(self, editable_properties):
         # Initial active band
@@ -960,12 +966,12 @@ class ColorGrader:
         # - negative correction is interpreted as positive correction of complimentary color
 
         # Editable properties
-        self.shadow_hue = filter(lambda ep: ep.name == "shadow_hue", editable_properties)[0]
-        self.shadow_saturation = filter(lambda ep: ep.name == "shadow_saturation", editable_properties)[0]
-        self.mid_hue = filter(lambda ep: ep.name == "mid_hue", editable_properties)[0]
-        self.mid_saturation = filter(lambda ep: ep.name == "mid_saturation", editable_properties)[0]
-        self.hi_hue = filter(lambda ep: ep.name == "hi_hue", editable_properties)[0]
-        self.hi_saturation = filter(lambda ep: ep.name == "hi_saturation", editable_properties)[0]
+        self.shadow_hue = [ep for ep in editable_properties if ep.name == "shadow_hue"][0]
+        self.shadow_saturation = [ep for ep in editable_properties if ep.name == "shadow_saturation"][0]
+        self.mid_hue = [ep for ep in editable_properties if ep.name == "mid_hue"][0]
+        self.mid_saturation = [ep for ep in editable_properties if ep.name == "mid_saturation"][0]
+        self.hi_hue = [ep for ep in editable_properties if ep.name == "hi_hue"][0]
+        self.hi_saturation = [ep for ep in editable_properties if ep.name == "hi_saturation"][0]
 
         # Create filter and init values
         self.filt = lutfilter.ColorGradeFilter(editable_properties)
@@ -1070,7 +1076,7 @@ class ColorGrader:
 
     def _display_values(self, band, hue, saturation):
         sat_str = str(int(((saturation - 0.5) * 2.0) * 100)) + "%"
-        hue_str = unicode(int(360 * hue)) + ColorGrader.DEGREE_CHAR + u' '
+        hue_str = str(int(360 * hue)) + ColorGrader.DEGREE_CHAR + ' '
         if band == SHADOW:
             self.sh_label.set_text(hue_str)
             self.ss_label.set_text(sat_str)
