@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2012 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -49,7 +49,7 @@ CORNER_DIVIDER = 5
 
 # Aug-2019 - SvdB - BB
 MB_BUTTONS_WIDTH = [317,634]
-MB_BUTTONS_HEIGHT = [30,60]
+MB_BUTTONS_HEIGHT = [23,60]
 MB_BUTTON_HEIGHT = [22,44]
 MB_BUTTON_WIDTH = [30,60]
 MB_BUTTON_Y = 4
@@ -93,8 +93,11 @@ class AbstractGlassButtons:
         self.icons = []
         self.image_x = []
         self.image_y = []
-        self.sensitive = []
-
+        self.sensitive = [] # not used currently
+        
+        self.prelight_icons = []
+        self.prelight_index = -1
+        
         if editorpersistance.prefs.buttons_style == editorpersistance.GLASS_STYLE:
             self.glass_style = True
         else:
@@ -104,9 +107,9 @@ class AbstractGlassButtons:
 
         # Dark theme comes with flat buttons
         self.dark_theme = False
-        if editorpersistance.prefs.theme != appconsts.LIGHT_THEME:
-            self.glass_style = False
-            self.dark_theme = True
+
+        self.glass_style = False
+        self.dark_theme = True
 
         self.draw_button_gradients = True # old code artifact, remove (set False at object creation site to kill all gradients)
 
@@ -142,6 +145,12 @@ class AbstractGlassButtons:
     def _release_event(self, event):
         print("_release_event not impl")
 
+    def _leave_notify_event(self, event):
+        print("_leave_notify_event not impl")
+        
+    def _enter_notify_event(self, event):
+        print("_enter_notify_event not impl")
+        
     def _draw(self, event, cr, allocation):
         print("_draw not impl")
 
@@ -164,6 +173,8 @@ class AbstractGlassButtons:
             x = self.button_x
             for i in range(0, len(self.icons)):
                 icon = self.icons[i]
+                if self.prelight_index == i:
+                    icon = self.prelight_icons[i]
                 cr.set_source_surface(icon, x + self.image_x[i], self.image_y[i])
                 cr.paint()
                 x += self.button_width
@@ -267,6 +278,24 @@ class AbstractGlassButtons:
                 cr.stroke()
             x += self.button_width
 
+    def show_prelight_icons(self):
+        for icon in self.icons:
+            surface_prelight = cairo.ImageSurface(cairo.FORMAT_ARGB32, icon.get_width(), icon.get_height())
+            cr = cairo.Context(surface_prelight)
+            cr.set_source_surface(icon, 0, 0)
+            cr.rectangle(0, 0, icon.get_width(), icon.get_height())
+            cr.fill()
+            
+            cr.set_operator(cairo.Operator.ATOP)
+            cr.set_source_rgba(1.0, 1.0, 1.0, 0.5)
+            cr.rectangle(0, 0, icon.get_width(), icon.get_height())
+            cr.fill()
+            
+            self.prelight_icons.append(surface_prelight)
+
+        self.widget.leave_notify_func = self._leave_notify_event
+        self.widget.enter_notify_func = self._enter_notify_event
+
 
 class PlayerButtons(AbstractGlassButtons):
 
@@ -275,26 +304,25 @@ class PlayerButtons(AbstractGlassButtons):
         size_ind = 0
         size_adj = 1
         prefs = editorpersistance.prefs
-        # Colorized icons
-        if prefs.colorized_icons is True:
-            icon_color = "_color"
-        else:
-            icon_color = ""
+
         # End of Colorized icons
         if prefs.double_track_hights:
            size_ind = 1
            size_adj = 2
-        AbstractGlassButtons.__init__(self, MB_BUTTON_WIDTH[size_ind], MB_BUTTON_HEIGHT[size_ind], MB_BUTTON_Y, MB_BUTTONS_WIDTH[size_ind], MB_BUTTONS_HEIGHT[size_ind])
+        AbstractGlassButtons.__init__(self, MB_BUTTON_WIDTH[size_ind], MB_BUTTON_HEIGHT[size_ind], MB_BUTTON_Y, MB_BUTTONS_WIDTH[size_ind], MB_BUTTONS_HEIGHT[size_ind] - 2)
 
-        play_pause_icon = guiutils.get_cairo_image("play_pause_s" + icon_color)
-        play_icon = guiutils.get_cairo_image("play_2_s"+ icon_color)
-        stop_icon = guiutils.get_cairo_image("stop_s"+ icon_color)
+        # Force no decorations for player buttons, this cannot be made to work.
+        self.no_decorations = True 
+
+        play_pause_icon = guiutils.get_cairo_image("play_pause_s")
+        play_icon = guiutils.get_cairo_image("play_2_s")
+        stop_icon = guiutils.get_cairo_image("stop_s")
         next_icon = guiutils.get_cairo_image("next_frame_s")
         prev_icon = guiutils.get_cairo_image("prev_frame_s")
-            # ------------------------------timeline_start_end_button
+        # ------------------------------timeline_start_end_button
         start_icon = guiutils.get_cairo_image("to_start") #  go to start
         end_icon = guiutils.get_cairo_image("to_end") #  go to end
-            # ------------------------------timeline_start_end_button
+        # ------------------------------timeline_start_end_button
         mark_in_icon = guiutils.get_cairo_image("mark_in_s")
         mark_out_icon = guiutils.get_cairo_image("mark_out_s")
         marks_clear_icon = guiutils.get_cairo_image("marks_clear_s")
@@ -322,39 +350,24 @@ class PlayerButtons(AbstractGlassButtons):
                               mark_in_icon, mark_out_icon,
                               marks_clear_icon, to_mark_in_icon, to_mark_out_icon]
                 #  go to start end add 5*size_adj, 5*size_adj,
-                self.image_x = [5*size_adj, 5*size_adj, 5*size_adj, 7*size_adj, 5*size_adj, 10*size_adj, 3*size_adj, 11*size_adj, 2*size_adj, 7*size_adj, 6*size_adj]
+                self.image_x = [7*size_adj, 7*size_adj, 5*size_adj, 7*size_adj, 20*size_adj, 10*size_adj, 0*size_adj, 6*size_adj, 2*size_adj, 10*size_adj, 4*size_adj]
             else:
                 self.icons = [prev_icon, next_icon, play_icon, stop_icon,
                               mark_in_icon, mark_out_icon,
                               marks_clear_icon, to_mark_in_icon, to_mark_out_icon]
-                self.image_x = [5*size_adj, 7*size_adj, 20*size_adj, 10*size_adj, 3*size_adj, 11*size_adj, 2*size_adj, 7*size_adj, 6*size_adj]
+                self.image_x = [5*size_adj, 7*size_adj, 20*size_adj, 10*size_adj, 0*size_adj, 6*size_adj, 2*size_adj, 10*size_adj, 4*size_adj]
             # ------------------------------End of timeline_start_end_button
-#            self.icons = [prev_icon, next_icon, play_pause_icon,
-#                          mark_in_icon, mark_out_icon,
-#                          marks_clear_icon, to_mark_in_icon, to_mark_out_icon]
-#            self.image_x = [5*size_adj, 7*size_adj, 5*size_adj, 3*size_adj, 11*size_adj, 2*size_adj, 7*size_adj, 6*size_adj]
-#        else:
-#            self.icons = [prev_icon, next_icon, play_icon, stop_icon,
-#                          mark_in_icon, mark_out_icon,
-#                          marks_clear_icon, to_mark_in_icon, to_mark_out_icon]
-#            self.image_x = [5*size_adj, 7*size_adj, 10*size_adj, 10*size_adj, 3*size_adj, 11*size_adj, 2*size_adj, 7*size_adj, 6*size_adj]
 
         for i in range(0, len(self.icons)):
-            self.image_y.append(MB_BUTTON_IMAGE_Y)
-
+            self.image_y.append(MB_BUTTON_IMAGE_Y - 6)
+        
         self.pressed_callback_funcs = None # set using set_callbacks()
 
         self.set_sensitive(True)
 
         focus_groups[DEFAULT_FOCUS_GROUP].append(self.widget)
 
-    def set_trim_sensitive_pattern(self):
-        # Jul-2016 - SvdB - For play/pause button
-        if (editorpersistance.prefs.play_pause == True):
-            self.sensitive = [True, True, True, False, False, False, False, False]
-        else:
-            self.sensitive = [True, True, True, True, False, False, False, False, False]
-        self.widget.queue_draw()
+        self.show_prelight_icons()
 
     def set_normal_sensitive_pattern(self):
         self.set_sensitive(True)
@@ -378,6 +391,10 @@ class PlayerButtons(AbstractGlassButtons):
         button_under = self._get_hit_code(x, y)
         if self.pressed_button != button_under: # pressed button is released
             self.pressed_button = NO_HIT
+
+        if len(self.prelight_icons) > 0:
+            self.prelight_index = button_under
+            
         self.widget.queue_draw()
 
     def _release_event(self, event):
@@ -387,6 +404,13 @@ class PlayerButtons(AbstractGlassButtons):
         self.pressed_button = -1
         self.widget.queue_draw()
 
+    def _leave_notify_event(self, event):
+        self.prelight_index = -1
+        self.widget.queue_draw()
+
+    def _enter_notify_event(self, event):
+        self.prelight_index = -1
+        
     def set_callbacks(self, pressed_callback_funcs):
         self.pressed_callback_funcs = pressed_callback_funcs
 
@@ -395,28 +419,34 @@ class PlayerButtons(AbstractGlassButtons):
         x, y, w, h = allocation
         self.allocation = allocation
 
-        mid_x = w / 2
+        mid_x = w // 2
         buttons_width = self.button_width * len(self.icons)
         # Jul-2016 - SvdB - No changes made here, but because of the calculation of button_x the row of buttons is slightly moved right if play/pause
         # is enabled. This could be solved by setting self.button_x = 1, if wished.
-        self.button_x = mid_x - (buttons_width / 2)
+        self.button_x = mid_x - (buttons_width // 2)
         self._draw_buttons(cr, w, h)
 
 
 class GmicButtons(AbstractGlassButtons):
 
     def __init__(self):
+        size_ind = 0
+        size_adj = 1
+        prefs = editorpersistance.prefs
 
-        AbstractGlassButtons.__init__(self, MB_BUTTON_WIDTH[0], MB_BUTTON_HEIGHT[0], MB_BUTTON_Y, GMIC_BUTTONS_WIDTH, MB_BUTTONS_HEIGHT[0])
+        # NOT IMPLEMENTED
+        if prefs.double_track_hights:
+           size_ind = 1
+           size_adj = 2
+        AbstractGlassButtons.__init__(self, MB_BUTTON_WIDTH[size_ind], MB_BUTTON_HEIGHT[size_ind], MB_BUTTON_Y, MB_BUTTONS_WIDTH[size_ind], MB_BUTTONS_HEIGHT[size_ind] - 2)
 
-        IMG_PATH = respaths.IMAGE_PATH
-        next_icon = cairo.ImageSurface.create_from_png(IMG_PATH + "next_frame_s.png")
-        prev_icon = cairo.ImageSurface.create_from_png(IMG_PATH + "prev_frame_s.png")
-        mark_in_icon = cairo.ImageSurface.create_from_png(IMG_PATH + "mark_in_s.png")
-        mark_out_icon = cairo.ImageSurface.create_from_png(IMG_PATH + "mark_out_s.png")
-        marks_clear_icon = cairo.ImageSurface.create_from_png(IMG_PATH + "marks_clear_s.png")
-        to_mark_in_icon = cairo.ImageSurface.create_from_png(IMG_PATH + "to_mark_in_s.png")
-        to_mark_out_icon = cairo.ImageSurface.create_from_png(IMG_PATH + "to_mark_out_s.png")
+        next_icon = guiutils.get_cairo_image("next_frame_s")
+        prev_icon = guiutils.get_cairo_image("prev_frame_s")
+        mark_in_icon = guiutils.get_cairo_image("mark_in_s")
+        mark_out_icon = guiutils.get_cairo_image("mark_out_s")
+        marks_clear_icon = guiutils.get_cairo_image("marks_clear_s")
+        to_mark_in_icon = guiutils.get_cairo_image("to_mark_in_s")
+        to_mark_out_icon = guiutils.get_cairo_image("to_mark_out_s")
 
         self.icons = [prev_icon, next_icon, mark_in_icon, mark_out_icon,
                       marks_clear_icon, to_mark_in_icon, to_mark_out_icon]
@@ -425,12 +455,13 @@ class GmicButtons(AbstractGlassButtons):
         for i in range(0, len(self.icons)):
             self.image_y.append(MB_BUTTON_IMAGE_Y)
 
-        self.pressed_callback_funcs = None # set using set_callbacks()
+        self.pressed_callback_funcs = [] # set using set_callbacks()
 
         self.set_sensitive(True)
 
         focus_groups[DEFAULT_FOCUS_GROUP].append(self.widget)
-
+        
+        self.show_prelight_icons()
 
     def set_normal_sensitive_pattern(self):
         self.set_sensitive(True)
@@ -445,6 +476,7 @@ class GmicButtons(AbstractGlassButtons):
         if self.pressed_button >= 0 and self.pressed_button < len(self.icons):
             callback_func = self.pressed_callback_funcs[self.pressed_button] # index is set to match at editorwindow.py where callback func list is created
             callback_func()
+
         self.widget.queue_draw()
 
     def _motion_notify_event(self, x, y, state):
@@ -454,6 +486,10 @@ class GmicButtons(AbstractGlassButtons):
         button_under = self._get_hit_code(x, y)
         if self.pressed_button != button_under: # pressed button is released
             self.pressed_button = NO_HIT
+
+        if len(self.prelight_icons) > 0:
+            self.prelight_index = button_under
+            
         self.widget.queue_draw()
 
     def _release_event(self, event):
@@ -463,6 +499,13 @@ class GmicButtons(AbstractGlassButtons):
         self.pressed_button = -1
         self.widget.queue_draw()
 
+    def _leave_notify_event(self, event):
+        self.prelight_index = -1
+        self.widget.queue_draw()
+
+    def _enter_notify_event(self, event):
+        self.prelight_index = -1
+        
     def set_callbacks(self, pressed_callback_funcs):
         self.pressed_callback_funcs = pressed_callback_funcs
 
@@ -487,10 +530,13 @@ class GlassButtonsGroup(AbstractGlassButtons):
         self.image_y_default = image_y_default
         focus_groups[focus_group].append(self.widget)
 
-    def add_button(self, pix_buf, release_callback):
+    def add_button(self, pix_buf, release_callback, image_x=None):
+        if image_x == None:
+            image_x = self.image_x_default
+            
         self.icons.append(pix_buf)
         self.released_callback_funcs.append(release_callback)
-        self.image_x.append(self.image_x_default)
+        self.image_x.append(image_x)
         self.image_y.append(self.image_y_default)
         self.sensitive.append(True)
         self.widget.set_pref_size(len(self.icons) * self.button_width + 2, self.button_height + 2)
@@ -512,6 +558,10 @@ class GlassButtonsGroup(AbstractGlassButtons):
                 release_func = self.released_callback_funcs[self.pressed_button]
                 release_func()
             self.pressed_button = NO_HIT
+
+        if len(self.prelight_icons) > 0:
+            self.prelight_index = button_under
+                
         self.widget.queue_draw()
 
     def _release_event(self, event):
@@ -521,8 +571,14 @@ class GlassButtonsGroup(AbstractGlassButtons):
         self.pressed_button = -1
         self.widget.queue_draw()
 
+    def _leave_notify_event(self, event):
+        self.prelight_index = -1
+        self.widget.queue_draw()
 
+    def _enter_notify_event(self, event):
+        self.prelight_index = -1
 
+        
 class GlassButtonsToggleGroup(GlassButtonsGroup):
     def set_pressed_button(self, pressed_button_index, fire_clicked_cb=False):
         self.pressed_button = pressed_button_index
