@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2013 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 import datetime
 
 from gi.repository import Gtk
+from gi.repository import GLib
 from gi.repository import GObject
 from gi.repository import GdkPixbuf
 from gi.repository import Pango
@@ -29,8 +30,9 @@ import appconsts
 import dialogs
 import dnd
 import edit
-import gui
+import editorlayout
 import guicomponents
+import guipopover
 import guiutils
 import editorpersistance # Aug-2019 - SvdB - BB
 import editorstate
@@ -45,8 +47,7 @@ import utils
 widgets = utils.EmptyClass()
 
 do_multiple_clip_insert_func = None # this is monkeypathched here in app.py
-
-actions_popup_menu = Gtk.Menu()        
+log_changed_since_last_save = False
 
 # Sort order
 TIME_SORT = appconsts.TIME_SORT
@@ -54,9 +55,7 @@ NAME_SORT = appconsts.NAME_SORT
 COMMENT_SORT = appconsts.COMMENT_SORT
 
 sorting_order = TIME_SORT
-
-range_log_notebook_index = 1 # this is set 0 for 2 window mode
-
+_use_comments_for_name = False
 
 # ----------------------------------- log data object
 class MediaLogEvent:
@@ -87,11 +86,30 @@ class MediaLogEvent:
         date_str = date_str.lstrip('0')
         return date_str
 
+def _mark_log_changed():
+    global log_changed_since_last_save
+    log_changed_since_last_save = True
 
 # ----------------------------------------------------------- dnd drop
 def clips_drop(clips):
     for clip in clips:
         if clip.media_type == appconsts.VIDEO or clip.media_type == appconsts.AUDIO or clip.media_type == appconsts.IMAGE_SEQUENCE:
+            if PROJECT().proxy_data.proxy_mode == appconsts.USE_ORIGINAL_MEDIA:
+                clip_path = clip.path
+            else:
+                # We are in proxy mode, find out original media path
+                media_item = PROJECT().get_media_file_for_path(clip.path)
+                if media_item != None:
+                    # 'media_item.second_file_path' points now to original media
+                    # if proxy file exits.
+                    clip_path = media_item.second_file_path
+                    if clip_path == None:
+                        # no proxy for this clip, use media from dragged clip
+                        clip_path = clip.path
+                else:
+                    # no media item for this clip, use media from dragged clip
+                    clip_path = clip.path
+
             log_event = MediaLogEvent(  appconsts.MEDIA_LOG_MARKS_SET,
                                         clip.clip_in,
                                         clip.clip_out,
@@ -100,7 +118,7 @@ def clips_drop(clips):
             log_event.ttl = clip.ttl
             editorstate.PROJECT().media_log.append(log_event)
     _update_list_view(log_event)
-    
+    _mark_log_changed()
 
 # ----------------------------------------------------------- gui events
 def media_log_filtering_changed():
@@ -114,7 +132,8 @@ def media_log_star_button_pressed():
         log_events[index].starred = True
 
     widgets.media_log_view.fill_data_model()
-
+    _mark_log_changed()
+    
 def media_log_no_star_button_pressed():
     selected = widgets.media_log_view.get_selected_rows_list()
     log_events = get_current_filtered_events()
@@ -123,8 +142,13 @@ def media_log_no_star_button_pressed():
         log_events[index].starred = False
 
     widgets.media_log_view.fill_data_model()
-
+    _mark_log_changed()
+    
 def log_range_clicked():
+    if editorstate.timeline_visible() == True:
+        dialogs.no_timeline_ranges_dialog()
+        return
+
     media_file = editorstate.MONITOR_MEDIA_FILE()
     if media_file == None:
         return
@@ -133,30 +157,43 @@ def log_range_clicked():
        return 
     if media_file.mark_in == -1 or media_file.mark_out == -1:
         return
-
+    
+    file_path = media_file.path
+    if PROJECT().proxy_data.proxy_mode == appconsts.USE_PROXY_MEDIA:
+        if media_file.second_file_path != None:
+            file_path = media_file.second_file_path # proxy file exists
+        else:
+            file_path = media_file.path # no proxy file created
+            
     log_event = MediaLogEvent(  appconsts.MEDIA_LOG_MARKS_SET,
                                 media_file.mark_in,
                                 media_file.mark_out,
                                 media_file.name,
-                                media_file.path)
+                                file_path)
     log_event.ttl = media_file.ttl
 
     editorstate.PROJECT().media_log.append(log_event)
     editorstate.PROJECT().add_to_group(_get_current_group_index(), [log_event])
     _update_list_view(log_event)
-
+    _mark_log_changed()
+    
 def _update_list_view(log_event):
     widgets.media_log_view.fill_data_model()
-    max_val = widgets.media_log_view.treeview.get_vadjustment().get_upper()
-    gui.middle_notebook.set_current_page(range_log_notebook_index)
+    max_val = widgets.media_log_view.scroll.get_vadjustment().get_upper()
+    editorlayout.show_panel(appconsts.PANEL_RANGE_LOG)
     view_group = get_current_filtered_events()
     try:
         event_index = view_group.index(log_event)
         widgets.media_log_view.treeview.get_selection().select_path(str(event_index))
-        widgets.media_log_view.treeview.get_vadjustment().set_value(max_val)
     except:
         pass # if non-starred are not displayed currently. TODO: think of logic, should new items into displayed category?
-        
+
+    GLib.idle_add(_scroll_window, max_val)
+
+def _scroll_window(max_val):
+    widgets.media_log_view.scroll.get_vadjustment().set_value(max_val)
+    return False
+
 def log_item_name_edited(cell, path, new_text, user_data):
     if len(new_text) == 0:
         return
@@ -164,9 +201,11 @@ def log_item_name_edited(cell, path, new_text, user_data):
     item_index = int(path)
     current_view_events = get_current_filtered_events()
     current_view_events[item_index].comment = new_text
-
-    widgets.media_log_view.fill_data_model()
-
+    tree_path = Gtk.TreePath.new_from_indices([item_index])
+    iter = widgets.media_log_view.storemodel.get_iter(tree_path)
+    widgets.media_log_view.storemodel.set_value (iter, 1, new_text)
+    _mark_log_changed()
+        
 def delete_selected():
     selected = widgets.media_log_view.get_selected_rows_list()
     log_events = get_current_filtered_events()
@@ -174,6 +213,7 @@ def delete_selected():
     for row in selected:
         index = max(row) # these are tuple, max to extract only value
         delete_events.append(log_events[index])
+        
     current_group_index = _get_current_group_index()
     if current_group_index != -1: # When user created group is displayed item is only deleted from that group
         PROJECT().remove_from_group(current_group_index, delete_events)
@@ -183,7 +223,8 @@ def delete_selected():
         PROJECT().delete_media_log_events(delete_events)
 
     widgets.media_log_view.fill_data_model()
-
+    _mark_log_changed()
+    
 def display_item(row):
     log_events = get_current_filtered_events()
     event_item = log_events[row]
@@ -206,10 +247,11 @@ def log_list_view_button_press(treeview, event):
     selection.select_path(path)
     row = int(max(path))
 
-    guicomponents.display_media_log_event_popup_menu(row, treeview, _log_event_menu_item_selected, event)                                    
+    guipopover.media_log_event_popover_show(row, treeview, event.x, event.y, _log_event_menu_item_selected)
+    
     return True
 
-def _log_event_menu_item_selected(widget, data):
+def _log_event_menu_item_selected(action, variant, data):
     item_id, row, treeview = data
     
     if item_id == "delete":
@@ -223,6 +265,16 @@ def _log_event_menu_item_selected(widget, data):
     elif item_id == "renderslowmo":
         render_slowmo_from_item(row)
 
+def _do_range_log_drop_on_monitor(row):
+    display_item(row)
+
+def _use_comments_toggled(action, variant, msg):
+    new_state = not(action.get_state().get_boolean())
+    action.set_state(GLib.Variant.new_boolean(new_state))
+        
+    global _use_comments_for_name
+    _use_comments_for_name = new_state
+
 def render_slowmo_from_item(row):
     log_events = get_current_filtered_events()
     event_item = log_events[row]
@@ -284,17 +336,26 @@ def insert_selected_log_events():
     do_multiple_clip_insert_func(track, clips, tline_pos)
 
 def get_log_event_clip(log_event):
-    # pre versions 1.16 do not have this attr in log_event objects
+    # Versions before 1.16 do not have this attr in log_event objects
     if not hasattr(log_event, "ttl"):
         log_event.ttl = None
 
     # currently quaranteed not to be a pattern producer
-    new_clip = editorstate.current_sequence().create_file_producer_clip(log_event.path, None, False, log_event.ttl)
-        
+    if PROJECT().proxy_data.proxy_mode == appconsts.USE_ORIGINAL_MEDIA:
+        new_clip = editorstate.current_sequence().create_file_producer_clip(log_event.path, None, False, log_event.ttl)
+    else:
+        # We are in proxy mode, use proxy clip if available.
+        media_item = PROJECT().get_media_file_for_second_path(log_event.path) # 'log_event.path' is always original media,
+                                                                              # if we are in proxy mode we want to use proxy media if possible
+        if media_item != None:
+            new_clip = editorstate.current_sequence().create_file_producer_clip(media_item.path, None, False, log_event.ttl)
+        else:
+            new_clip = editorstate.current_sequence().create_file_producer_clip(log_event.path, None, False, log_event.ttl)
+            
     # Set clip in and out points
     new_clip.clip_in = log_event.mark_in
     new_clip.clip_out = log_event.mark_out
-    if widgets.use_comments_check.get_active() == True:
+    if _use_comments_for_name == True:
         new_clip.name = log_event.comment
         if len(new_clip.name) == 0:
             new_clip.name = log_event.name
@@ -314,82 +375,18 @@ def get_clips_for_rows(rows):
 def display_log_clip_double_click_listener(treeview, path, view_column):
     row = int(max(path))
     data = ("display", row, treeview)
-    _log_event_menu_item_selected(treeview, data)
+    _log_event_menu_item_selected(None, None, data)
 
-def _group_action_pressed(widget, event):
-    actions_menu = actions_popup_menu
-    guiutils.remove_children(actions_menu)      
-    actions_menu.add(guiutils.get_menu_item(_("New Group..."), _actions_callback, "new"))
-    actions_menu.add(guiutils.get_menu_item(_("New Group From Selected..."), _actions_callback, "newfromselected"))
-    
-    guiutils.add_separetor(actions_menu)
-
-    item = guiutils.get_menu_item(_("Rename Current Group..."), _actions_callback, "rename")
-    _unsensitive_for_all_view(item)
-    actions_menu.add(item)
-
-    guiutils.add_separetor(actions_menu)
-    
-    move_menu_item = Gtk.MenuItem(_("Move Selected Items To Group"))
-    move_menu = Gtk.Menu()
-    if len(PROJECT().media_log_groups) == 0:
-        move_menu.add(guiutils.get_menu_item(_("No Groups"), _actions_callback, "dummy", False))
-    else:
-        index = 0
-        for group in PROJECT().media_log_groups:
-            name, items = group
-            move_menu.add(guiutils.get_menu_item(name, _actions_callback, str(index)))
-            index = index + 1
-    move_menu_item.set_submenu(move_menu)
-    actions_menu.add(move_menu_item)
-    move_menu_item.show()
-
-    guiutils.add_separetor(actions_menu)
-
-    item = guiutils.get_menu_item(_("Delete Current Group"), _actions_callback, "delete")
-    _unsensitive_for_all_view(item)
-    actions_menu.add(item)
-
-    guiutils.add_separetor(actions_menu)
-    
-    sort_item = Gtk.MenuItem(_("Sort by"))
-    sort_menu = Gtk.Menu()
-    time_item = Gtk.RadioMenuItem()
-    time_item.set_label(_("Time"))
-    time_item.set_active(True)
-    time_item.show()
-    time_item.connect("activate", lambda w: _sorting_changed("time"))
-    sort_menu.append(time_item)
-
-    name_item = Gtk.RadioMenuItem.new_with_label([time_item], _("File Name"))
-    name_item.connect("activate", lambda w: _sorting_changed("name"))
-    name_item.show()
-    sort_menu.append(name_item)
-
-    comment_item = Gtk.RadioMenuItem.new_with_label([time_item], _("Comment"))
-    comment_item.connect("activate", lambda w: _sorting_changed("comment"))
-    comment_item.show()
-    sort_menu.append(comment_item)
-
-    global sorting_order
-    if sorting_order == TIME_SORT:
-        time_item.set_active(True)
-    elif sorting_order == NAME_SORT:
-        name_item.set_active(True)
-    else:# "comment"
-        comment_item.set_active(True)
-
-    sort_item.set_submenu(sort_menu)
-    sort_item.show()
-    actions_menu.add(sort_item)
-        
-    actions_menu.popup(None, None, None, None, event.button, event.time)
+def _group_action_pressed(launcher, widget, event, data):
+    guipopover.range_log_hamburger_menu_show(launcher, widget, widgets.group_view_select.get_active() == 0, sorting_order, \
+                                       _use_comments_for_name, PROJECT().media_log_groups,
+                                        _actions_callback, _sorting_callback, _use_comments_toggled)
 
 def _unsensitive_for_all_view(item):
     if widgets.group_view_select.get_active() == 0:
         item.set_sensitive(False)
 
-def _actions_callback(widget, data):
+def _actions_callback(action, variant, data):
     if data == "newfromselected":
         next_index = len(PROJECT().media_log_groups)
         dialogs.new_media_log_group_name_dialog(_new_group_name_callback, next_index, True)
@@ -432,6 +429,12 @@ def _actions_callback(widget, data):
         PROJECT().add_to_group(to_group_index, move_items)
         widgets.group_view_select.set_active(to_group_index + 1) # 0 index items is "All" items group not a user created group
 
+def _sorting_callback(action, variant):
+    action.set_state(variant)
+    print(variant.get_string())
+    _sorting_changed(variant.get_string())
+    guipopover._range_log_popover.hide()
+
 def _delete_with_items_dialog_callback(dialog, response_id):
     dialog.destroy()
     if response_id != Gtk.ResponseType.ACCEPT:
@@ -443,7 +446,8 @@ def _delete_with_items_dialog_callback(dialog, response_id):
     PROJECT().media_log_groups.pop(current_group_index)
     _create_group_select()
     widgets.group_view_select.set_active(0)
-    
+    _mark_log_changed()
+        
 def _rename_callback(dialog, response_id, entry):
     new_name = entry.get_text()
     dialog.destroy()
@@ -459,7 +463,8 @@ def _rename_callback(dialog, response_id, entry):
     PROJECT().media_log_groups.insert(current_group_index, (new_name, items))
     _create_group_select()
     widgets.group_view_select.set_active(current_group_index + 1)
-
+    _mark_log_changed()
+    
 def _viewed_group_changed(widget):
     update_media_log_view()
 
@@ -493,7 +498,8 @@ def _new_group_name_callback(dialog, response_id, data):
     _create_group_select()
     widgets.group_view_select.set_active(len(PROJECT().media_log_groups))
     update_media_log_view()
-
+    _mark_log_changed()
+    
 def _sorting_changed(msg):
     global sorting_order
     if msg == "time":
@@ -515,8 +521,8 @@ def get_media_log_list_view():
 def update_media_log_view():
     widgets.media_log_view.fill_data_model()
     # Does not show last line, do we need timer?
-    max_val = widgets.media_log_view.treeview.get_vadjustment().get_upper()
-    widgets.media_log_view.treeview.get_vadjustment().set_value(max_val)
+    max_val = widgets.media_log_view.scroll.get_vadjustment().get_upper()
+    widgets.media_log_view.scroll.get_vadjustment().set_value(max_val)
 
     
 class MediaLogListView(Gtk.VBox):
@@ -530,11 +536,9 @@ class MediaLogListView(Gtk.VBox):
         # Scroll container
         self.scroll = Gtk.ScrolledWindow()
         self.scroll.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
-        self.scroll.set_shadow_type(Gtk.ShadowType.ETCHED_IN)
 
         # View
-        self.treeview = Gtk.TreeView(self.storemodel)
-        self.treeview.set_property("rules_hint", True)
+        self.treeview = Gtk.TreeView(model=self.storemodel)
         self.treeview.set_headers_visible(True)
         tree_sel = self.treeview.get_selection()
         tree_sel.set_mode(Gtk.SelectionMode.MULTIPLE)
@@ -567,6 +571,7 @@ class MediaLogListView(Gtk.VBox):
         self.text_rend_2 = Gtk.CellRendererText()
         self.text_rend_2.set_property("yalign", 0.0)
         self.text_rend_2.set_property("editable", True)
+        self.text_rend_2.set_property("ellipsize", Pango.EllipsizeMode.END)
         self.text_rend_2.connect("edited", log_item_name_edited, (self.storemodel, 2))
                                  
         self.text_rend_3 = Gtk.CellRendererText()
@@ -590,7 +595,7 @@ class MediaLogListView(Gtk.VBox):
 
         self.text_col_2.set_expand(True)
         self.text_col_2.set_sizing(Gtk.TreeViewColumnSizing.GROW_ONLY)
-        self.text_col_2.set_min_width(150)
+        self.text_col_2.set_min_width(250)
         self.text_col_2.pack_start(self.text_rend_2, True)
         self.text_col_2.add_attribute(self.text_rend_2, "text", 1)
 
@@ -654,11 +659,13 @@ def get_media_log_events_panel(events_list_view):
     global widgets
 
     group_actions_menu = guicomponents.HamburgerPressLaunch(_group_action_pressed)
+    group_actions_menu.do_popover_callback = True
     guiutils.set_margins(group_actions_menu.widget, 10, 0, 2, 18)
 
     star_check = Gtk.CheckButton()
     star_check.set_active(True)
     star_check.connect("clicked", lambda w:media_log_filtering_changed())
+    star_check.set_margin_end(5)
     widgets.star_check = star_check
 
     star_label = Gtk.Image()
@@ -668,6 +675,7 @@ def get_media_log_events_panel(events_list_view):
     star_not_active_check = Gtk.CheckButton()
     star_not_active_check.set_active(True)
     star_not_active_check.connect("clicked", lambda w:media_log_filtering_changed())
+    star_not_active_check.set_margin_end(5)
     widgets.star_not_active_check = star_not_active_check
 
     star_not_active_label = Gtk.Image()
@@ -718,11 +726,6 @@ def get_media_log_events_panel(events_list_view):
     delete_button.set_size_request(80, 30)
     delete_button.connect("clicked", lambda w:delete_selected())
 
-    use_comments_label = Gtk.Label(label=_("Use Comments as Clip Names"))
-    use_comments_check =  Gtk.CheckButton()
-    use_comments_check.set_active(False)
-    widgets.use_comments_check = use_comments_check
-
     insert_displayed = Gtk.Button()
     insert_displayed.set_image(guiutils.get_image("insert_media_log"))
     insert_displayed.set_size_request(80, 22)
@@ -738,28 +741,25 @@ def get_media_log_events_panel(events_list_view):
     row2.pack_start(widgets.log_range, False, True, 0)
     row2.pack_start(delete_button, False, True, 0)
     row2.pack_start(Gtk.Label(), True, True, 0)
-    row2.pack_start(use_comments_label, False, True, 0)
-    row2.pack_start(use_comments_check, False, True, 0)
-    row2.pack_start(Gtk.Label(), True, True, 0)
     row2.pack_start(insert_displayed, False, True, 0)
     row2.pack_start(append_displayed, False, True, 0)
 
+    # NOTE: Panel width determined by row1 widgets sizes.
     panel = Gtk.VBox()
     panel.pack_start(row1, False, True, 0)
     panel.pack_start(events_list_view, True, True, 0)
     panel.pack_start(row2, False, True, 0)
-    panel.set_size_request(400, 10)
 
     star_check.set_tooltip_text(_("Display starred ranges"))    
     star_not_active_check.set_tooltip_text(_("Display non-starred ranges"))
     star_button.set_tooltip_text(_("Set selected ranges starred"))
     no_star_button.set_tooltip_text(_("Set selected ranges non-starred"))
-    widgets.log_range.set_tooltip_text(_("Log current marked range"))
+    widgets.log_range.set_tooltip_text(_("Log current marked Media Item range"))
     delete_button.set_tooltip_text(_("Delete selected ranges"))
     insert_displayed.set_tooltip_text(_("Insert selected ranges on Timeline"))
     append_displayed.set_tooltip_text(_("Append displayed ranges on Timeline"))
 
-    dnd.connect_range_log(events_list_view.treeview)
+    dnd.connect_range_log(events_list_view.treeview, _do_range_log_drop_on_monitor)
         
     return panel
 
