@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2012 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 """
 
 """
-Module contains objects that wrap mlt.Transition objects used to mix video betweeen
+Module contains objects that wrap mlt.Transition objects used to mix video between
 two tracks.
 """
 import copy
@@ -31,6 +31,7 @@ import os
 import xml.dom.minidom
 
 import appconsts
+from editorstate import PROJECT
 import mltrefhold
 import mltfilters
 import propertyparse
@@ -49,7 +50,7 @@ PROP_INT = appconsts.PROP_INT
 PROP_FLOAT = appconsts.PROP_FLOAT
 PROP_EXPRESSION = appconsts.PROP_EXPRESSION
 
-# Renderered transitions
+# Rendered transitions
 RENDERED_DISSOLVE = appconsts.RENDERED_DISSOLVE
 RENDERED_WIPE = appconsts.RENDERED_WIPE
 RENDERED_COLOR_DIP = appconsts.RENDERED_COLOR_DIP
@@ -187,7 +188,7 @@ def init_module():
 # ------------------------------------------ compositors
 class CompositorTransitionInfo:
     """
-    Constructor input is a XML dom node object. Convers XML data to another form
+    Constructor input is a XML dom node object. Converts XML data to another form
     used to create CompositorTransition objects.
     """
     def __init__(self, compositor_node):
@@ -313,7 +314,7 @@ class CompositorTransition:
 
 class CompositorObject:
     """
-    These objects are saved with projects. Thay are used to create, 
+    These objects are saved with projects. They are used to create, 
     update and hold references to mlt.Transition
     objects that define a composite between two tracks.
 
@@ -334,7 +335,7 @@ class CompositorObject:
     
         self.destroy_id = os.urandom(16) # HACK, HACK, HACK - find a way to remove this stuff  
                                          # Compositors are recreated often in Sequence.restack_compositors()
-                                         # and cannot be destroyd in undo/redo with object identidy.
+                                         # and cannot be destroyed in undo/redo with object identidy.
                                          # This is cloned in clone_properties
 
     def get_length(self):
@@ -473,8 +474,8 @@ def get_rendered_transition_tractor(current_sequence,
     # Create tractor and tracks
     tractor = mlt.Tractor()
     multitrack = tractor.multitrack()
-    track0 = mlt.Playlist()
-    track1 = mlt.Playlist()
+    track0 = mlt.Playlist(PROJECT().profile)
+    track1 = mlt.Playlist(PROJECT().profile)
     multitrack.connect(track0, 0)
     multitrack.connect(track1, 1)
 
