@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2012 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,12 +22,11 @@
 Module has methods that build panels from widgets. Created panels
 are used to build gui at callsites.
 """
-import cairo
-
 from gi.repository import Gtk, Gdk
 
 import appconsts
 import editorpersistance # Aug-2019 - SvdB - BB
+from editorstate import current_sequence
 import gui
 import guicomponents
 import guiutils
@@ -35,7 +34,6 @@ import editorstate
 import mltfilters
 import mlttransitions
 import renderconsumer
-import respaths
 import utils
 
 HALF_ROW_WIDTH = 160 # Size of half row when using two column row components created here
@@ -54,10 +52,11 @@ def get_media_files_panel(media_list_view, add_cb, del_cb, col_changed_cb, hambu
     if prefs.double_track_hights:
         size_adj = 2
     hamburger_launcher = guicomponents.HamburgerPressLaunch(hamburger_launch_pressed)
+    hamburger_launcher.do_popover_callback = True
     guiutils.set_margins(hamburger_launcher.widget, 2, 0, 4, 12)
 
     columns_img = guiutils.get_cairo_image("columns")
-    columns_launcher = guicomponents.PressLaunch(col_changed_cb, columns_img, w=22*size_adj, h=22*size_adj)
+    columns_launcher = guicomponents.PressLaunchPopover(col_changed_cb, columns_img, w=22*size_adj, h=22*size_adj)
     columns_launcher.surface_y = 6
     columns_launcher.widget.set_tooltip_text(_("Number of Media File columns."))
     
@@ -69,7 +68,7 @@ def get_media_files_panel(media_list_view, add_cb, del_cb, col_changed_cb, hambu
     pattern_pixbuf = guiutils.get_cairo_image("show_pattern_producers")
     unused_pixbuf = guiutils.get_cairo_image("show_unused_files")
 
-    files_filter_launcher = guicomponents.ImageMenuLaunch(filtering_cb, [all_pixbuf, video_pixbuf, audio_pixbuf, graphics_pixbuf, imgseq_pixbuf, pattern_pixbuf, unused_pixbuf], 24*size_adj, 22*size_adj)
+    files_filter_launcher = guicomponents.ImageMenuLaunchPopover(filtering_cb, [all_pixbuf, video_pixbuf, audio_pixbuf, graphics_pixbuf, imgseq_pixbuf, pattern_pixbuf, unused_pixbuf], 24*size_adj, 22*size_adj)
     files_filter_launcher.surface_x  = 3
     files_filter_launcher.surface_y  = 4
     files_filter_launcher.widget.set_tooltip_text(_("Visible Media File types."))
@@ -84,42 +83,35 @@ def get_media_files_panel(media_list_view, add_cb, del_cb, col_changed_cb, hambu
     buttons_box.pack_start(files_filter_launcher.widget, False, False, 0)
     buttons_box.pack_start(Gtk.Label(), True, True, 0)
     buttons_box.pack_start(bin_info, False, False, 0)
-    #buttons_box.pack_start(Gtk.Label(), True, True, 0)
-
+    buttons_box.set_name("darker-bg-widget")
+    guiutils.set_margins(buttons_box, 0, 6, 4, 6)
+    
+    buttons_box_wrapper = Gtk.HBox(False,1)
+    buttons_box_wrapper.pack_start(buttons_box, False, False, 0)
+    buttons_box_wrapper.set_name("darker-bg-widget")
+    
     panel = Gtk.VBox()
     panel.pack_start(media_list_view, True, True, 0)
-    panel.pack_start(buttons_box, False, True, 0)
+    panel.pack_start(buttons_box_wrapper, False, True, 0)
 
     return (panel, bin_info)
 
-def get_bins_tree_panel(bin_list_view):   
+def get_bins_tree_panel(bin_list_view, callback):   
     panel = Gtk.VBox()
     panel.pack_start(bin_list_view, True, True, 0)
 
-    return get_named_frame(_("Bins"), panel, 0, 0, 0, "A <b>Bin</b> is a named collection of media.")
+    hamburger = guicomponents.HamburgerPressLaunch(callback)
+    hamburger.do_popover_callback = True
     
-def get_sequences_panel(sequence_list_view, edit_seq_cb, add_seq_cb, del_seq_cb):
-    # Create buttons and connect signals
-    add_b = Gtk.Button(_("Add"))
-    del_b = Gtk.Button(_("Delete"))
-    edit_b = Gtk.Button(_("Edit"))
-    add_b.set_tooltip_text(_("Add new Sequence to Project"))
-    del_b.set_tooltip_text(_("Delete Sequence from Project"))
-    edit_b.set_tooltip_text(_("Start editing Sequence"))
-    edit_b.connect("clicked", edit_seq_cb, None)
-    add_b.connect("clicked", add_seq_cb, None)
-    del_b.connect("clicked", del_seq_cb, None)
-
-    buttons_box = Gtk.HBox(True,1)
-    buttons_box.pack_start(edit_b, True, True, 0)
-    buttons_box.pack_start(add_b, True, True, 0)
-    buttons_box.pack_start(del_b, True, True, 0)
+    return get_named_frame_with_hamburger(_("Bins"), panel, 0, 0, 0, _("A <b>Bin</b> is a named collection of media."), hamburger.widget)
     
+def get_sequences_panel(sequence_list_view, callback):
     panel = Gtk.VBox()
-    #panel.pack_start(buttons_box, False, True, 0)
     panel.pack_start(sequence_list_view, True, True, 0)
 
-    return get_named_frame(_("Sequences"), panel, 0, 6, 4, _("A <b>Sequence</b> is the full contents of the timeline creating a program, a movie."))
+    hamburger = guicomponents.HamburgerPressLaunch(callback)
+    
+    return get_named_frame_with_hamburger(_("Sequences"), panel, 0, 0, 0, _("A <b>Sequence</b> is the full contents of the timeline creating a program, a movie."), hamburger.widget)
 
 def _set_sensive_widgets(sensitive, list):
     for widget in list:
@@ -150,6 +142,32 @@ def get_named_frame(name, widget, left_padding=12, right_padding=6, right_out_pa
     
     return frame
 
+def get_named_frame_with_hamburger(name, widget, left_padding=12, right_padding=6, right_out_padding=4, tooltip_txt=None, hamburger_widget=None):
+    """
+    Gnome style named panel
+    """
+    if name != None:
+        label = guiutils.bold_label(name)
+        label.set_justify(Gtk.Justification.LEFT)
+        label.set_margin_start(4)
+        label_box = Gtk.HBox()
+        label_box.pack_start(label, False, False, 0)
+        label_box.pack_start(Gtk.Label(), True, True, 0)
+        label_box.pack_start(hamburger_widget, False, False, 0)
+        if tooltip_txt != None:        
+            label.set_tooltip_markup(tooltip_txt)
+
+    guiutils.set_margins(widget, right_padding, 0, left_padding, 0)
+
+    frame = Gtk.VBox()
+    if name != None:
+        frame.pack_start(label_box, False, False, 0)
+    frame.pack_start(widget, True, True, 0)
+
+    guiutils.set_margins(frame, 4, 4, 0, right_out_padding)
+    
+    return frame
+    
 def get_two_text_panel(primary_txt, secondary_txt):
     p_label = guiutils.bold_label(primary_txt)
     s_label = Gtk.Label(label=secondary_txt)
@@ -253,8 +271,6 @@ def get_transition_panel(trans_data):
     type_combo_box.append_text(name)
     name, t_service_id = mlttransitions.rendered_transitions[1]
     type_combo_box.append_text(name)
-    name, t_service_id = mlttransitions.rendered_transitions[2]
-    type_combo_box.append_text(name)
     type_combo_box.set_active(0)
 
     type_row = get_two_column_box(Gtk.Label(label=_("Type:")), 
@@ -270,17 +286,10 @@ def get_transition_panel(trans_data):
     wipe_row = get_two_column_box(wipe_label, 
                                  wipe_luma_combo_box)
 
-    color_button = Gtk.ColorButton.new_with_rgba(Gdk.RGBA(0,0,0,1))
-    color_button_box = guiutils.get_left_justified_box([color_button])
-    color_label = Gtk.Label(label=_("Dip Color:"))
-    color_row = get_two_column_box(color_label, color_button_box)
-
     wipe_luma_combo_box.set_sensitive(False)
-    color_button.set_sensitive(False)
     wipe_label.set_sensitive(False)
-    color_label.set_sensitive(False)
-
-    transition_type_widgets = (type_combo_box, wipe_luma_combo_box, color_button, wipe_label, color_label)
+    
+    transition_type_widgets = (type_combo_box, wipe_luma_combo_box, wipe_label)
     type_combo_box.connect("changed", 
                               lambda w,e: _transition_type_changed(transition_type_widgets), 
                               None)
@@ -332,7 +341,6 @@ def get_transition_panel(trans_data):
     edit_vbox.pack_start(type_row, False, False, 0)
     edit_vbox.pack_start(length_row, False, False, 0)
     edit_vbox.pack_start(wipe_row, False, False, 0)
-    edit_vbox.pack_start(color_row, False, False, 0)
 
     data_vbox = Gtk.VBox(False, 2)
     data_vbox.pack_start(out_handle_row, False, False, 0)
@@ -351,7 +359,7 @@ def get_transition_panel(trans_data):
 
     alignment = guiutils.set_margins(vbox, 12, 24, 12, 12)
 
-    return (alignment, type_combo_box, length_entry, encodings_cb, quality_cb, wipe_luma_combo_box, color_button, steal_frames, encodings)
+    return (alignment, type_combo_box, length_entry, encodings_cb, quality_cb, wipe_luma_combo_box, steal_frames, encodings)
 
 def _get_encodings_widget_and_list():
     # We have an unexplained issue with rendering using libx264 vcodec
@@ -399,40 +407,6 @@ def get_transition_re_render_panel(trans_data):
     
     return (alignment, encodings_cb, quality_cb, encodings)
 
-def get_fade_re_render_panel(trans_data):
-    fade_length = trans_data["clip"] .clip_out - trans_data["clip"].clip_in + 1 # +1 out inclusive
-    fade_length_label = Gtk.Label(label=_("Length:"))
-    fade_length_value = Gtk.Label(label=str(fade_length))
-    fade_length_row = get_two_column_box(fade_length_label, fade_length_value)
-
-    # Encoding widgets
-    encodings, encodings_cb = _get_encodings_widget_and_list()
-
-    quality_cb = Gtk.ComboBoxText()
-    transition_widgets = (encodings_cb, encodings, quality_cb)
-    encodings_cb.connect("changed", 
-                              lambda w,e: _transition_encoding_changed(transition_widgets), 
-                              None)
-    _fill_transition_quality_combo_box(transition_widgets, 10)
-    
-    _set_saved_encoding(transition_widgets)
-
-    fade_vbox = Gtk.VBox(False, 2)
-    fade_vbox.pack_start(fade_length_row, False, False, 0)
-   
-    enconding_vbox = Gtk.VBox(False, 2)
-    enconding_vbox.pack_start(encodings_cb, False, False, 0)
-    enconding_vbox.pack_start(quality_cb, False, False, 0)
-    
-    vbox = Gtk.VBox(False, 2)
-    vbox.pack_start(get_named_frame(_("Fade"),  fade_vbox), True, True, 0)
-    vbox.pack_start(get_named_frame(_("Encoding"),  enconding_vbox), True, True, 0)
-
-    alignment = guiutils.set_margins(vbox, 12, 24, 12, 12)
-    alignment.set_size_request(450, 200)
-    
-    return (alignment, encodings_cb, quality_cb, encodings)
-
 def get_re_render_all_panel(rerender_list, unrenderable):
     rerendercount_label = Gtk.Label(label=_("Transitions / Fades to be rerendered:"))
     rerendercount_value = Gtk.Label(label=str(len(rerender_list)))
@@ -440,7 +414,7 @@ def get_re_render_all_panel(rerender_list, unrenderable):
     
     if unrenderable > 0:
         unrenderable_info = _("There are ") + str(unrenderable) + _(" Transitions / Fades that cannot be rerendered, either because they are\ncreated with Flowblade version <=1.14 or the source clips are no longer on timeline.")
-        unrenderable_info_label = Gtk.Label(unrenderable_info)
+        unrenderable_info_label = Gtk.Label(label=unrenderable_info)
         
     info_vbox = Gtk.VBox(False, 2)
     info_vbox.pack_start(rerendercount_row, False, False, 0)
@@ -473,63 +447,10 @@ def get_re_render_all_panel(rerender_list, unrenderable):
     
     return (alignment, encodings_cb, quality_cb, encodings)
     
-def get_fade_panel(fade_data):
-    type_combo_box = Gtk.ComboBoxText()    
-    type_combo_box.append_text(_("Fade In"))
-    type_combo_box.append_text(_("Fade Out"))
-    type_combo_box.set_active(0)
-
-    type_row = get_two_column_box(Gtk.Label(label=_("Type:")), 
-                                 type_combo_box)
-        
-    color_button = Gtk.ColorButton.new_with_rgba(Gdk.RGBA(0,0,0,1))
-    color_button_box = guiutils.get_left_justified_box([color_button])
-    color_label = Gtk.Label(label=_("Color:"))
-    color_row = get_two_column_box(color_label, color_button_box)
-                              
-    length_entry = Gtk.Entry()
-    fade_length = 30
-    if editorstate.fade_length > 0: # use last invocation length if available
-        fade_length = editorstate.fade_length
-    length_entry.set_text(str(fade_length))    
-    length_row = get_two_column_box(Gtk.Label(label=_("Length:")), 
-                                    length_entry)
-
-    # Encoding widgets
-    encodings, encodings_cb = _get_encodings_widget_and_list()
-
-    quality_cb = Gtk.ComboBoxText()
-    transition_widgets = (encodings_cb, encodings, quality_cb)
-    encodings_cb.connect("changed", 
-                              lambda w,e: _transition_encoding_changed(transition_widgets), 
-                              None)
-
-    _fill_transition_quality_combo_box(transition_widgets, 10)
-    _set_saved_encoding(transition_widgets)
-    
-    # Build panel
-    edit_vbox = Gtk.VBox(False, 2)
-    edit_vbox.pack_start(type_row, False, False, 0)
-    edit_vbox.pack_start(length_row, False, False, 0)
-    edit_vbox.pack_start(color_row, False, False, 0)
-
-    enconding_vbox = Gtk.VBox(False, 2)
-    enconding_vbox.pack_start(encodings_cb, False, False, 0)
-    enconding_vbox.pack_start(quality_cb, False, False, 0)
-
-    vbox = Gtk.VBox(False, 2)
-    vbox.pack_start(get_named_frame(_("Transition Options"),  edit_vbox), True, True, 0)
-    vbox.pack_start(get_named_frame(_("Encoding"),  enconding_vbox), True, True, 0)
-
-    alignment = guiutils.set_margins(vbox, 12, 24, 12, 12)
-    
-    return (alignment, type_combo_box, length_entry, encodings_cb, quality_cb, color_button, encodings)
-    
 def _transition_encoding_changed(widgets):
     _fill_transition_quality_combo_box(widgets)
  
 def _fill_transition_quality_combo_box(widgets, quality_index=-1):
-
     encodings_cb, encodings, quality_cb = widgets
     sel_enc_index = encodings_cb.get_active()
     sel_enc = encodings[sel_enc_index]
@@ -565,33 +486,25 @@ def _set_saved_encoding(transition_widgets):
         quality_cb.set_active(quality_index)
     
 def _transition_type_changed(transition_type_widgets):
-    type_combo_box, wipe_luma_combo_box, color_button, wipe_label, color_label = transition_type_widgets
+    type_combo_box, wipe_luma_combo_box, wipe_label = transition_type_widgets
     if type_combo_box.get_active() == 0:
         wipe_luma_combo_box.set_sensitive(False)
-        color_button.set_sensitive(False)
         wipe_label.set_sensitive(False)
-        color_label.set_sensitive(False)
     elif type_combo_box.get_active() == 1:
         wipe_luma_combo_box.set_sensitive(True)
-        color_button.set_sensitive(False)
         wipe_label.set_sensitive(True)
-        color_label.set_sensitive(False)
-    else:
-        wipe_luma_combo_box.set_sensitive(False)
-        color_button.set_sensitive(True)
-        wipe_label.set_sensitive(False)
-        color_label.set_sensitive(True)
     
 def get_effect_selection_panel(double_click_cb):
     effects_list_view = guicomponents.FilterListView(None)
     group_combo_box = Gtk.ComboBoxText()
-
     for group in mltfilters.groups:
         group_name, filters_array = group
+
         group_combo_box.append_text(group_name)
+        
     group_combo_box.set_active(0)
     group_combo_box.connect("changed", 
-                            lambda w,e: _group_selection_changed(w,effects_list_view), 
+                            lambda w,e: _group_selection_changed(w, effects_list_view), 
                             None)
 
     combo_row = Gtk.HBox(False, 2)
@@ -610,9 +523,16 @@ def get_effect_selection_panel(double_click_cb):
 
 def _group_selection_changed(group_combo, filters_list_view):
     group_name, filters_array = mltfilters.groups[group_combo.get_active()]
+
     filters_list_view.fill_data_model(filters_array)
     filters_list_view.treeview.get_selection().select_path("0")
     
+    # "Blend" group available only when in compositing_mode COMPOSITING_MODE_STANDARD_FULL_TRACK.
+    if filters_array[0].mlt_service_id == "cairoblend_mode" and current_sequence().compositing_mode != appconsts.COMPOSITING_MODE_STANDARD_FULL_TRACK:
+        filters_list_view.set_sensitive(False)
+    else:
+        filters_list_view.set_sensitive(True)
+
 # -------------------------------------------------- guiutils
 def get_bold_label(text):
     return guiutils.bold_label(text)
