@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2020 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,11 +19,10 @@
 """
 
 """
-This module holds funtions that maintain campatibility between project savefiles
+This module holds functions that maintain compatibility between project savefiles
 created by different versions of application.
-
-Refactoring to move code here is an ongoing effort.
 """
+
 import appconsts
 
 # ------------------------------------------------------- FIXING MISSING ATTRS
@@ -67,7 +66,7 @@ def FIX_MISSING_FILTER_ATTRS(filter):
         filter.info.filter_mask_filter = None
             
 def FIX_MISSING_COMPOSITOR_ATTRS(compositor):
-    # Keeping backwards compability
+    # Keeping backwards compatibility
     if not hasattr(compositor, "obey_autofollow"): # "obey_autofollow" attr was added for 1.16
         compositor.obey_autofollow = True
 
@@ -95,3 +94,7 @@ def FIX_MISSING_PROJECT_ATTRS(project):
     if(not hasattr(project, "update_media_lengths_on_load")):
         project.update_media_lengths_on_load = True # old projects < 1.10 had wrong media length data which just was never used.
                                                     # 1.10 needed that data for the first time and required recreating it correctly for older projects
+
+    if(not hasattr(project, "vault_folder")):
+        project.vault_folder = None
+        project.project_data_id = None
