@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2012 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,9 +23,13 @@ Module contains objects used to capture project data.
 """
 import cairo
 import datetime
-import mlt
+try:
+    import mlt7 as mlt
+except:
+    import mlt
 import hashlib
 import os
+import shutil
 
 from gi.repository import GdkPixbuf
 
@@ -46,7 +50,6 @@ SAVEFILE_VERSION = 5 # This has freezed at 5 for long time,
 
 FALLBACK_THUMB = "fallback_thumb.png"
 
-PROXIES_DIR = "/proxies/"
  
 # Project events
 EVENT_CREATED_BY_NEW_DIALOG = 0
@@ -63,7 +66,7 @@ thumbnailer = None
 # Default values for project properties.
 _project_properties_default_values = {appconsts.P_PROP_TLINE_SHRINK_VERTICAL:False, # Shink timeline max height if < 9 tracks
                                       appconsts.P_PROP_LAST_RENDER_SELECTIONS: None, # tuple for last render selections data
-                                      appconsts.P_PROP_TRANSITION_ENCODING: None, # tuple for last renderered transition render selections data
+                                      appconsts.P_PROP_TRANSITION_ENCODING: None, # tuple for last rendered transition render selections data
                                       appconsts.P_PROP_DEFAULT_FADE_LENGTH: 10}  
 
 # Flag used to decide if user should be prompt to save project on project exit.
@@ -95,6 +98,9 @@ class Project:
                                                   # 1.10 needed that data for the first time and required recreating it correctly for older projects
         self.project_properties = {} # Key value pair for misc persistent properties, dict is used that we can add these without worrying loading
 
+        self.vault_folder = None
+        self.project_data_id = None
+
         self.SAVEFILE_VERSION = SAVEFILE_VERSION
         
         # c_seq is the currently edited Sequence
@@ -106,7 +112,17 @@ class Project:
         self.c_bin = self.bins[0]
         
         self.init_thumbnailer()
-    
+
+    def create_vault_folder_data(self, vault_dir):
+        # This is called just once when Project is created for the first time.
+        # All versions of project that are saved after this is called one time 
+        # on creation time use this same data and therefore save project data in the same place.
+        self.vault_folder = vault_dir
+        
+        md_key = str(datetime.datetime.now()) + str(os.urandom(16))
+        md_str = hashlib.md5(md_key.encode('utf-8')).hexdigest()
+        self.project_data_id = md_str
+
     def init_thumbnailer(self):
         global thumbnailer
         thumbnailer = Thumbnailer()
@@ -133,9 +149,14 @@ class Project:
             icon_path = respaths.IMAGE_PATH + "audio_file.png"
             length = thumbnailer.get_file_length(file_path)
             info = None
-        else: # For non-audio we need write a thumbbnail file and get file lengh while we're at it
+        else: # For non-audio we need write a thumbnail file and get file length while we're at it
              (icon_path, length, info) = thumbnailer.write_image(file_path)
 
+        # Refuse files giving "fps_den" == 0, these have been seen in the wild.
+        if media_type == appconsts.VIDEO and info["fps_den"] == 0.0: 
+            msg = _("Video file gives value 0 for 'fps_den' property.")
+            raise ProducerNotValidError(msg, file_path)
+        
         # Hide file extension if enabled in user preferences
         clip_name = file_name
         if editorpersistance.prefs.hide_file_ext == True:
@@ -181,11 +202,20 @@ class Project:
         for key, media_file in list(self.media_files.items()):
             if media_file.type == appconsts.PATTERN_PRODUCER:
                 continue
+            if media_file.container_data != None:
+                continue
             if file_path == media_file.path:
                 return True
 
         return False
 
+    def get_bin_for_media_file_id(self, media_file_id):
+        for bin in self.bins:
+            for file_id in bin.file_ids:
+                if file_id == media_file_id:
+                    return bin
+        return None
+        
     def get_media_file_for_path(self, file_path):
         for key, media_file in list(self.media_files.items()):
             if media_file.type == appconsts.PATTERN_PRODUCER:
@@ -197,12 +227,21 @@ class Project:
     def get_media_file_for_second_path(self, file_path):
         for key, media_file in list(self.media_files.items()):
             if media_file.type == appconsts.PATTERN_PRODUCER:
-                continue
-            print(file_path, media_file.path, media_file.second_file_path)
-            
+                continue            
             if file_path == media_file.second_file_path:
                 return media_file
         return None
+
+    def get_media_file_for_container_data(self, container_data):
+        for key, media_file in list(self.media_files.items()):
+            if media_file.type == appconsts.PATTERN_PRODUCER:
+                continue
+            if media_file.container_data == None:
+                continue
+            if media_file.container_data.program == container_data.program and \
+                media_file.container_data.unrendered_media == container_data.unrendered_media:
+                return media_file
+        return None
         
     def delete_media_file_from_current_bin(self, media_file):
         global media_files_changed_since_last_save
@@ -239,8 +278,9 @@ class Project:
         self.add_named_sequence(name)
         
     def add_named_sequence(self, name):
-        seq = sequence.Sequence(self.profile, name)
+        seq = sequence.Sequence(self.profile, appconsts.COMPOSITING_MODE_STANDARD_FULL_TRACK)
         seq.create_default_tracks()
+        seq.name = name
         self.sequences.append(seq)
         self.next_seq_number += 1
 
@@ -388,7 +428,7 @@ class MediaFile:
         if hasattr(self, "use_unique_proxy"): # This may have been added in proxyediting.py to prevent interfering with existing projects
             proxy_md_key = proxy_md_key + str(os.urandom(16))
         md_str = hashlib.md5(proxy_md_key.encode('utf-8')).hexdigest()
-        return str(userfolders.get_render_dir() + "/proxies/" + md_str + "." + file_extesion) # str() because we get unicode here
+        return str(userfolders.get_proxies_dir() + md_str + "." + file_extesion) # str() because we get unicode here
 
     def _create_img_seg_proxy_path(self,  proxy_width, proxy_height):
         folder, file_name = os.path.split(self.path)
@@ -396,7 +436,7 @@ class MediaFile:
         if hasattr(self, "use_unique_proxy"): # This may have been added in proxyediting.py to prevent interfering with existing projects
             proxy_md_key = proxy_md_key + str(os.urandom(16))
         md_str = hashlib.md5(proxy_md_key.encode('utf-8')).hexdigest()
-        return str(userfolders.get_render_dir() + "/"+ appconsts.PROXIES_DIR + md_str + "/" + file_name)
+        return str(userfolders.get_proxies_dir() + md_str + "/" + file_name)
 
     def add_proxy_file(self, proxy_path):
         self.has_proxy_file = True
@@ -434,7 +474,7 @@ class MediaFile:
 
 class BinColorClip:
     # DECPRECATED, this is replaced by patternproducer.BinColorClip.
-    # This is kept for project file backwards compatiblity,
+    # This is kept for project file backwards compatibility,
     # unpickle fails for color clips if this isn't here.
     # kill 2016-ish
     def __init__(self, id, name, gdk_color_str):
@@ -469,8 +509,10 @@ class Bin:
         
         
 class ProducerNotValidError(Exception):
-    def __init__(self, value):
+    def __init__(self, value, file_path):
         self.value = value
+        self.file_path = file_path
+
     def __str__(self):
         return repr(self.value)
 
@@ -488,19 +530,21 @@ class Thumbnailer:
         """
         # Get data
         md_str = hashlib.md5(file_path.encode('utf-8')).hexdigest()
-        thumbnail_path = userfolders.get_cache_dir() + appconsts.THUMBNAILS_DIR + "/" + md_str +  ".png"
+        thumbnail_path = userfolders.get_thumbnail_dir() + md_str + ".png"
+        render_image_path = userfolders.get_cache_dir() + "thumbnail%03d.png"
 
         # Create consumer
         consumer = mlt.Consumer(self.profile, "avformat", 
-                                     thumbnail_path)
+                                     render_image_path)
         consumer.set("real_time", 0)
         consumer.set("vcodec", "png")
 
         # Create one frame producer
         producer = mlt.Producer(self.profile, str(file_path))
         if producer.is_valid() == False:
-            raise ProducerNotValidError(file_path)
-
+            msg = _("MLT reports that file is not a valid media producer.")
+            raise ProducerNotValidError(msg, file_path)
+            
         info = utils.get_file_producer_info(producer)
 
         length = producer.get_length()
@@ -511,14 +555,19 @@ class Thumbnailer:
         consumer.connect(producer)
         consumer.run()
         
+        # consumer.run() blocks until done so the thubnailfile is now ready to be copied.
+        shutil.copyfile(userfolders.get_cache_dir() + "thumbnail001.png", thumbnail_path)
+        
         return (thumbnail_path, length, info)
 
     def get_file_length(self, file_path):
         # This is used for audio files which don't need a thumbnail written
         # but do need file length known
 
-        # Create one frame producer
         producer = mlt.Producer(self.profile, str(file_path))
+        if producer.get("seekable") == "0":
+            msg = _("Audio file not seekable, cannot be edited.\n\n")
+            raise ProducerNotValidError(msg, file_path)
         return producer.get_length()
 
 
@@ -564,7 +613,6 @@ def get_default_project():
     """
     profile = mltprofiles.get_default_profile()
     project = Project(profile)
-    project.c_seq.compositing_mode = editorpersistance.prefs.default_compositing_mode
     return project
 
 
