@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2013 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,6 +23,8 @@ from gi.repository import Gtk
 from gi.repository import GObject
 from gi.repository import Pango
 
+import dialogutils
+import editorstate
 from editorstate import PROJECT
 import guicomponents
 import guiutils
@@ -30,13 +32,14 @@ import utils
 
 widgets = utils.EmptyClass()
 
-PROJECT_INFO_PANEL_HEIGHT = 200
-PROJECT_TOP_LEVEL_PANE_HEIGHT = 150
 
 def get_project_info_panel():
     project_name_label = Gtk.Label(label=PROJECT().name)
     name_row = guiutils.get_left_justified_box([project_name_label])
-    name_panel = guiutils.get_named_frame(_("Name"), name_row, 4)
+    name_vbox = Gtk.VBox()
+    name_vbox.pack_start(name_row, False, False, 0)
+    name_vbox.pack_start(Gtk.Label(), True, True, 0)
+    name_panel = guiutils.get_named_frame(_("Name"), name_vbox, 4)
     
     profile = PROJECT().profile
     desc_label = Gtk.Label(label=profile.description())
@@ -46,64 +49,60 @@ def get_project_info_panel():
     vbox.pack_start(info_box, False, True, 0)
     profile_panel = guiutils.get_named_frame(_("Profile"), vbox, 4)
 
-    events_list = ProjectEventListView()
-    events_list.fill_data_model()
-    events_panel = guiutils.get_named_frame(_("Project Events"), events_list, 4)
-
-    project_info_vbox = Gtk.VBox()
-    project_info_vbox.pack_start(name_panel, False, True, 0)
-    project_info_vbox.pack_start(profile_panel, False, True, 0)
-    project_info_vbox.set_size_request(250, PROJECT_INFO_PANEL_HEIGHT)
-    
     project_info_hbox = Gtk.HBox()
-    project_info_hbox.pack_start(project_info_vbox, False, False, 0)
-    project_info_hbox.pack_start(events_panel, True, True, 0)
+    project_info_hbox.pack_start(name_panel, False, True, 0)
+    project_info_hbox.pack_start(guiutils.pad_label(24, 24), False, True, 0)
+    project_info_hbox.pack_start(profile_panel, False, True, 0)
+    project_info_hbox.pack_start(Gtk.Label(), True, True, 0)
     
     widgets.project_name_label = project_name_label
     widgets.desc_label = desc_label
     widgets.info_box = info_box
-    widgets.events_list = events_list
 
     return project_info_hbox
 
 def get_top_level_project_info_panel():
-
-    project_name_label = Gtk.Label(label=PROJECT().name)
-    name_row = guiutils.get_left_justified_box([project_name_label])
-    name_panel = guiutils.get_named_frame(_("Project"), name_row, 0, 6, 4, _("A <b>Project</b> contains one or more <b>Sequences</b> of edited media and a collection of media files stored in <b>Bins.</b>"))
-
+    project_name_label = guiutils.bold_label(_("Project Profile"))
+    project_name_label.set_justify(Gtk.Justification.LEFT)
+    name_row = guiutils.set_margins(guiutils.get_left_justified_box([project_name_label]), 0, 4,0,0)
+    
     profile = PROJECT().profile
     desc_label = Gtk.Label(label=profile.description())
-    info_box = guicomponents.get_profile_info_small_box(profile)
-    vbox = Gtk.VBox()
-    vbox.pack_start(guiutils.get_left_justified_box([desc_label]), False, True, 0)
-    vbox.pack_start(info_box, False, True, 0)
-    profile_panel = guiutils.get_named_frame(_("Profile"), vbox, 0, 6, 4, _("<b>Profile</b> determines frame rate per second, image size in pixels and pixel aspect ratio for all <b>Sequences</b> in <b>Project</b> ."))
-
-    events_list = ProjectEventListView()
-    events_list.fill_data_model()
-    events_panel = guiutils.get_named_frame(_("Project Events"), events_list, 0)
-
+    if editorstate.screen_size_small_height() == True:
+        font_desc = "sans bold 8"
+    else:
+        font_desc = "sans bold 9"
+    desc_label.modify_font(Pango.FontDescription(font_desc))
+    desc_label.set_sensitive(False)
+    desc_row = guiutils.get_left_justified_box([desc_label])
+    desc_row.set_margin_start(4)
+    # We are leaving this for updates for now because smaller window sizes use this reference 'widgets.info_box' still.
+    info_box = guicomponents.get_profile_info_reduced_small_box(profile)
+    
     project_info_vbox = Gtk.VBox()
-    project_info_vbox.pack_start(name_panel, False, True, 0)
-    project_info_vbox.pack_start(profile_panel, False, True, 0)
-    guiutils.set_margins(project_info_vbox, 4,4,4,4)
+    project_info_vbox.pack_start(name_row, False, True, 0)
+    project_info_vbox.pack_start(desc_row, False, True, 0)
+    guiutils.set_margins(project_info_vbox, 4,0,4,4)
+
+    project_info_vbox.set_tooltip_text(guicomponents.get_full_profile_info_text(profile))
 
     widgets.project_name_label = project_name_label
     widgets.desc_label = desc_label
     widgets.info_box = info_box
-    widgets.events_list = events_list
-
+ 
     return project_info_vbox
     
 def update_project_info():
     profile = PROJECT().profile
-    widgets.project_name_label.set_markup(guiutils.bold_text(PROJECT().name))
     widgets.desc_label.set_text(profile.description())
-    profile_info_text = guicomponents.get_profile_info_text(profile)
+    profile_info_text = guicomponents.get_profile_reduced_info_text(profile)
     widgets.info_box.get_children()[0].set_text(profile_info_text)
-    widgets.events_list.fill_data_model()
 
+def show_project_events_dialog():
+    events_list = ProjectEventListView()
+    events_list.fill_data_model()
+    events_list.set_size_request(620, 200)
+    dialogutils.panel_ok_dialog(_("Project Events"), events_list)
 
 class ProjectEventListView(Gtk.VBox):
 
@@ -116,10 +115,9 @@ class ProjectEventListView(Gtk.VBox):
         # Scroll container
         self.scroll = Gtk.ScrolledWindow()
         self.scroll.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
-        self.scroll.set_shadow_type(Gtk.ShadowType.ETCHED_IN)
 
         # View
-        self.treeview = Gtk.TreeView(self.storemodel)
+        self.treeview = Gtk.TreeView(model=self.storemodel)
         self.treeview.set_property("rules_hint", True)
         self.treeview.set_headers_visible(True)
         tree_sel = self.treeview.get_selection()
@@ -131,7 +129,7 @@ class ProjectEventListView(Gtk.VBox):
         self.text_col_2 = Gtk.TreeViewColumn("text2")
         self.text_col_2.set_title(_("Event"))
         self.text_col_3 = Gtk.TreeViewColumn("text3")
-        self.text_col_3.set_title(_("Path"))
+        self.text_col_3.set_title(_("Data"))
 
         # Cell renderers
         self.text_rend_1 = Gtk.CellRendererText()
