@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2012 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,10 +20,9 @@
 
 """
 Modules provides functions that:
-- parse strings to property tuples or argument dicts
+- parse strings to property tuples or argument dicts.
 - build value strings from property tuples.
 """
-from gi.repository import Gtk
 import json
 
 import appconsts
@@ -47,6 +46,7 @@ VALUE_REPLACEMENT = "value_replacement"                     # attr name for repl
 FADE_IN_REPLAMENT = "fade_in_replament"                     # replace with fade in keyframes
 FADE_OUT_REPLAMENT = "fade_out_replament"                   # replace with fade out keyframes
 FADE_IN_OUT_REPLAMENT = "fade_in_out_replament"             # replace with fade in and out keyframes
+WIPE_IN_REPLAMENT = "wipe_in_replament"
 
 # ------------------------------------------- parse funcs
 def node_list_to_properties_array(node_list):
@@ -183,12 +183,20 @@ def replace_values_using_clip_data(properties, info, clip):
 
                         properties[i] = (prop_name, value, prop_type)
                         replacement_happened = True
+                    elif arg_val == WIPE_IN_REPLAMENT:
+                        frame_1 = clip.clip_in
+                        frame_2 = clip.clip_in + int(round(utils.fps())) # Make 1 second the default.
+                        value = ""
+                        if frame_1 != 0:
+                            value += "0=0;"
+                        
+                        value += str(frame_1) + "=0;" + str(frame_2) + "=100"
 
+                        properties[i] = (prop_name, value, prop_type)
+                        replacement_happened = True
+    
     return replacement_happened
-                
 
-
-            
 def get_args_num_value(val_str):
     """
     Returns numerical value for expression in property
@@ -210,28 +218,49 @@ def get_args_num_value(val_str):
 # ------------------------------------------ kf editor values strings to kf arrays funcs
 def single_value_keyframes_string_to_kf_array(keyframes_str, out_to_in_func):
     new_keyframes = []
-    keyframes_str = keyframes_str.strip('"') # expression have sometimes quotes that need to go away
+    keyframes_str = keyframes_str.strip('"') # expressions have sometimes quotes that need to go away
     kf_tokens = keyframes_str.split(";")
     for token in kf_tokens:
-        sides = token.split("=")
-        add_kf = (int(sides[0]), out_to_in_func(float(sides[1]))) # kf = (frame, value)
-        new_keyframes.append(add_kf)
+        sides = token.split(appconsts.KEYFRAME_DISCRETE_EQUALS_STR)
+        if len(sides) == 2:
+            kf_type = appconsts.KEYFRAME_DISCRETE
+        else:
+            sides = token.split(appconsts.KEYFRAME_SMOOTH_EQUALS_STR)
+            if len(sides) == 2:
+                kf_type = appconsts.KEYFRAME_SMOOTH
+            else:
+                sides = token.split(appconsts.KEYFRAME_LINEAR_EQUALS_STR)
+                kf_type = appconsts.KEYFRAME_LINEAR
         
+        # Find out saved keyframe type here.
+        add_kf = (int(sides[0]), out_to_in_func(float(sides[1])), kf_type) # kf = (frame, value, type)
+        new_keyframes.append(add_kf)
+
     return new_keyframes
     
 def geom_keyframes_value_string_to_opacity_kf_array(keyframes_str, out_to_in_func):
+    # THIS SHOULD ONLY BE IN DEPRECATED COMPOSITORS
     # Parse "composite:geometry" properties value string into (frame,opacity_value)
     # keyframe tuples.
     new_keyframes = []
     keyframes_str = keyframes_str.strip('"') # expression have sometimes quotes that need to go away
     kf_tokens =  keyframes_str.split(";")
     for token in kf_tokens:
-        sides = token.split("=")
+        sides = token.split(appconsts.KEYFRAME_DISCRETE_EQUALS_STR)
+        if len(sides) == 2:
+            kf_type = appconsts.KEYFRAME_DISCRETE
+        else:
+            sides = token.split(appconsts.KEYFRAME_SMOOTH_EQUALS_STR)
+            if len(sides) == 2:
+                kf_type = appconsts.KEYFRAME_SMOOTH
+            else:
+                sides = token.split(appconsts.KEYFRAME_LINEAR_EQUALS_STR)
+                kf_type = appconsts.KEYFRAME_LINEAR
+                
         values = sides[1].split(':')
-        add_kf = (int(sides[0]), out_to_in_func(float(values[2]))) # kf = (frame, opacity)
+
+        add_kf = (int(sides[0]), out_to_in_func(float(values[2])), kf_type) # kf = (frame, opacity, type)
         new_keyframes.append(add_kf)
- 
-    return new_keyframes
 
 def geom_keyframes_value_string_to_geom_kf_array(keyframes_str, out_to_in_func):
     # Parse "composite:geometry" properties value string into (frame, source_rect, opacity)
@@ -240,12 +269,22 @@ def geom_keyframes_value_string_to_geom_kf_array(keyframes_str, out_to_in_func):
     keyframes_str = keyframes_str.strip('"') # expression have sometimes quotes that need to go away
     kf_tokens =  keyframes_str.split(';')
     for token in kf_tokens:
-        sides = token.split('=')
+        sides = token.split(appconsts.KEYFRAME_DISCRETE_EQUALS_STR)
+        if len(sides) == 2:
+            kf_type = appconsts.KEYFRAME_DISCRETE
+        else:
+            sides = token.split(appconsts.KEYFRAME_SMOOTH_EQUALS_STR)
+            if len(sides) == 2:
+                kf_type = appconsts.KEYFRAME_SMOOTH
+            else:
+                sides = token.split(appconsts.KEYFRAME_LINEAR_EQUALS_STR)
+                kf_type = appconsts.KEYFRAME_LINEAR
+                
         values = sides[1].split(':')
         pos = values[0].split('/')
         size = values[1].split('x')
         source_rect = [int(pos[0]), int(pos[1]), int(size[0]), int(size[1])] #x,y,width,height
-        add_kf = (int(sides[0]), source_rect, out_to_in_func(float(values[2])))
+        add_kf = (int(sides[0]), source_rect, out_to_in_func(float(values[2])), kf_type)
         new_keyframes.append(add_kf)
  
     return new_keyframes
@@ -257,20 +296,30 @@ def rect_keyframes_value_string_to_geom_kf_array(keyframes_str, out_to_in_func):
     keyframes_str = keyframes_str.strip('"') # expression have sometimes quotes that need to go away
     kf_tokens =  keyframes_str.split(';')
     for token in kf_tokens:
-        sides = token.split('=')
+        sides = token.split(appconsts.KEYFRAME_DISCRETE_EQUALS_STR)
+        if len(sides) == 2:
+            kf_type = appconsts.KEYFRAME_DISCRETE
+        else:
+            sides = token.split(appconsts.KEYFRAME_SMOOTH_EQUALS_STR)
+            if len(sides) == 2:
+                kf_type = appconsts.KEYFRAME_SMOOTH
+            else:
+                sides = token.split(appconsts.KEYFRAME_LINEAR_EQUALS_STR)
+                kf_type = appconsts.KEYFRAME_LINEAR
+                
         values = sides[1].split(' ')
         x = values[0]
         y = values[1]
         w = values[2] 
         h = values[3] 
         source_rect = [int(x), int(y), int(w), int(h)] #x,y,width,height
-        add_kf = (int(sides[0]), source_rect, out_to_in_func(float(1)))
+        add_kf = (int(sides[0]), source_rect, out_to_in_func(float(1)), kf_type)
         new_keyframes.append(add_kf)
     
     return new_keyframes
     
 def rotating_geom_keyframes_value_string_to_geom_kf_array(keyframes_str, out_to_in_func):
-    # THIS WAS CREATED FOR frei0r cairoaffineblend FILTER. That filter has to use a very particular paramter values
+    # THIS WAS CREATED FOR frei0r cairoaffineblend FILTER. That filter has to use a very particular parameter values
     # scheme to satisty the frei0r requirement of all float values being in range 0.0 - 1.0.
     #
     # Parse extraeditor value properties value string into (frame, [x, y, x_scale, y_scale, rotation], opacity)
@@ -281,11 +330,21 @@ def rotating_geom_keyframes_value_string_to_geom_kf_array(keyframes_str, out_to_
     keyframes_str = keyframes_str.strip('"') # expression have sometimes quotes that need to go away
     kf_tokens =  keyframes_str.split(';')
     for token in kf_tokens:
-        sides = token.split('=')
+        sides = token.split(appconsts.KEYFRAME_DISCRETE_EQUALS_STR)
+        if len(sides) == 2:
+            kf_type = appconsts.KEYFRAME_DISCRETE
+        else:
+            sides = token.split(appconsts.KEYFRAME_SMOOTH_EQUALS_STR)
+            if len(sides) == 2:
+                kf_type = appconsts.KEYFRAME_SMOOTH
+            else:
+                sides = token.split(appconsts.KEYFRAME_LINEAR_EQUALS_STR)
+                kf_type = appconsts.KEYFRAME_LINEAR
+                
         values = sides[1].split(':')
         frame = int(sides[0])
-        # get values and convert "frei0r.cairoaffineblend" values to editor values
-        # this because all frei0r plugins require values in range 0 - 1
+        # Get values and convert "frei0r.cairoaffineblend" values to editor values
+        # This is done because all frei0r plugins require values in range 0 - 1
         x = _get_pixel_pos_from_frei0r_cairo_pos(float(values[0]), screen_width)
         y = _get_pixel_pos_from_frei0r_cairo_pos(float(values[1]), screen_height)
         x_scale = _get_scale_from_frei0r_cairo_scale(float(values[2]))
@@ -293,7 +352,7 @@ def rotating_geom_keyframes_value_string_to_geom_kf_array(keyframes_str, out_to_
         rotation = float(values[4]) * 360
         opacity = float(values[5]) * 100
         source_rect = [x,y,x_scale,y_scale,rotation]
-        add_kf = (frame, source_rect, float(opacity))
+        add_kf = (frame, source_rect, float(opacity), kf_type)
         new_keyframes.append(add_kf)
 
     return new_keyframes
@@ -302,8 +361,6 @@ def non_freior_rotating_geom_keyframes_value_string_to_geom_kf_array(keyframes_s
     # Parse extraeditor value properties value string into (frame, [x, y, x_scale, y_scale, rotation], opacity)
     # keyframe tuples.
     new_keyframes = []
-    screen_width = current_sequence().profile.width()
-    screen_height = current_sequence().profile.height()
     keyframes_str = keyframes_str.strip('"') # expression have sometimes quotes that need to go away
     kf_tokens =  keyframes_str.split(';')
     for token in kf_tokens:
@@ -329,61 +386,37 @@ def rotomask_json_value_string_to_kf_array(keyframes_str, out_to_in_func):
     json_obj = json.loads(keyframes_str)
     for kf in json_obj:
         kf_obj = json_obj[kf]
-        add_kf = (int(kf), kf_obj)
+        add_kf = (int(kf), kf_obj, appconsts.KEYFRAME_LINEAR) # Rotomask has own kf system and type KEYFRAME_LINEAR is here just to work with other components.
         new_keyframes.append(add_kf)
 
     return sorted(new_keyframes, key=lambda kf_tuple: kf_tuple[0]) 
-    
-    
+
+
 # ----------------------------------------------------------------------------- AFFINE BLEND
-def create_editable_property_for_affine_blend(clip, editable_properties):
-    # Build a custom object that duck types for TransitionEditableProperty to use in editor
-    # 
-    ep = utils.EmptyClass()
-    # pack real properties to go
-    ep.x = [ep for ep in editable_properties if ep.name == "x"][0]
-    ep.y = [ep for ep in editable_properties if ep.name == "y"][0]
-    ep.x_scale = [ep for ep in editable_properties if ep.name == "x scale"][0]
-    ep.y_scale = [ep for ep in editable_properties if ep.name == "y scale"][0]
-    ep.rotation = [ep for ep in editable_properties if ep.name == "rotation"][0]
-    ep.opacity = [ep for ep in editable_properties if ep.name == "opacity"][0]
-    # Screen width and height are needeed for frei0r conversions
-    ep.profile_width = current_sequence().profile.width()
-    ep.profile_height = current_sequence().profile.height()
-    # duck type methods, using opacity is not meaningful, any property with clip member could do
-    ep.get_clip_tline_pos = lambda : ep.opacity.clip.clip_in # clip is compositor, compositor in and out points straight in timeline frames
-    ep.get_clip_length = lambda : ep.opacity.clip.clip_out - ep.opacity.clip.clip_in + 1
-    ep.get_input_range_adjustment = lambda : Gtk.Adjustment(value=float(100), lower=float(0), upper=float(100), step_incr=float(1))
-    ep.get_display_name = lambda : "Opacity"
-    ep.get_pixel_aspect_ratio = lambda : (float(current_sequence().profile.sample_aspect_num()) / current_sequence().profile.sample_aspect_den())
-    ep.get_in_value = lambda out_value : out_value # hard coded for opacity 100 -> 100 range
-    ep.write_out_keyframes = lambda w_kf : rotating_ge_write_out_keyframes(ep, w_kf)
-    ep.update_prop_value = lambda : rotating_ge_update_prop_value(ep) # This is needed to get good update after adding kfs with fade buttons, iz all kinda fugly
-                                                                            # We need this to reinit GUI components after programmatically added kfs.
-    # duck type members
-    x_tokens = ep.x.value.split(";")
-    y_tokens = ep.y.value.split(";")
-    x_scale_tokens = ep.x_scale.value.split(";")
-    y_scale_tokens = ep.y_scale.value.split(";")
-    rotation_tokens = ep.rotation.value.split(";")
-    opacity_tokens = ep.opacity.value.split(";")
+def _get_roto_geom_frame_value(token):
+    sides = token.split(appconsts.KEYFRAME_DISCRETE_EQUALS_STR)
+    if len(sides) == 2:
+        kf_type = appconsts.KEYFRAME_DISCRETE
+    else:
+        sides = token.split(appconsts.KEYFRAME_SMOOTH_EQUALS_STR)
+        if len(sides) == 2:
+            kf_type = appconsts.KEYFRAME_SMOOTH
+        else:
+            sides = token.split(appconsts.KEYFRAME_LINEAR_EQUALS_STR)
+            kf_type = appconsts.KEYFRAME_LINEAR
     
-    value = ""
-    for i in range(0, len(x_tokens)): # these better match, same number of keyframes for all values, or this will not work
-        frame, x = x_tokens[i].split("=")
-        frame, y = y_tokens[i].split("=")
-        frame, x_scale = x_scale_tokens[i].split("=")
-        frame, y_scale = y_scale_tokens[i].split("=")
-        frame, rotation = rotation_tokens[i].split("=")
-        frame, opacity = opacity_tokens[i].split("=")
-        
-        frame_str = str(frame) + "=" + str(x) + ":" + str(y) + ":" + str(x_scale) + ":" + str(y_scale) + ":" + str(rotation) + ":" + str(opacity)
-        value += frame_str + ";"
+    return(sides[0], sides[1], kf_type)
 
-    ep.value = value.strip(";")
+def _get_eq_str(kf_type):
+    if kf_type == appconsts.KEYFRAME_DISCRETE:
+        eq_str = appconsts.KEYFRAME_DISCRETE_EQUALS_STR
+    elif kf_type == appconsts.KEYFRAME_SMOOTH:
+        eq_str = appconsts.KEYFRAME_SMOOTH_EQUALS_STR
+    else:
+        eq_str = appconsts.KEYFRAME_LINEAR_EQUALS_STR
+    
+    return eq_str
     
-    return ep
-
 def rotating_ge_write_out_keyframes(ep, keyframes):
     x_val = ""
     y_val = ""
@@ -393,14 +426,17 @@ def rotating_ge_write_out_keyframes(ep, keyframes):
     opacity_val = ""
     
     for kf in keyframes:
-        frame, transf, opacity = kf
+        frame, transf, opacity, kf_type = kf
         x, y, x_scale, y_scale, rotation = transf
-        x_val += str(frame) + "=" + str(get_frei0r_cairo_position(x, ep.profile_width)) + ";"
-        y_val += str(frame) + "=" + str(get_frei0r_cairo_position(y, ep.profile_height)) + ";"
-        x_scale_val += str(frame) + "=" + str(get_frei0r_cairo_scale(x_scale)) + ";"
-        y_scale_val += str(frame) + "=" + str(get_frei0r_cairo_scale(y_scale)) + ";"
-        rotation_val += str(frame) + "=" + str(rotation / 360.0) + ";"
-        opacity_val += str(frame) + "=" + str(opacity / 100.0) + ";"
+        
+        eq_str = _get_eq_str(kf_type)
+            
+        x_val += str(frame) + eq_str + str(get_frei0r_cairo_position(x, ep.profile_width)) + ";"
+        y_val += str(frame) + eq_str + str(get_frei0r_cairo_position(y, ep.profile_height)) + ";"
+        x_scale_val += str(frame) + eq_str + str(get_frei0r_cairo_scale(x_scale)) + ";"
+        y_scale_val += str(frame) + eq_str + str(get_frei0r_cairo_scale(y_scale)) + ";"
+        rotation_val += str(frame) + eq_str + str(rotation / 360.0) + ";"
+        opacity_val += str(frame) + eq_str + str(opacity / 100.0) + ";"
 
     x_val = x_val.strip(";")
     y_val = y_val.strip(";")
@@ -417,7 +453,6 @@ def rotating_ge_write_out_keyframes(ep, keyframes):
     ep.opacity.write_value(opacity_val)
 
 def rotating_ge_update_prop_value(ep):
-
     # duck type members
     x_tokens = ep.x.value.split(";")
     y_tokens = ep.y.value.split(";")
@@ -428,14 +463,16 @@ def rotating_ge_update_prop_value(ep):
     
     value = ""
     for i in range(0, len(x_tokens)): # these better match, same number of keyframes for all values, or this will not work
-        frame, x = x_tokens[i].split("=")
-        frame, y = y_tokens[i].split("=")
-        frame, x_scale = x_scale_tokens[i].split("=")
-        frame, y_scale = y_scale_tokens[i].split("=")
-        frame, rotation = rotation_tokens[i].split("=")
-        frame, opacity = opacity_tokens[i].split("=")
+        frame, x, kf_type = _get_roto_geom_frame_value(x_tokens[i])
+        frame, y, kf_type = _get_roto_geom_frame_value(y_tokens[i])
+        frame, x_scale, kf_type = _get_roto_geom_frame_value(x_scale_tokens[i])
+        frame, y_scale, kf_type = _get_roto_geom_frame_value(y_scale_tokens[i])
+        frame, rotation, kf_type = _get_roto_geom_frame_value(rotation_tokens[i])
+        frame, opacity, kf_type = _get_roto_geom_frame_value(opacity_tokens[i])
+
+        eq_str = _get_eq_str(kf_type)
         
-        frame_str = str(frame) + "=" + str(x) + ":" + str(y) + ":" + str(x_scale) + ":" + str(y_scale) + ":" + str(rotation) + ":" + str(opacity)
+        frame_str = str(frame) + eq_str + str(x) + ":" + str(y) + ":" + str(x_scale) + ":" + str(y_scale) + ":" + str(rotation) + ":" + str(opacity)
         value += frame_str + ";"
 
     ep.value = value.strip(";")
