@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2012 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,8 +21,6 @@
 """
 Module loads render options, provides them in displayable form 
 and builds a mlt.Consumer for rendering on request.
-
-Rendering is done in app.player object of class mltplayer.Player
 """
 
 
@@ -184,7 +182,6 @@ def create_widgets():
     Widgets for editing render properties and viewing render progress.
     """
     widgets.file_panel = rendergui.RenderFilePanel()
-    #widgets.render_type_panel = rendergui.RenderTypePanel(_render_type_changed, _preset_selection_changed)
     widgets.profile_panel = rendergui.RenderProfilePanel(_out_profile_changed)
     widgets.encoding_panel = rendergui.RenderEncodingPanel(widgets.file_panel.extension_label)
     if (editorstate.SCREEN_HEIGHT > 898):
@@ -231,7 +228,7 @@ def set_default_values_for_widgets(movie_name_too=False):
     widgets.profile_panel.use_project_profile_check.set_active(True)
 
 def update_encoding_selector():
-    # Lets make doubly sure we're not updating somthing that soes npt exist.
+    # Lets make doubly sure we're not updating something that soes npt exist.
     while hasattr(widgets, "encoding_panel") == False:
         print("no encoding_panel")
         time.sleep(0.5)
@@ -373,11 +370,11 @@ def _render_frame_buffer_clip_dialog_callback(dialog, response_id, fb_widgets, m
             if end_frame > motion_producer.get_length() - 1:
                 end_frame = motion_producer.get_length() - 1
             
-            render_full_range = False # consumer wont stop automatically and needs to stopped explicitly
+            render_full_range = False # consumer won't stop automatically and needs to stopped explicitly
         
         session_id = hashlib.md5(str(os.urandom(32)).encode('utf-8')).hexdigest()
         
-        args = ("session_id:" + str(session_id), 
+        args = ("session_id:" + str(session_id),
                 "speed:" + str(speed), 
                 "write_file:" + str(write_file),
                 "profile_desc:" + str(profile_desc),
@@ -460,7 +457,7 @@ def _render_reverse_clip_dialog_callback(dialog, response_id, fb_widgets, media_
             if start_frame < 0:
                 start_frame = 0
             
-            render_full_range = False # consumer wont stop automatically and needs to stopped explicitly
+            render_full_range = False # consumer won't stop automatically and needs to stopped explicitly
             
         session_id = hashlib.md5(str(os.urandom(32)).encode('utf-8')).hexdigest()
         
@@ -494,7 +491,7 @@ def _REVERSE_render_stop(dialog, response_id):
     
 # ----------------------------------------------------------------------- single track transition render 
 def render_single_track_transition_clip(transition_producer, encoding_option_index, quality_option_index, file_ext, transition_render_complete_cb, window_text):
-    # Set render complete callback to availble render stop callback using global variable
+    # Set render complete callback to available render stop callback using global variable
     global transition_render_done_callback
     transition_render_done_callback = transition_render_complete_cb
 
@@ -504,7 +501,7 @@ def render_single_track_transition_clip(transition_producer, encoding_option_ind
     folder = userfolders.get_render_dir()
 
     file_name = hashlib.md5(str(os.urandom(32)).encode('utf-8')).hexdigest()
-    write_file = folder + "/"+ file_name + file_ext
+    write_file = folder + file_name + file_ext
 
     # Render consumer
     consumer = renderconsumer.get_render_consumer_for_encoding_and_quality(write_file, profile, encoding_option_index, quality_option_index)
