@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2012 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,20 +22,21 @@
 Module contains objects and methods needed to create render consumers.
 """
 
-
-from gi.repository import Gdk
-
-import mlt
+try:
+    import mlt7 as mlt
+except:
+    import mlt
 import time
 import threading
 import xml.dom.minidom
 import os
-# Jan-2017 - SvdB
-import editorpersistance
+import subprocess
 
+import appconsts
+import editorpersistance
 import mltenv
 import respaths
-
+import translations
 
 # File describing existing encoding and quality options
 RENDER_ENCODING_FILE = "/res/render/renderencoding.xml"
@@ -60,8 +61,33 @@ QUALITY = "quality"
 BITRATE = "bitrate"
 AUDIO_DESCRIPTION = "audiodesc"
 NON_USER = "nonuser"
+FFMPEG_GPU_ENC = "ffmpeggpuenc"
+PRESET_GROUP = appconsts.PRESET_GROUP
+PRESET_GROUP_H264 = appconsts.PRESET_GROUP_H264
+PRESET_GROUP_NVENC = appconsts.PRESET_GROUP_NVENC 
+PRESET_GROUP_VAAPI = appconsts.PRESET_GROUP_VAAPI 
+PRESET_GROUP_MPEG = appconsts.PRESET_GROUP_MPEG 
+PRESET_GROUP_LOSSLESS = appconsts.PRESET_GROUP_LOSSLESS
+PRESET_GROUP_IMAGE_SEQUENCE = appconsts.PRESET_GROUP_IMAGE_SEQUENCE
+PRESET_GROUP_AUDIO = appconsts.PRESET_GROUP_AUDIO
+PRESET_GROUP_MISC = appconsts.PRESET_GROUP_MISC 
+PRESET_GROUP_ALPHA = appconsts.PRESET_GROUP_ALPHA 
 
-# Replace strings and attribute values
+# ffmpeg arg values sometimes need equals signs in them.
+EQUALS_SIGN_ENCODING = "@#@#"
+
+# GPU encoding availability.
+FFMPEG_TEST = ["ffmpeg", "-version"]
+
+# These are filled here with all possible GPU encodings, and then rendergputest.py 
+# uses these to test and make available GPU encodings.
+NVENC_encs = []
+VAAPI_encs = []
+
+# Default encoding name.
+DEFAULT_ENCODING_NAME = "H.264 / .mp4" 
+
+# Replace strings and attribute values.
 BITRATE_RPL = "%BITRATE%"
 VARIABLE_VAL = "%VARIABLE%"
 SCREEN_SIZE_RPL = "%SCREENSIZE%"
@@ -69,13 +95,14 @@ ASPECT_RPL = "%ASPECT%"
 
 render_encoding_doc = None
 encoding_options = []
+categorized_encoding_options = []
 not_supported_encoding_options = []
 quality_option_groups = {}
 quality_option_groups_default_index = {}
 non_user_encodings = []
 proxy_encodings = None
 
-# This is used to turn performance settings off for proxy rendering
+# This is used to turn performance settings off for proxy rendering.
 performance_settings_enabled = True
 
 # replace empty strings with None values
@@ -85,7 +112,14 @@ def _get_attribute(node, attr_name):
         return None
     
     return value
-    
+
+def get_encoding_index(encoding):
+    for i in range(0, len(encoding_options)):
+        if encoding == encoding_options[i]:
+            return i
+            
+    return -1
+
 class QualityOption:
     """
     A render quality option for an EncodingOption.
@@ -122,11 +156,13 @@ class EncodingOption:
     def __init__(self, option_node):
         self.name = _get_attribute(option_node, NAME)
         self.type = _get_attribute(option_node, TYPE)
+        self.presetgroup = _get_attribute(option_node, PRESET_GROUP)
         self.resizable = (_get_attribute(option_node, RESIZABLE) == "True")
         self.extension = _get_attribute(option_node, EXTENSION)
         self.nonuser = _get_attribute(option_node, NON_USER)
         self.quality_qroup_id = _get_attribute(option_node, QGROUP)
         self.quality_options = quality_option_groups[self.quality_qroup_id]
+        self.ffmpeggpuenc =  _get_attribute(option_node, FFMPEG_GPU_ENC)
         try:
             quality_default_index = int(quality_option_groups_default_index[self.quality_qroup_id])
         except KeyError:
@@ -162,6 +198,8 @@ class EncodingOption:
             token_sides = token.split("=")
             arg1 = str(token_sides[0])
             arg2 = str(token_sides[1])
+            # Sometimes arg values need equals signs in them
+            arg2 = arg2.replace(EQUALS_SIGN_ENCODING, "=")
             
             # Replace keyword values
             if arg2 == SCREEN_SIZE_RPL:
@@ -184,7 +222,7 @@ class EncodingOption:
             desc = self.audio_desc 
         return "<small>" + desc + "</small>"
 
-    
+        
 def load_render_profiles():
     """
     Load render profiles from xml into DOM at start-up and build
@@ -195,6 +233,12 @@ def load_render_profiles():
     global render_encoding_doc
     render_encoding_doc = xml.dom.minidom.parse(file_path)
 
+    ret_code = _test_command(FFMPEG_TEST, True)
+    if (ret_code == 0):
+        print("ffmpeg CLI available")
+    else:
+        print("ffmpeg CLI NOT available")
+
     # Create quality option groups
     global quality_option_groups
     qgroup_nodes = render_encoding_doc.getElementsByTagName(QUALITY_GROUP)
@@ -221,9 +265,56 @@ def load_render_profiles():
             else:
                 non_user_encodings.append(encoding_option) 
         else:
-            msg = "...NOT available, " + encoding_option.err_msg + " missing"
             not_supported_encoding_options.append(encoding_option)
-            #print encoding_option.name + msg
+
+    # Create categorised structure.
+    global categorized_encoding_options, NVENC_encs, VAAPI_encs
+    H264_encs = []
+    NVENC_encs = []
+    VAAPI_encs = []
+    MPEG_encs = []
+    OGG_ETC_encs = []
+    LOSSLESS_encs = []
+    IMG_SEQ_encs = []
+    AUDIO_encs = []
+    ALPHA_encs = []
+    
+    for enc in encoding_options:
+        if enc.presetgroup == PRESET_GROUP_H264:
+            H264_encs.append((enc.name, enc))
+        elif enc.presetgroup == PRESET_GROUP_NVENC:
+            NVENC_encs.append((enc.name, enc))
+        elif enc.presetgroup == PRESET_GROUP_VAAPI:
+            VAAPI_encs.append((enc.name, enc))
+        elif enc.presetgroup == PRESET_GROUP_MPEG:
+            MPEG_encs.append((enc.name, enc))
+        elif enc.presetgroup == PRESET_GROUP_MISC:
+            OGG_ETC_encs.append((enc.name, enc))
+        elif enc.presetgroup == PRESET_GROUP_LOSSLESS:
+            LOSSLESS_encs.append((enc.name, enc))
+        elif enc.presetgroup == PRESET_GROUP_IMAGE_SEQUENCE:
+            IMG_SEQ_encs.append((enc.name, enc))
+        elif enc.presetgroup == PRESET_GROUP_AUDIO:
+            AUDIO_encs.append((enc.name, enc))
+        elif enc.presetgroup == PRESET_GROUP_ALPHA:
+            ALPHA_encs.append((enc.name, enc))
+
+    if len(H264_encs) > 0:
+        categorized_encoding_options.append((translations.get_encoder_group_name(PRESET_GROUP_H264), H264_encs))
+    if len(MPEG_encs) > 0:
+        categorized_encoding_options.append((translations.get_encoder_group_name(PRESET_GROUP_MPEG), MPEG_encs))
+    if len(OGG_ETC_encs) > 0:
+        categorized_encoding_options.append((translations.get_encoder_group_name(PRESET_GROUP_MISC), OGG_ETC_encs))
+    if len(LOSSLESS_encs) > 0:
+        categorized_encoding_options.append((translations.get_encoder_group_name(PRESET_GROUP_LOSSLESS), LOSSLESS_encs))
+    if len(IMG_SEQ_encs) > 0:
+        categorized_encoding_options.append((translations.get_encoder_group_name(PRESET_GROUP_IMAGE_SEQUENCE), IMG_SEQ_encs))
+    if len(ALPHA_encs) > 0:
+        categorized_encoding_options.append((translations.get_encoder_group_name(PRESET_GROUP_ALPHA), ALPHA_encs))
+    if len(AUDIO_encs) > 0:
+        categorized_encoding_options.append((translations.get_encoder_group_name(PRESET_GROUP_AUDIO), AUDIO_encs))
+
+    # If GPU rendering available it is added from rendergputest.py
 
     # Proxy encoding
     proxy_encoding_nodes = render_encoding_doc.getElementsByTagName(PROXY_ENCODING_OPTION)
@@ -231,19 +322,36 @@ def load_render_profiles():
     for proxy_node in proxy_encoding_nodes:
         proxy_encoding_option = EncodingOption(proxy_node)
         if proxy_encoding_option.supported:
-            msg = " ...available"
             found_proxy_encodings.append(proxy_encoding_option)
         else:
-            msg = " ...NOT available, " + encoding_option.err_msg + " missing"
-        #print "Proxy encoding " + proxy_encoding_option.name + msg
+            print("proxy encoding " + proxy_encoding_option.name + " NOT AVAILABLE.")
+
     global proxy_encodings
     proxy_encodings = found_proxy_encodings
 
+def remove_non_working_proxy_encodings(vcodec):
+    global proxy_encodings
+    proxy_encodings = [proxy_enc_opt for proxy_enc_opt in proxy_encodings if proxy_enc_opt.ffmpeggpuenc != vcodec]
+
+def _test_command(bash_args_list, print_output=False):
+    if print_output == False:
+        process = subprocess.Popen(bash_args_list, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
+    else:
+        process = subprocess.Popen(bash_args_list)
+    
+    out, err = process.communicate()
+    
+    if print_output == True:
+        print(bash_args_list)
+        print("return code:", process.returncode)
+
+    return process.returncode
+
 def get_default_render_consumer(file_path, profile):
     return get_render_consumer_for_encoding_and_quality(file_path, profile, 0, 10) # values get their meaning from /res/renderencoding.xml
                                                                                     # first <encodingoption> with 10th quality option
                                                                                     # should be H.264 with 10000 kb/s
-    
+
 def get_render_consumer_for_encoding_and_quality(file_path, profile, enc_opt_index, quality_opt_index):
     args_vals_list = get_args_vals_tuples_list_for_encoding_and_quality(profile,
                                                                        enc_opt_index,
@@ -341,7 +449,7 @@ def get_args_vals_tuples_list_for_encoding_and_quality(profile, enc_opt_index, q
         quality_option = None
 
     args_vals_list = encoding_option.get_args_vals_tuples_list(profile, quality_option)
-
+    
     # Quality options  key, value list
     if quality_option != None:
         for k, v in quality_option.add_map.items():
@@ -384,9 +492,16 @@ def _parse_line(line_start, line_end, buf):
         return (None, _("No \'=\' found."))
     sides = line.split("=")
     if len(sides) != 2:
-        return (None, _("Number of tokens on line is ")+ str(len(sides)) + _(", should be 2 (key, value)."))
-    k = sides[0].strip()
-    v = sides[1].strip()
+        k = sides[0].strip()
+        rest = sides[1:len(sides)]
+        v = ""
+        for token in rest:
+            v = v + token + "="
+        v = v.strip("=")
+        v = v.strip()
+    else:
+        k = sides[0].strip()
+        v = sides[1].strip()
     if len(k) == 0:
         return (None, _("Arg name token is empty."))
     if len(v) == 0:
@@ -395,9 +510,20 @@ def _parse_line(line_start, line_end, buf):
         return (None,  _("Whitespace in Arg name."))
     if v.find(" ") != -1:
         return (None,  _("Whitespace in Arg value."))
-        
+
     return ((k,v), None)
 
+# Convenience function needed because FileRenderPlayer no longer stops on last
+# frame with 'wait_for_producer_end_stop' set True and naked producer as producer.
+# With tractor we get full length rendered and player stops correctly.
+def get_producer_as_tractor(producer, last_frame):
+    tractor = mlt.Tractor()
+    multitrack = tractor.multitrack()
+    track0 = mlt.Playlist(producer.profile())
+    multitrack.connect(track0, 0)
+    track0.insert(producer, 0, 0, last_frame)
+    return tractor
+            
 
 class FileRenderPlayer(threading.Thread):
     def __init__(self, file_name, producer, consumer, start_frame, stop_frame):
@@ -410,32 +536,40 @@ class FileRenderPlayer(threading.Thread):
         self.wait_for_producer_end_stop = True
         self.running = False
         self.has_started_running = False
+        self.do_consumer_position_wait = True
         print("FileRenderPlayer started, start frame: " + str(self.start_frame) + ", stop frame: " + str(self.stop_frame))
         self.consumer_pos_stop_add = 1 # HACK!!! File renders work then this is one, screenshot render requires this to be 2 to work 
         threading.Thread.__init__(self)
 
     def run(self):
+        
+        self.consumer.set("plays", 1) # maybe not strictly necessary but default value here seems to  be 'None' which is wrong.
+        
         self.running = True
         self.has_started_running = True
         self.connect_and_start()
 
         while self.running: # set false at shutdown() for abort
+
             if self.producer.frame() >= self.stop_frame:
                 # This method of stopping makes sure that whole producer is rendered and written to disk
                 # Used when producer out frame is last frame.
                 if self.wait_for_producer_end_stop:
+
                     while self.producer.get_speed() > 0:
                         time.sleep(0.2)
                     while not self.consumer.is_stopped():
                         time.sleep(0.2)
+                    
                 # This method of stopping stops producer
                 # and waits for consumer to reach that frame.
                 # Used when producer out frame is NOT last frame.
                 else:
                     self.producer.set_speed(0)
                     last_frame = self.producer.frame()
-                    while self.consumer.position() + self.consumer_pos_stop_add < last_frame:
-                        time.sleep(0.2)
+                    if self.do_consumer_position_wait == True:
+                        while self.consumer.position() + self.consumer_pos_stop_add < last_frame:
+                            time.sleep(0.2)
 
                     self.consumer.stop()
                                         
@@ -486,16 +620,10 @@ class XMLRenderPlayer(threading.Thread):
     def run(self):
         print("Starting XML render")
         player = self.player
-        
-        # Don't try anything if somehow this was started 
-        # while timeline rendering is running
-        if player.is_rendering:
-            print("Can't render XML when another render is already running!")
-            return
 
         # Stop all playback before producer is disconnected
         self.current_playback_frame = player.producer.frame()
-        player.ticker.stop_ticker()
+        player.stop_ticker()
         player.consumer.stop()
         player.producer.set_speed(0)
         player.producer.seek(0)
@@ -564,32 +692,3 @@ class XMLCompoundRenderPlayer(threading.Thread):
         print("XML compound clip render done")
 
         self.render_done_callback(self.file_name, self.media_name)
-
-
-class ProgressWindowThread(threading.Thread):
-    def __init__(self, dialog, progress_bar, clip_renderer, callback):
-        self.dialog = dialog
-        self.progress_bar = progress_bar
-        self.clip_renderer = clip_renderer
-        self.callback = callback
-        threading.Thread.__init__(self)
-    
-    def run(self):        
-        self.running = True
-        
-        while self.running:         
-            render_fraction = self.clip_renderer.get_render_fraction()
-            Gdk.threads_enter()
-            self.progress_bar.set_fraction(render_fraction)
-            pros = int(render_fraction * 100)
-            self.progress_bar.set_text(str(pros) + "%")
-            Gdk.threads_leave()
-            if self.clip_renderer.stopped == True:
-                Gdk.threads_enter()
-                self.progress_bar.set_fraction(1.0)
-                self.progress_bar.set_text("Render Complete!")
-                self.callback(self.dialog, 0)
-                Gdk.threads_leave()
-                self.running = False
-
-            time.sleep(0.33)
