@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2012 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,8 +22,10 @@
 Module contains class Sequence that is the multitrack media object being edited
 by the application. A project has 1-n of these.
 """
-import time
-import mlt
+try:
+    import mlt7 as mlt
+except:
+    import mlt
 import os
 
 import appconsts
@@ -33,7 +35,7 @@ import mltfilters
 import mlttransitions
 import mltrefhold
 import patternproducer
-import tlinerender
+import tlineypage
 import utils
 
 # Media types for tracks or clips
@@ -97,10 +99,10 @@ class Sequence:
     """
     Multitrack MLT object
     """
-    def __init__(self, profile, name="sequence"):
+    def __init__(self, profile, initial_comp_mode):
 
         # Data members
-        self.name = name # name of sequence
+        self.name = "sequence" # initial name of sequence
         self.next_id = 0 # id for next created clip
         self.profile = profile
         self.master_audio_gain = 1.0
@@ -113,11 +115,11 @@ class Sequence:
         self.watermark_filter = None
         self.watermark_file_path = None
         self.seq_len = 0 # used in trim crash hack, remove when fixed
-        self.compositing_mode = appconsts.COMPOSITING_MODE_TOP_DOWN_FREE_MOVE
+        self.compositing_mode = initial_comp_mode
 
         # MLT objects for a multitrack sequence
         self.init_mlt_objects()
-
+        
     # ----------------------------------- mlt init
     def init_mlt_objects(self):
         # MLT objects for multitrack sequence
@@ -204,6 +206,10 @@ class Sequence:
         self.tracks[0].clips.append(black_track_clip) # py
         self.tracks[0].append(black_track_clip, 0, 0) # mlt
 
+        # Create fulltrack compositors if needed.
+        if self.compositing_mode == appconsts.COMPOSITING_MODE_STANDARD_FULL_TRACK:
+            self.add_full_track_compositors()
+            
     def _create_black_track_clip(self):
         # Create 1 fr long black bg clip and set in and out
         global black_track_clip
@@ -231,7 +237,7 @@ class Sequence:
         Creates a MLT playlist object, adds project
         data and adds to tracks list.
         """
-        new_track = mlt.Playlist()
+        new_track = mlt.Playlist(self.profile)
 
         self._add_track_attributes(new_track, track_type)
         new_track.is_sync_track = False
@@ -324,7 +330,7 @@ class Sequence:
             track = self.tracks[i]
             track.height = TRACK_HEIGHT_NORMAL
     
-        self.resize_tracks_to_fit(allocation)
+        tlineypage.vertical_size_update(allocation)
 
     def maximize_video_tracks_height(self, allocation):
         self.minimize_tracks_height()
@@ -332,7 +338,7 @@ class Sequence:
             track = self.tracks[i]
             track.height = TRACK_HEIGHT_NORMAL
     
-        self.resize_tracks_to_fit(allocation)
+        tlineypage.vertical_size_update(allocation)
 
     def maximize_audio_tracks_height(self, allocation):
         self.minimize_tracks_height()
@@ -340,7 +346,7 @@ class Sequence:
             track = self.tracks[i]
             track.height = TRACK_HEIGHT_NORMAL
     
-        self.resize_tracks_to_fit(allocation)
+        tlineypage.vertical_size_update(allocation)
 
     def get_tracks_height(self):
         h = 0
@@ -371,10 +377,14 @@ class Sequence:
         pan_filter.set("start", value)
         track.attach(pan_filter)
         track.pan_filter = pan_filter 
-
+            
     def set_track_pan_value(self, track, value):
-        track.pan_filter.set("start", str(value))
-        track.audio_pan = value
+        if hasattr(track, "pan_filter") == False or track.pan_filter == None:
+            self.add_track_pan_filter(track, value)
+            track.audio_pan = value
+        else:
+            track.pan_filter.set("start", str(value))
+            track.audio_pan = value
     
     def remove_track_pan_filter(self, track):
         # This method is used for master too, and called with tractor then
@@ -434,6 +444,8 @@ class Sequence:
         if ttl != None:
             producer.set("ttl", str(ttl))
 
+        producer.set("mute_on_pause", str(1))
+
         return producer
 
     def create_slowmotion_producer(self, path, speed):
@@ -494,7 +506,7 @@ class Sequence:
     def clone_track_clip(self, track, index):
         orig_clip = track.clips[index]
         if orig_clip.is_blanck_clip == True:
-            # Blank clips are created by adding a blank clip into MLT track and that is not approprite here,
+            # Blank clips are created by adding a blank clip into MLT track and that is not appropriate here,
             # so blank clips are represent by their length as int.
             return orig_clip.clip_length()
         return self.create_clone_clip(orig_clip)
@@ -608,11 +620,6 @@ class Sequence:
         return compositor
 
     def restack_compositors(self):
-        if self.compositing_mode == appconsts.COMPOSITING_MODE_STANDARD_FULL_TRACK:
-            # we should only see this on sequence creation and adding removing tracks for COMPOSITING_MODE_STANDARD_FULL_TRACK
-            # remove this later
-            print("restacking compositors!")
-
         self.sort_compositors()
 
         new_compositors = []
@@ -689,18 +696,18 @@ class Sequence:
         self.field.disconnect_service(old_compositor.transition.mlt_transition)
 
     def destroy_compositors(self):
-        # This can be called when undo stack destroyd too.
+        # This can be called when undo stack destroyed too.
         for compositor in self.compositors:
             self.field.disconnect_service(compositor.transition.mlt_transition)
         self.compositors = []
 
     def add_full_track_compositors(self):
-        print("Adding full track compositors")
+        #print("Adding full track compositors")
         
         for i in range(self.first_video_index, len(self.tracks) - 1):
             track = self.tracks[i]
         
-            compositor = self.create_compositor("##affineblend")
+            compositor = self.create_compositor("##blend")
 
             a_track = self.first_video_index
             b_track = track.id 
@@ -712,7 +719,7 @@ class Sequence:
             self.add_compositor(compositor)
         
         self.restack_compositors()
-        print("Adding full track compositors DONE")
+        #print("Adding full track compositors DONE")
     
     def get_compositor_for_destroy_id(self, destroy_id):
         for comp in self.compositors:
@@ -794,7 +801,7 @@ class Sequence:
 
     def hide_hidden_clips(self):
         """
-        Called to temporarely remove hidden clips for trim mode loop playback
+        Called to temporarily remove hidden clips for trim mode loop playback
         """
         self.tracks[-1].clear()
         self._unmute_editable()
@@ -837,7 +844,7 @@ class Sequence:
         self._unmute_editable()
 
     def update_edit_tracks_length(self):
-        # Needed for timeline renderering updates
+        # Needed for timeline rendering updates
         self.seq_len = 0 
         for i in range(1, len(self.tracks) - 1):
             track_len = self.tracks[i].get_length()
@@ -852,11 +859,9 @@ class Sequence:
         seq_len = self.seq_len
         if seq_len < 1:
             seq_len = 1
-        
-        tlinerender.get_renderer().update_hidden_track(self.tracks[-1], seq_len)
 
     def fix_v1_for_render(self): 
-        # This is a workaround to fix Issue #941 with H248 encoder not being able handle 
+        # This is a workaround to fix Issue #941 with H248 encoder not being able to handle 
         # blanks and crashing or losing working audio. Underlying reason still 
         # not known.
         track_v1 = self.tracks[self.first_video_index]
@@ -876,7 +881,7 @@ class Sequence:
             edit.append_clip(track_v1, white_clip, white_clip.clip_in, white_clip.clip_out)
 
     def get_seq_range_frame(self, frame):
-        # Needed for timeline renderering updates
+        # Needed for timeline rendering updates
         if frame >= (self.seq_len - 1):
             return self.seq_len - 1
         else:
@@ -973,7 +978,7 @@ class Sequence:
                 track_id += 1
                 continue
             elif track_id == len(self.tracks) - 2:
-                # This shold not happen because track heights should be set up so that minimized app 
+                # This should not happen because track heights should be set up so that minimized app 
                 fix_next = False
                 print("sequence.resize_tracks_to_fit (): could not make tracks fit in timeline vertical space")
             else:
@@ -1046,7 +1051,85 @@ class Sequence:
                 cut_frame = prev_cut_frame
             
         return cut_frame
-    
+
+    def find_next_editable_clip_and_track(self, tline_frame):
+        """
+        Returns next selectable clip and track.
+        """
+        cut_frame = -1
+        clip_track = None
+        select_clip = None
+        for i in range(1, len(self.tracks) - 1):
+            track = self.tracks[i]
+            
+            # Get index and clip
+            index = track.get_clip_index_at(tline_frame)
+            try:
+                clip = track.clips[index]
+                clip_start_in_tline = track.clip_start(index)
+                # We are looking for media clips only and after tline_frame.
+                while clip.is_blanck_clip == True or clip_start_in_tline < tline_frame:
+                    clip = track.clips[index + 1]
+                    clip_start_in_tline = track.clip_start(index + 1)
+                    index = index + 1
+            except Exception as e:
+                continue # No selectable clip on track after frame
+
+            # Get next cut frame
+            clip_start_in_tline = track.clip_start(index)
+            length = clip.clip_out - clip.clip_in 
+            next_cut_frame = clip_start_in_tline + length + 1 # +1 clip out inclusive
+                     
+            # Set cut frame
+            if cut_frame == -1 and next_cut_frame > tline_frame:
+                cut_frame = next_cut_frame
+                clip_track = track
+                select_clip = clip
+            elif next_cut_frame < cut_frame and next_cut_frame > tline_frame:
+                cut_frame = next_cut_frame
+                clip_track = track
+                select_clip = clip
+
+        return (select_clip, clip_track)
+
+    def find_prev_editable_clip_and_track(self, tline_frame):
+        """
+        Returns prev selectable clip and track.
+        """
+        cut_frame = -1
+        clip_track = None
+        select_clip = None
+        for i in range(1, len(self.tracks) - 1):
+            track = self.tracks[i]
+            
+            # Get index and clip
+            index = track.get_clip_index_at(tline_frame)
+            try:
+                clip = track.clips[index]
+                clip_start_in_tline = track.clip_start(index)
+                # We are looking for media clips only and before tline_frame.
+                while clip.is_blanck_clip == True or clip_start_in_tline > tline_frame:
+                    clip = track.clips[index - 1]
+                    clip_start_in_tline = track.clip_start(index - 1)
+                    index = index - 1
+            except Exception as e:
+                continue # No selectable clip on track before frame
+
+            # Get prev cut frame
+            prev_cut_frame = clip_start_in_tline
+                     
+            # Set cut frame
+            if cut_frame == -1 and prev_cut_frame < tline_frame:
+                cut_frame = prev_cut_frame
+                clip_track = track
+                select_clip = clip
+            elif prev_cut_frame > cut_frame and prev_cut_frame < tline_frame:
+                cut_frame = prev_cut_frame
+                clip_track = track
+                select_clip = clip
+
+        return (select_clip, clip_track)
+
     def get_closest_cut_frame(self, track_id, frame):
         track = self.tracks[track_id]
         index = track.get_clip_index_at(frame)
@@ -1161,6 +1244,24 @@ class Sequence:
                 if clip.is_blanck_clip == False:
                     clip.waveform_data = None
 
+    # ----------------------------------- tracks auto expand feature
+    def get_tracks_clips_counts(self):
+        clips_counts = []
+        for i in range(1, len(self.tracks) - 1): # hidden tracks
+            clips_counts.append(len(self.tracks[i].clips))
+ 
+        return clips_counts
+    
+    def get_inital_drop_target_track(self, tracks_clips_count_before):
+        current_tracks_clips_count = self.get_tracks_clips_counts()
+        for i in range(0, len(current_tracks_clips_count)):
+            if tracks_clips_count_before[i] == 0:
+                if current_tracks_clips_count[i] > 0:
+                    return i + 1 # lists don't include bottom bg track.
+
+        return None
+
+    # ----------------------------------- info pronts
     def print_all(self):
         print("------------------------######")
         for i in range(0, len(self.tracks)):
@@ -1199,35 +1300,7 @@ def get_media_type(file_path):
     Returns media type of file.
     """
     return utils.get_media_type(file_path)
-    """
-    if os.path.exists(file_path):
-        mime_type = utils.get_file_type(file_path)
-    else:
-        # IMAGE_SEQUENCE media objects have a MLT formatted resource path that does not
-        # point to an existing file in the file system. 
-        # We're doing a heuristic here to identify those.
-        pros_index = file_path.find("%0")
-        d_index = file_path.find("d.")
-        if pros_index != -1 and d_index != -1:
-            return IMAGE_SEQUENCE
-        all_index = file_path.find(".all")
-        if all_index != -1:
-            return IMAGE_SEQUENCE
-            
-        return FILE_DOES_NOT_EXIST
-        
-    if mime_type.startswith("video"):
-        return VIDEO
-    
-    if mime_type.startswith("audio"):
-        return AUDIO
-    
-    if mime_type.startswith("image"):
-        return IMAGE
-    
-    return UNKNOWN
-    """
-    
+
 def _clip_length(clip):
     return clip.clip_out - clip.clip_in + 1
 
@@ -1236,27 +1309,25 @@ def _sort_compositors_comparator(a_comp):
 
 # ----------------------------- sequence cloning for tracks count change
 def create_sequence_clone_with_different_track_count(old_seq, v_tracks, a_tracks):
-    # Create new sequence with different number of tracks
+    # Create new sequence with different number of tracks.
     global AUDIO_TRACKS_COUNT, VIDEO_TRACKS_COUNT
     AUDIO_TRACKS_COUNT = a_tracks
     VIDEO_TRACKS_COUNT = v_tracks
-    new_seq = Sequence(old_seq.profile, old_seq.name)
+    new_seq = Sequence(old_seq.profile, old_seq.compositing_mode)
     new_seq.create_default_tracks()
-
-    # Clone track clips from old sequence to clone sequence
+    new_seq.name = old_seq.name
+ 
+    # Clone track clips from old sequence to clone sequence.
     if old_seq.first_video_index - 1 > a_tracks:
         _clone_for_fewer_tracks(old_seq, new_seq)
     else:
         _clone_for_more_tracks(old_seq, new_seq)
 
-    # Clone compositors from old seq to new to correct tracks on new seq
+    # Clone compositors from old seq to new to correct tracks on new seq.
     track_delta = new_seq.first_video_index - old_seq.first_video_index
     new_seq.clone_compositors_from_sequence(old_seq, track_delta)
-
-    # Copy modes values
-    new_seq.compositing_mode = old_seq.compositing_mode
         
-    # copy next clip id data
+    # Copy next clip id data.
     new_seq.next_id = old_seq.next_id
     return new_seq
         
