@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2013 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -69,8 +69,6 @@ def load_shortcut_files():
             if root.tag == appconsts.SHORTCUTS_ROOT_TAG:
                 # Check if this is a shortcuts file
                 if root.get('file') == appconsts.SHORTCUTS_TAG:
-                    # Get name and comments
-                    file_len = len(f) - 4
                     # Default file is added last to always be at index 0
                     if f != DEFAULT_SHORTCUTS_FILE:
                         shortcut_files.append(f)
@@ -89,7 +87,7 @@ def load_shortcut_files():
             print("Shortcuts file " + f + " found, but has incorrect format.")
     
     # Default shortcuts file always goes to index 0
-    if default_shortcuts_file_found == True:# this is a bit unnecessery, it is there unless someone destroys it manually
+    if default_shortcuts_file_found == True:# this is a bit unnecessary, it is there unless someone destroys it manually
         shortcut_files.insert(0, DEFAULT_SHORTCUTS_FILE)
         shortcut_files_display_names.insert(0, "Flowblade Default")
 
@@ -153,6 +151,46 @@ def set_keyboard_shortcuts():
 
     #_print_shortcuts()
 
+
+def update_custom_shortcuts():
+    # If new shortcuts have been added and user is using custom shortcuts when updating, we need to update custom shortcuts.
+    custom_files = os.listdir(userfolders.get_data_dir() + "/" + appconsts.USER_SHORTCUTS_DIR)
+    for custom_prefs_file in custom_files:
+        _update_custom_xml_file_nodes_to_default(_get_shortcut_file_fullpath(custom_prefs_file))
+
+def _update_custom_xml_file_nodes_to_default(custom_xml_file_path):
+    pref_shortcuts = etree.parse(custom_xml_file_path)
+    default_shortcuts = etree.parse(_get_shortcut_file_fullpath(DEFAULT_SHORTCUTS_FILE))
+    pref_root = pref_shortcuts.getroot()
+    default_root = default_shortcuts.getroot()
+    pref_dict = _get_events_dict(pref_root)
+    default_dict = _get_events_dict(default_root)
+
+    changed = False
+    for code in default_dict:
+        try:
+            pref_event = pref_dict[code]
+        except:
+            print( "Adding missing keyevent ", code, " to custom shortcuts file ", custom_xml_file_path)
+            default_event = default_dict[code]
+            new_pref_event = etree.fromstring(etree.tostring(default_event))
+            pref_shortcuts_node = pref_root.find("shortcuts")
+            pref_shortcuts_node.append(new_pref_event)
+            changed = True
+    
+    if changed == True:
+        print("Writing ", custom_xml_file_path)
+        pref_shortcuts.write(custom_xml_file_path)
+
+# code -> xml.etree.ElementTree.Element
+def _get_events_dict(xml_root):
+    events_dict = {}
+    events = xml_root.iter('event')
+    for event in events:
+        events_dict[event.get('code')] = event
+
+    return events_dict
+
 def get_root():
     shortcuts = etree.parse(_get_shortcut_file_fullpath(editorpersistance.prefs.shortcuts))
     return shortcuts.getroot()
@@ -215,12 +253,21 @@ def delete_active_custom_shortcuts_xml():
     editorpersistance.prefs.shortcuts = DEFAULT_SHORTCUTS_FILE
     editorpersistance.save()
 
-def change_custom_shortcut(code, key_val_name, mods_list):
+def change_custom_shortcut(code, key_val_name, mods_list, add_event=False):
     shortcuts_file = _get_shortcut_file_fullpath(editorpersistance.prefs.shortcuts)
     shortcuts = etree.parse(shortcuts_file)
     root = shortcuts.getroot()
-    events = root.iter('event')
 
+    # Add new element if so ordered.
+    if add_event == True:
+        new_event = etree.Element("event")    
+        new_event.text = key_val_name
+        new_event.set('code', code)
+        shortcuts_node = root.find("shortcuts")
+        shortcuts_node.append(new_event)
+        
+    events = root.iter('event')
+    
     target_event = None
     for event in events:
         if event.get('code') == code:
@@ -251,7 +298,6 @@ def change_custom_shortcut(code, key_val_name, mods_list):
 
 def is_blocked_shortcut(key_val, mods_list):
     for reserved in RESERVED_SHORTCUTS:
-        print(reserved)
         r_key_val, r_mods_list = reserved
         if len(r_mods_list) == 0 and key_val == r_key_val:
             return True
@@ -284,7 +330,6 @@ def get_shortcut_gtk_code(root, code):
                 gtk_code += _gtk_mod_names[mod]
                 
             gtk_code += event.text.upper()
-            #gtk_code = "'" + gtk_code + "'"
             return gtk_code
 
     return None
@@ -296,20 +341,6 @@ def _get_mod_string(event):
     
     return _mod_names[mod]
 
-def get_diff_to_defaults(xml_file):
-    diff_str = ""
-    test_root = get_shortcuts_xml_root_node(xml_file)
-    def_root = get_shortcuts_xml_root_node(DEFAULT_SHORTCUTS_FILE)
-    
-    for code, action_name in _keyboard_action_names.items():
-        key_name_test, action_name = get_shortcut_info(test_root, code)
-        key_name_def, action_name = get_shortcut_info(def_root , code)
-    
-        if key_name_def != key_name_test:
-            diff_str = diff_str + action_name + " (" + key_name_test + ")    "
-    
-    return diff_str
-
 def _get_shortcut_file_fullpath(f):
     full_path = respaths.SHORTCUTS_PATH + f
     if os.path.isfile(full_path) == False:
@@ -337,7 +368,8 @@ def _set_keyboard_action_names():
     _keyboard_action_names['stop'] = _("Stop")
     _keyboard_action_names['faster'] =  _("Forward Faster")
     _keyboard_action_names['log_range'] = _("Log Marked Clip Range")
-    _keyboard_action_names['resync'] = _("Resync selected Clip or Compositor")
+    _keyboard_action_names['split_selected'] = _("Split Audio Synched")
+    _keyboard_action_names['resync'] = _("Resync Track")
     _keyboard_action_names['delete'] = _("Delete Selected Item")
     _keyboard_action_names['lift'] = _("Lift Selected Item")
     _keyboard_action_names['to_start'] = _("Go To Start")
@@ -366,6 +398,15 @@ def _set_keyboard_action_names():
     _keyboard_action_names['open_next'] =  _("Open Next Media Item In Monitor")
     _keyboard_action_names['clear_filters'] = _("Clear Filters")
     _keyboard_action_names['sync_all'] = _("Sync All Compositors")
+    _keyboard_action_names['select_next'] = _("Open Next Clip In Filter Editor")
+    _keyboard_action_names['select_prev'] = _("Open Previous Clip In Filter Editor")
+    _keyboard_action_names['play_pause_loop_marks'] = _("Play / Pause Mark In to Mark Out Loop")
+    _keyboard_action_names['trim_start'] = _("Trim Clip Start To Playhead")
+    _keyboard_action_names['trim_end'] = _("Trim Clip End To Playhead")
+    _keyboard_action_names['tline_render_request'] = _("Render Timeline Range")
+    _keyboard_action_names['toggle_track_output'] = _('Enable/Disable Selected Clip Track Output')
+    _keyboard_action_names['tline_page_up'] = _('Timeline Vertical Page Up')
+    _keyboard_action_names['tline_page_down'] = _('Timeline Vertical Page Down')
 
 def _set_key_names():
     global _key_names, _mod_names, _gtk_mod_names
@@ -434,7 +475,9 @@ def _set_key_names():
     _key_names['equal'] = _("=")
     _key_names['comma'] = _(",")
     _key_names['period'] = _(".")
-    
+    _key_names['page_up'] = _("Page Up")
+    _key_names['page_down'] = _("Page Down")
+
     _mod_names["ALT"] = _("Alt")
     _mod_names["SHIFT"] =  _("Shift")
     _mod_names["ALT+SHIFT"] = _("Alt + Shift")
