@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2012 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -32,22 +32,69 @@ from editorstate import current_sequence
 from editorstate import get_track
 import gui
 import movemodes
+import resync
 import tlinewidgets
 import updater
 import utils
 
-# NOTE: THIS AND resync.py SHOULD PROBABLY BE THE SAME MODULE
 
 parent_selection_data = None
 
+
 # ----------------------------------- split audio
 def split_audio_synched(popup_data):
     """
-    We do two separate edits to do this, so if user undoes this he'll need
-    to two undos, which may not be to user expectation as doing this is only one edit 
+    We have do two separate edits to do this, because we need to have a clip on timeline 
+    to be able to sync it. So if user undoes this he'll need
+    to two undos, which may not be to user expectation as doing this is only one edit. 
     """
-    (parent_clip, child_clip, child_clip_track) = _do_split_audio_edit(popup_data)
+    clip, track, item_id, x = popup_data
+    
+    # We can only split audio from unmuted clips because splitting mutes audio
+    # and we want to avoid splitting audio twice.
+    if clip.mute_filter != None:
+        return
+        
+    split_action, parent_clip, child_clip, child_clip_track = _get_split_audio_edit_action(popup_data)
+    split_action.do_edit()
+    sync_action = _get_set_sync_action(child_clip_track, child_clip, parent_clip)
+    sync_action.do_edit()
 
+def split_audio_synched_from_clips_list(clips, track):
+    item_id = "not actually used"
+    split_actions_data = []
+    split_actions = []
+    for clip in clips:
+        # We're using the existing function to do this need x for clip frame to use it
+        index = track.clips.index(clip)
+        frame = track.clip_start(index)
+        x = tlinewidgets._get_frame_x(frame)
+
+        popup_data = (clip, track, item_id, x)
+
+        # We can only split audio from unmuted clips because splitting mutes audio
+        # and we want to avoid splitting audio twice.
+        if clip.mute_filter == None:
+            split_action, parent_clip, child_clip, child_clip_track = _get_split_audio_edit_action(popup_data)
+            split_actions.append(split_action)
+            split_actions_data.append((parent_clip, child_clip, child_clip_track))
+    
+    # We need to do split action before creating set sync actio0ns because we need a clip on timeline 
+    # to actually sync with.
+    split_consolidated_action = edit.ConsolidatedEditAction(split_actions)
+    split_consolidated_action.do_consolidated_edit()
+
+    set_sync_actions = []
+    for split_data in split_actions_data:
+        parent_clip, child_clip, child_clip_track = split_data
+        
+        sync_action = _get_set_sync_action(child_clip_track, child_clip, parent_clip)
+        set_sync_actions.append(sync_action)
+
+    sync_consolidated_action = edit.ConsolidatedEditAction(set_sync_actions)
+    sync_consolidated_action.do_consolidated_edit()
+    
+def _get_set_sync_action(child_clip_track, child_clip, parent_clip):
     # This is quarenteed because GUI option to do this is only available on this track
     parent_track = current_sequence().tracks[current_sequence().first_video_index]
     child_index = child_clip_track.clips.index(child_clip)
@@ -58,24 +105,46 @@ def split_audio_synched(popup_data):
             "parent_index":parent_clip_index,
             "parent_track":parent_track}
     action = edit.set_sync_action(data)
-    action.do_edit()
-
+    return action
+    #action.do_edit()
+    
 def split_audio(popup_data):
-    _do_split_audio_edit(popup_data)
+    clip, track, item_id, x = popup_data
+    
+    # We can only split audio from unmuted clips because splitting mutes audio
+    # and we want to avoid splitting audio twice.
+    if clip.mute_filter != None:
+        return
+        
+    action, clip, audio_clip, to_track = _get_split_audio_edit_action(popup_data)
+
+    action.do_edit()
 
 def split_audio_from_clips_list(clips, track):
     item_id = "not actually used"
+    actions_list = []
     for clip in clips:
-        # We're using the existing function to do thid need x for clip frame to use it
+        # We're using the existing function to do this need x for clip frame to use it
         index = track.clips.index(clip)
         frame = track.clip_start(index)
         x = tlinewidgets._get_frame_x(frame)
 
         popup_data = (clip, track, item_id, x)
-        _do_split_audio_edit(popup_data)
+
+        # We can only split audio from unmuted clips because splitting mutes audio
+        # and we want to avoid splitting audio twice.
+        if clip.mute_filter == None:
+            action, clip, audio_clip, to_track = _get_split_audio_edit_action(popup_data)
+            actions_list.append(action)
     
+    consolidated_action = edit.ConsolidatedEditAction(actions_list)
+    consolidated_action.do_consolidated_edit()
+
 def _do_split_audio_edit(popup_data):
-    # NOTE: THIS HARD CODES ALL SPLITS TO HAPPEN ON TRACK A1, THIS MAY CHANGE
+    return _get_split_audio_edit_action(popup_data)
+
+def _get_split_audio_edit_action(popup_data):
+    # NOTE: THIS HARD CODES ALL SPLITS TO HAPPEN TO TRACK A1, THIS MAY CHANGE
     to_track = current_sequence().tracks[current_sequence().first_video_index - 1]
 
     clip, track, item_id, x = popup_data
@@ -93,9 +162,8 @@ def _do_split_audio_edit(popup_data):
              "to_track":to_track}
 
     action = edit.audio_splice_action(data)
-    action.do_edit()
     
-    return (clip, audio_clip, to_track)
+    return (action, clip, audio_clip, to_track)
 
 # ---------------------------------------------- sync parent clips
 def init_select_master_clip(popup_data):
@@ -108,8 +176,8 @@ def init_select_master_clip(popup_data):
         print("big fu at _init_select_master_clip(...)")
         return
 
-    gdk_window = gui.tline_display.get_parent_window();
-    gdk_window.set_cursor(Gdk.Cursor.new(Gdk.CursorType.TCROSS))
+    gdk_window = gui.tline_display.get_parent_window()
+    gdk_window.set_cursor(Gdk.Cursor.new_for_display(Gdk.Display.get_default(), Gdk.CursorType.TCROSS))
     editorstate.edit_mode = editorstate.SELECT_PARENT_CLIP
     global parent_selection_data
     parent_selection_data = (clip, child_index, track)
@@ -117,8 +185,8 @@ def init_select_master_clip(popup_data):
 def select_sync_parent_mouse_pressed(event, frame):
     _set_sync_parent_clip(event, frame)
     
-    gdk_window = gui.tline_display.get_parent_window();
-    gdk_window.set_cursor(Gdk.Cursor.new(Gdk.CursorType.LEFT_PTR))
+    gdk_window = gui.tline_display.get_parent_window()
+    gdk_window.set_cursor(Gdk.Cursor.new_for_display(Gdk.Display.get_default(), Gdk.CursorType.LEFT_PTR))
    
     global parent_selection_data
     parent_selection_data = None
@@ -150,7 +218,7 @@ def _set_sync_parent_clip(event, frame):
         return
 
     # Parent and child can't be on the same track.
-    # Now that all parent clips must be on track V1 this is no longer shoild be possible.
+    # Now that all parent clips must be on track V1 this is no longer should be possible.
     if parent_track == child_clip_track:
         print("parent_track == child_clip_track")
         return
@@ -158,7 +226,7 @@ def _set_sync_parent_clip(event, frame):
     parent_clip = parent_track.clips[parent_clip_index]
     
     # These cannot be chained.
-    # Now that all parent clips must be on track V1 this is no longer shoild be possible.
+    # Now that all parent clips must be on track V1 this is no longer should be possible.
     if parent_clip.sync_data != None:
         print("parent_clip.sync_data != None")
         return
@@ -170,74 +238,46 @@ def _set_sync_parent_clip(event, frame):
     action = edit.set_sync_action(data)
     action.do_edit()
 
+def resync_clip_from_button():
+    track = get_track(movemodes.selected_track)
+    clip = track.clips[movemodes.selected_range_in]
+    _do_single_clip_resync(clip, track)
+
 def resync_clip(popup_data):
     clip, track, item_id, x = popup_data
-    clip_list=[(clip, track)]
+    _do_single_clip_resync(clip, track)
+
+def _do_single_clip_resync(clip, track):
+    clip_list = [(clip, track)]
     
+    resync_data_list = resync.get_resync_data_list_for_clip_list(clip_list)
+    if len(resync_data_list) == 0:
+        return # Parent clip is gone.
+
+    # Get sync data. If we're in sync, do nothing.
+    clip, track, index, child_clip_start, pos_offset = resync_data_list[0]
+    if pos_offset == clip.sync_data.pos_offset:
+        return None
+    
+    # Do resync.
     data = {"clips":clip_list}
-    action = edit.resync_some_clips_action(data)
+    action = edit.resync_clip_action(data)
     action.do_edit()
     
     updater.repaint_tline()
 
-def resync_everything():
-    # Selection not valid after resync action
-    if movemodes.selected_track == -1:
-        movemodes.clear_selected_clips()
-    
-    action = edit.resync_all_action({})
-    action.do_edit()
-    
-    updater.repaint_tline()
-
-def resync_selected():
+def resync_track():
     if movemodes.selected_track == -1:
         return
+    
+    resync_clips_data_list = resync.get_track_resync_clips_data_list(get_track(movemodes.selected_track))
+    
+    if len(resync_clips_data_list) == 0:
+        return
 
-    track = get_track(movemodes.selected_track)
-    clip_list = []
-    for index in range(movemodes.selected_range_in, movemodes.selected_range_out + 1):
-        clip_list.append((track.clips[index], track))
-
-    # Selection not valid after resync action
-    movemodes.clear_selected_clips()
-
-    # Chack if synced clips have same or consecutive parent clips
-    all_same_or_consecutive = True
-    master_id = -1
-    current_master_clip = -1
-    current_master_index = -1
-    master_track = current_sequence().first_video_track()
-    for t in clip_list:
-        clip, track = t
-        try:
-            if master_id == -1:
-                master_id = clip.sync_data.master_clip.id
-                current_master_clip = clip.sync_data.master_clip
-                current_master_index = master_track.clips.index(current_master_clip)
-            else:
-                if clip.sync_data.master_clip.id != master_id:
-                    next_master_index = master_track.clips.index(clip.sync_data.master_clip)
-                    if current_master_index + 1 == next_master_index:
-                        # Masters are consecutive, save data to test next
-                        master_id = clip.sync_data.master_clip.id
-                        current_master_index = master_track.clips.index(current_master_clip)
-                    else:
-                        all_same_or_consecutive = False
-        except:
-            all_same_or_consecutive = False
-
-    # If clips are all for same or consecutive sync parent clips, sync them as a unit.
-    if len(clip_list) > 1 and all_same_or_consecutive == True:
-        data = {"clips":clip_list}
-        action = edit.resync_clips_sequence_action(data)
-        action.do_edit()
-    else: # Single or non-consecutive clips are synched separately
-        data = {"clips":clip_list}
-        action = edit.resync_some_clips_action(data)
-        action.do_edit()
-
-    updater.repaint_tline()
+    data = {"resync_clips_data_list":resync_clips_data_list}
+    action = edit.resync_track_action(data)
+    action.do_edit()
 
 def clear_sync_relation(popup_data):
     clip, track, item_id, x = popup_data
