@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2012 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,7 +23,10 @@ Clip player used to select frames for preview and range selection.
 """
 
 
-import mlt
+try:
+    import mlt7 as mlt
+except:
+    import mlt
 import os
 from os import listdir
 from os.path import isfile, join
@@ -50,17 +53,52 @@ def set_current_profile(clip_path):
     _current_profile = mltprofiles.get_profile_for_index(profile_index)
     return profile_index
 
+def set_current_profile_for_profile_name(profile_name):
+    global _current_profile
+    _current_profile = mltprofiles.get_profile(profile_name)
+
+def get_current_profile_fps():
+    if _current_profile == None:
+        return None
+
+    return _current_profile.fps()
+
 def get_frames_range_writer_for_current_profile(file_path, callback):
     return FramesRangeWriter(file_path, callback, _current_profile)
 
-        
+
+
 class GmicPlayer:
     
-    def __init__(self, clip_path):
+    def __init__(self, clip_path, ticker=None):
         self.producer = mlt.Producer(_current_profile, str(clip_path))
         self.producer.mark_in = -1
         self.producer.mark_out = -1
+        self.ticker = ticker
         
+    def set_producer(self, producer):
+        if self.producer != None:
+            self.consumer.stop()
+            self.consumer.disconnect_all_producers()
+
+            producer.mark_in = self.producer.mark_in
+            producer.mark_out = self.producer.mark_out
+
+        else:
+            producer.mark_in = -1
+            producer.mark_out = -1
+
+        self.producer = producer
+            
+        new_length = self.get_active_length()
+        if new_length <= self.producer.mark_in:
+            self.producer.mark_in = -1
+        if new_length <= self.producer.mark_out:
+            self.producer.mark_in = -1
+            self.producer.mark_out = -1
+    
+        self.connect_and_start()
+                
     def create_sdl_consumer(self):
         """
         Creates consumer with sdl output to a gtk+ widget.
@@ -88,13 +126,29 @@ class GmicPlayer:
         self.producer.set_speed(0)
         self.consumer.connect(self.producer)
         self.consumer.start()
-
+        if self.ticker != None:
+            self.ticker.start_ticker()
+            
     def current_frame(self):
         return self.producer.frame()
 
     def get_active_length(self):
         return self.producer.get_length()
-                
+
+    def start_playback(self):
+        self.producer.set_speed(1)
+        if self.ticker != None:
+            self.ticker.stop_ticker()
+            self.ticker.start_ticker()
+
+    def stop_playback(self):
+        if self.ticker != None:
+            self.ticker.stop_ticker()
+        self.producer.set_speed(0)
+
+    def is_playing(self):
+        return (self.producer.get_speed() != 0)
+        
     def seek_position_normalized(self, pos, length):
         frame_number = pos * length
         self.seek_frame(int(frame_number)) 
@@ -129,7 +183,8 @@ class GmicPlayer:
     def shutdown(self):
         self.producer.set_speed(0)
         self.consumer.stop()
-
+        if self.ticker != None:
+            self.ticker.stop_ticker()
 
 class PreviewFrameWriter:
 
