@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2012 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -44,7 +44,6 @@ disable_audio_encoding = False
 default_profile_index = None
 
 
-
 class ToolsRenderData():
     """
     This is used to save and communicate render selections defined by user
@@ -53,7 +52,7 @@ class ToolsRenderData():
     """
     def __init__(self):
         self.profile_index = None
-        self.use_default_profile = None
+        self.use_default_profile = None # NOT USED, 'profile_index' is the meaningful data here, this one should not have been included in this data struct.
         self.use_preset_encodings = None
         self.presets_index = None
         self.encoding_option_index = None
@@ -68,13 +67,26 @@ class ToolsRenderData():
         self.frame_name = "frame"
         self.is_preview_render = False
         self.is_flatpak_render = False
+    
+    
+    
+def get_args_vals_list_for_render_data(render_data):
+    profile = mltprofiles.get_profile_for_index(render_data.profile_index)
+
+    args_vals_list = renderconsumer.get_args_vals_tuples_list_for_encoding_and_quality( profile, 
+                                                                                        render_data.encoding_option_index, 
+                                                                                        render_data.quality_option_index)
+    # sample rate not supported
+    # args rendering not supported
+
+    return args_vals_list
 
 
 def create_container_clip_default_render_data_object(profile):
     # When ToolsRenderData is used by G'Mic tool we need to have default values be 'None', for container clis we need different
     # default values.
     # 
-    # When first render is attempted this created to have data availeble for render process
+    # When first render is attempted this created to have data available for render process
     # even if user has not set any values.
     render_data = ToolsRenderData()
     render_data.profile_index = mltprofiles.get_profile_index_for_profile(profile)
@@ -105,7 +117,7 @@ def create_widgets(def_profile_index, disable_audio=True, create_container_file_
     widgets.profile_panel = RenderProfilePanel(_out_profile_changed)
     widgets.encoding_panel = RenderEncodingPanel(widgets.file_panel.extension_label)
 
-    widgets.profile_panel.out_profile_combo.fill_options()
+    widgets.profile_panel.out_profile_combo.set_initial_selection()
     _display_default_profile()
     
 def get_encoding_panel(render_data, create_container_file_panel=False):
@@ -118,12 +130,13 @@ def get_encoding_panel(render_data, create_container_file_panel=False):
     else:
         file_panel_title = _("Save Location")
 
-    file_opts_panel = guiutils.get_named_frame(file_panel_title, widgets.file_panel.vbox, 4)         
-    profile_panel = guiutils.get_named_frame(_("Render Profile"), widgets.profile_panel.vbox, 4)
+    file_opts_panel = guiutils.get_named_frame(file_panel_title, widgets.file_panel.vbox, 4)
+    if create_container_file_panel == False:      
+        profile_panel = guiutils.get_named_frame(_("Render Profile"), widgets.profile_panel.vbox, 4)
     encoding_panel = guiutils.get_named_frame(_("Encoding Format"), widgets.encoding_panel.vbox, 4)
 
     if create_container_file_panel == True:
-        widgets.video_clip_panel = RenderVideoClipPanel(profile_panel, encoding_panel)
+        widgets.video_clip_panel = RenderVideoClipPanel(encoding_panel)
         video_clip_panel = guiutils.get_named_frame(_("Video Clip"), widgets.video_clip_panel.vbox, 4)
     else:
         widgets.video_clip_panel = None
@@ -140,9 +153,10 @@ def get_encoding_panel(render_data, create_container_file_panel=False):
         widgets.file_panel.movie_name.set_text(render_data.file_name)
         widgets.file_panel.extension_label.set_text(render_data.file_extension)
         widgets.file_panel.out_folder.set_current_folder(render_data.render_dir + "/")
-        widgets.encoding_panel.encoding_selector.widget.set_active(render_data.encoding_option_index) 
+        widgets.encoding_panel.encoding_selector.categorised_combo.set_selected(mltprofiles.get_profile_for_index(render_data.encoding_option_index).description())
         widgets.encoding_panel.quality_selector.widget.set_active(render_data.quality_option_index)
-        widgets.profile_panel.out_profile_combo.widget.set_active(render_data.profile_index)
+        profile_desc = mltprofiles.get_profile_for_index(render_data.profile_index).description()
+        widgets.profile_panel.out_profile_combo.categories_combo.set_selected(profile_desc)
         widgets.profile_panel.use_project_profile_check.set_active(render_data.use_default_profile)
         
         if create_container_file_panel == True:
@@ -191,9 +205,10 @@ def get_profile_info_text(profile):
 
 def get_render_data_for_current_selections():
     render_data = ToolsRenderData()
-    render_data.profile_index = widgets.profile_panel.out_profile_combo.widget.get_active()
+    profile_desc = widgets.profile_panel.out_profile_combo.categories_combo.get_selected()
+    render_data.profile_index = mltprofiles.get_profile_index_for_profile(mltprofiles.get_profile(profile_desc))
     render_data.use_default_profile = widgets.profile_panel.use_project_profile_check.get_active()
-    render_data.encoding_option_index = widgets.encoding_panel.encoding_selector.widget.get_active()
+    render_data.encoding_option_index = widgets.encoding_panel.encoding_selector.get_selected_encoding_index()
     render_data.quality_option_index = widgets.encoding_panel.quality_selector.widget.get_active()
     render_data.presets_index = 0 # presents rendering not available
     render_data.use_preset_encodings = False # presents rendering not available
@@ -208,27 +223,6 @@ def get_render_data_for_current_selections():
 
     return render_data
 
-def get_args_vals_list_for_render_data(render_data):
-    profile = mltprofiles.get_profile_for_index(render_data.profile_index)
-    if render_data.use_preset_encodings == 1:
-        # Preset encodings THIS HAS BEEN DEACTIVATED FOR NOW.
-        # Preset encodings THIS HAS BEEN DEACTIVATED FOR NOW.
-        # Preset encodings THIS HAS BEEN DEACTIVATED FOR NOW.
-        encs = renderconsumer.non_user_encodings
-        if disable_audio_encoding == True:
-            encs = renderconsumer.get_video_non_user_encodigs()
-        encoding_option = encs[render_data.presets_index]
-        args_vals_list = encoding_option.get_args_vals_tuples_list(profile)
-    
-    
-    else: # User encodings
-        args_vals_list = renderconsumer.get_args_vals_tuples_list_for_encoding_and_quality( profile, 
-                                                                                            render_data.encoding_option_index, 
-                                                                                            render_data.quality_option_index)
-    # sample rate not supported
-    # args rendering not supported
-
-    return args_vals_list
 
 def get_encoding_desc(args_vals_list):
     print(args_vals_list)
@@ -248,6 +242,7 @@ def get_encoding_desc(args_vals_list):
 
 
 # ----------------------------------------------------------------- helper functions
+"""
 def _render_type_changed(w):
     if w.get_active() == 0: # User Defined
         widgets.render_type_panel.presets_selector.widget.set_sensitive(False)
@@ -264,9 +259,10 @@ def _preset_selection_changed(w):
     enc_index = w.get_active()
     ext = encs[enc_index].extension
     widgets.file_panel.extension_label.set_text("." + ext)
-    
-def _out_profile_changed(w):
-    profile = mltprofiles.get_profile_for_index(w.get_active())
+"""
+
+def _out_profile_changed(categorized_combo):
+    profile = mltprofiles.get_profile(categorized_combo.get_selected())
     _fill_info_box(profile)
 
 def _display_default_profile():
@@ -349,9 +345,8 @@ class RenderFilePanel():
 
 
 class RenderVideoClipPanel:
-    def __init__(self, profile_panel, encoding_panel):
+    def __init__(self, encoding_panel):
 
-        self.profile_panel = profile_panel
         self.encoding_panel = encoding_panel
 
         self.video_clip_combo = Gtk.ComboBoxText() # filled later when current sequence known
@@ -361,7 +356,6 @@ class RenderVideoClipPanel:
         self.video_clip_combo.connect('changed', lambda w: self.video_clip_combo_changed(w))
 
         encoding_vbox = Gtk.VBox(False, 2)
-        encoding_vbox.pack_start(profile_panel, False, False, 0)
         encoding_vbox.pack_start(encoding_panel, False, False, 0)
 
         encoding_hbox = Gtk.HBox(False, 2)
@@ -375,7 +369,6 @@ class RenderVideoClipPanel:
         self.vbox.pack_start(encoding_hbox, False, False, 0)
 
     def enable_video_encoding(self, enabled):
-        self.profile_panel.set_sensitive(enabled)
         self.encoding_panel.set_sensitive(enabled)
         
     def video_clip_combo_changed(self, combo):
@@ -440,7 +433,6 @@ class RenderProfilePanel():
         self.out_profile_combo.widget.set_sensitive(checkbutton.get_active() == False)
         if checkbutton.get_active() == True:
             self.out_profile_combo.widget.set_active(default_profile_index)
-            #_display_default_profile()
             
 
 class RenderEncodingPanel():
@@ -473,24 +465,22 @@ class RenderEncodingPanel():
 
 class ProfileSelector():
     def __init__(self, out_profile_changed_callback=None):
-        self.widget = Gtk.ComboBoxText() # filled later when current sequence known
+        self.categories_combo = get_profiles_combo()
+        self.widget = self.categories_combo.widget
         if out_profile_changed_callback != None:
-            self.widget.connect('changed', lambda w:  out_profile_changed_callback(w))
+            self.widget.connect('changed', lambda w:  out_profile_changed_callback(self.categories_combo))
         self.widget.set_sensitive(False)
         self.widget.set_tooltip_text(_("Select render profile"))
+        
+    def set_initial_selection(self):
+        default_profile = mltprofiles.get_profile_for_index(default_profile_index)
+        self.categories_combo.set_selected(default_profile.description())
 
-    def fill_options(self):
-        self.widget.get_model().clear()
-        #self.widget.append_text(current_sequence().profile.description())
-        profiles = mltprofiles.get_profiles()
-        for profile in profiles:
-            self.widget.append_text(profile[0])
-        self.widget.set_active(default_profile_index)
 
 
 class RenderQualitySelector():
     """
-    Component displays quality option relevant for encoding slection.
+    Component displays quality option relevant for encoding selection.
     """
     def __init__(self):
         self.widget = Gtk.ComboBoxText()
@@ -545,11 +535,9 @@ class ProfileInfoBox(Gtk.VBox):
 class RenderEncodingSelector():
 
     def __init__(self, quality_selector, extension_label, audio_desc_label):
-        self.widget = Gtk.ComboBoxText()
-        for encoding in renderconsumer.encoding_options:
-            self.widget.append_text(encoding.name)
-            
-        self.widget.set_active(0)
+        self.categorised_combo = get_encodings_combo()
+        self.widget = self.categorised_combo.widget
+        self.categorised_combo.set_selected(renderconsumer.DEFAULT_ENCODING_NAME)
         self.widget.connect("changed", 
                             lambda w,e: self.encoding_selection_changed(), 
                             None)
@@ -560,15 +548,93 @@ class RenderEncodingSelector():
         self.audio_desc_label = audio_desc_label
         
     def encoding_selection_changed(self):
-        enc_index = self.widget.get_active()
+        try:
+            name, encoding = self.categorised_combo.get_selected()
+            enc_index = renderconsumer.get_encoding_index(encoding)
+            self.quality_selector.update_quality_selection(enc_index)
+            
+            self.extension_label.set_text("." + encoding.extension)
+
+            if self.audio_desc_label != None:
+                self.audio_desc_label.set_markup(encoding.get_audio_description())
+        except:
+            pass # this gets called too early on start-up
+
+    def get_selected_encoding_index(self):
+        name, encoding = self.categorised_combo.get_selected()
+        return renderconsumer.get_encoding_index(encoding)
         
-        self.quality_selector.update_quality_selection(enc_index)
+
+
+def get_profiles_combo():
+    return CategoriesModelComboBox(mltprofiles._categorized_profiles)
+
+
+class CategoriesModelComboBox:
+    
+    def __init__(self, categories_list):
+        self.categories_list = categories_list # categories_list is list of form [("category_name", [category_items]), ...]
+                                               # with category_items list of form ["item_name", ...]
+        self.model = Gtk.TreeStore.new([str])
         
-        encoding = renderconsumer.encoding_options[enc_index]
-        self.extension_label.set_text("." + encoding.extension)
+        for i in range(0, len(categories_list)):
+            name, items = categories_list[i]
+            self.model.append(None, [name])
+            for item_name in items:
+                category_iter = self.model.get_iter_from_string(str(i))
+                self.model.append(category_iter, [item_name])
 
-        if self.audio_desc_label != None:
-            self.audio_desc_label.set_markup(encoding.get_audio_description())
+        self.widget = Gtk.ComboBox.new_with_model(self.model)
+        renderer_text = Gtk.CellRendererText()
+        self.widget.pack_start(renderer_text, True)
+        self.widget.add_attribute(renderer_text, "text", 0)
 
+    def set_selected(self, active_item_name):
+        for i in range(0, len(self.categories_list)):
+            name, items = self.categories_list[i]
+            for j in range(0, len(items)):
+                if items[j] == active_item_name:
+                    iter = self.model.get_iter_from_string(str(i) + ":" + str(j))
+                    self.widget.set_active_iter(iter)
+                    
+    def get_selected(self):        
+        indices = self.model.get_path(self.widget.get_active_iter()).get_indices()
+        name, items = self.categories_list[indices[0]]
+        return items[indices[1]]
 
+def get_encodings_combo():
+    return CategoriesModelComboBoxWithData(renderconsumer.categorized_encoding_options)
 
+class CategoriesModelComboBoxWithData:
+    
+    def __init__(self, categories_list):
+        self.categories_list = categories_list # categories_list is list of form [("category_name", [category_items]), ...]
+                                               # with category_items list of form [("item_name", data_object), ...]
+        self.model = Gtk.TreeStore.new([str])
+        
+        for i in range(0, len(categories_list)):
+            name, items = categories_list[i]
+            self.model.append(None, [name])
+            for item in items:
+                item_name, item_data = item
+                category_iter = self.model.get_iter_from_string(str(i))
+                self.model.append(category_iter, [item_name])
+
+        self.widget = Gtk.ComboBox.new_with_model(self.model)
+        renderer_text = Gtk.CellRendererText()
+        self.widget.pack_start(renderer_text, True)
+        self.widget.add_attribute(renderer_text, "text", 0)
+
+    def set_selected(self, active_item_name):
+        for i in range(0, len(self.categories_list)):
+            name, items = self.categories_list[i]
+            for j in range(0, len(items)):
+                item_name, item_data = items[j]
+                if item_name == active_item_name:
+                    iter = self.model.get_iter_from_string(str(i) + ":" + str(j))
+                    self.widget.set_active_iter(iter)
+                    
+    def get_selected(self):        
+        indices = self.model.get_path(self.widget.get_active_iter()).get_indices()
+        name, items = self.categories_list[indices[0]]
+        return items[indices[1]]
