@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2012 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,16 +22,17 @@
 Module manages undo and redo stacks and executes edit actions from them
 on user requests.
 """
+import time
 import editorstate
 
-set_post_undo_redo_edit_mode = None # This is set at startup to avoid circular imports
+set_post_undo_redo_edit_mode = None # This is set at startup to avoid circular imports.
 repaint_tline = None
 
-# Max stack size
+# Max stack size.
 MAX_UNDOS = 35
 
 # EditActions are placed in this stack after their do_edit()
-# method has been called
+# method has been called.
 undo_stack = []
 
 # Index is the stack pointer that tracks done undos and redos.
@@ -40,7 +41,7 @@ undo_stack = []
 # no redos.
 index = 0
 
-# Some menu items are set active/deactive based on undo stack state
+# Some menu items are set active/deactivate based on undo stack state.
 save_item = None
 undo_item = None 
 redo_item = None
@@ -76,10 +77,11 @@ def register_edit(undo_edit):
         index = index - 1
         
     # Add to stack and grow index
-    undo_stack.append(undo_edit);
+    undo_stack.append(undo_edit)
     index = index + 1
     
-    save_item.set_sensitive(True) # Disabled at load and save, first edit enables
+    if editorstate.PROJECT().last_save_path != None:
+        save_item.set_sensitive(True) # Disabled at load and save, first edit enables if project has been saved.
     undo_item.set_sensitive(True)
     redo_item.set_sensitive(False)
 
@@ -146,18 +148,19 @@ def _set_post_edit_mode():
         set_post_undo_redo_edit_mode()
 
 def undo_redo_stress_test():
+    global undo_stack, index
     times = 10
     delay = 0.100
     
     for r in range(0, times):
-        while undo.index > 0:
-            print "undo:", undo.index
+        while index > 0:
+            print("undo:", index)
             do_undo()
 
             time.sleep(delay)
     
-        while undo.index < len(undo.undo_stack):
-            print "redo:", undo.index
+        while index < len(undo_stack):
+            print("redo:", index)
             do_redo()
 
             time.sleep(delay)
