@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2012 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,11 +23,6 @@ Helper functions and data
 """
 import time
 
-# TODO: THIS NEEDS TO GO, MAKE NEW utilsgtk.py MODULE.
-import gi
-gi.require_version('Gtk', '3.0')
-from gi.repository import Gtk, Gdk
-
 import math
 import hashlib
 import os
@@ -107,13 +102,17 @@ def get_fps_str_with_two_decimals(fps_str):
     except:
         decimals = fps_sides[1]
     return fps_sides[0] + "." + decimals
-    
-def clip_length_string(length):
+
+# frames_per_second needs to be float
+def clip_length_string(length, frames_per_second=None):
     """ 
     Returns length string for length in frames.
     """
-    fr = length % fps()
-    sec = length / fps()
+    if frames_per_second == None:
+        frames_per_second = fps()
+        
+    fr = length % frames_per_second
+    sec = length / frames_per_second
     mins = sec / 60
     sec = int(math.floor(sec % 60))
     hours = int(math.floor(mins / 60))
@@ -233,124 +232,6 @@ def get_track_name(track, sequence):
         text = "A" + str(sequence.first_video_index - track.id)
     return text
 
-def get_media_source_file_filter(include_audio=True):
-    # No idea if these actually play or not, except images mime types
-    f = Gtk.FileFilter()
-    f.set_name("Media MIME types")
-    f.add_mime_type("image*")
-    f.add_mime_type("video*")
-    f.add_mime_type("audio*")
-    f.add_mime_type("video/x-theora+ogg")
-    f.add_mime_type("video/x-sgi-movie")
-    f.add_mime_type("video/ogg")
-    f.add_mime_type("video/x-ogm")
-    f.add_mime_type("video/x-ogm+ogg")
-    f.add_mime_type("video/x-ms-asf")
-    f.add_mime_type("video/x-ms-wmv")
-    f.add_mime_type("video/x-msvideo")
-    f.add_mime_type("video/x-matroska")
-    f.add_mime_type("video/x-flv")
-    f.add_mime_type("video/vnd.rn-realvideo")
-    f.add_mime_type("video/quicktime")
-    f.add_mime_type("video/ogg")
-    f.add_mime_type("video/mpeg")
-    f.add_mime_type("video/mp4")
-    f.add_mime_type("video/mp2t")
-    f.add_mime_type("video/isivideo")
-    f.add_mime_type("video/dv")
-    f.add_mime_type("video/annodex")
-    f.add_mime_type("video/3gpp")
-    f.add_mime_type("video/webm")
-    
-    if include_audio == True:
-        f.add_mime_type("audio/aac")
-        f.add_mime_type("audio/ac3")
-        f.add_mime_type("audio/AMR")
-        f.add_mime_type("audio/ogg")
-        f.add_mime_type("audio/midi")
-        f.add_mime_type("audio/mp2")
-        f.add_mime_type("audio/mp3")
-        f.add_mime_type("audio/mp4")
-        f.add_mime_type("audio/mpeg")
-        f.add_mime_type("audio/ogg")
-        f.add_mime_type("audio/vnd.rn-realaudio")
-        f.add_mime_type("audio/vorbis")
-        f.add_mime_type("audio/x-adpcm")
-        f.add_mime_type("audio/x-aifc")
-        f.add_mime_type("audio/x-aiff")
-        f.add_mime_type("audio/x-aiffc")
-        f.add_mime_type("audio/x-flac")
-        f.add_mime_type("audio/x-flac+ogg")
-        f.add_mime_type("audio/x-m4b")
-        f.add_mime_type("audio/x-matroska")
-        f.add_mime_type("audio/x-ms-wma")
-        f.add_mime_type("audio/x-oggflac")
-        f.add_mime_type("audio/x-ms-asx")
-        f.add_mime_type("audio/x-ms-wma")
-        f.add_mime_type("audio/x-ms-wma")
-        f.add_mime_type("audio/x-gsm")
-        f.add_mime_type("audio/x-riff")
-        f.add_mime_type("audio/x-speex")
-        f.add_mime_type("audio/x-speex+ogg")
-        f.add_mime_type("audio/x-tta")
-        f.add_mime_type("audio/x-voc")
-        f.add_mime_type("audio/x-vorbis+ogg")
-        f.add_mime_type("audio/x-wav")
-    f.add_mime_type("audio/annodex")
-
-    f.add_mime_type("image/bmp")
-    f.add_mime_type("image/tiff")
-    f.add_mime_type("image/gif")
-    f.add_mime_type("image/x-tga")
-    f.add_mime_type("image/png")
-    f.add_mime_type("image/jpeg")
-    f.add_mime_type("image/svg+xml")
-
-    return f
-
-def get_video_source_file_filter():
-    # No idea if these actually play or not, except images mime types
-    f = Gtk.FileFilter()
-    f.set_name("Video files")
-    f.add_mime_type("image*")
-    f.add_mime_type("video*")
-    f.add_mime_type("audio*")
-    f.add_mime_type("video/x-theora+ogg")
-    f.add_mime_type("video/x-sgi-movie")
-    f.add_mime_type("video/ogg")
-    f.add_mime_type("video/x-ogm")
-    f.add_mime_type("video/x-ogm+ogg")
-    f.add_mime_type("video/x-ms-asf")
-    f.add_mime_type("video/x-ms-wmv")
-    f.add_mime_type("video/x-msvideo")
-    f.add_mime_type("video/x-matroska")
-    f.add_mime_type("video/x-flv")
-    f.add_mime_type("video/vnd.rn-realvideo")
-    f.add_mime_type("video/quicktime")
-    f.add_mime_type("video/ogg")
-    f.add_mime_type("video/mpeg")
-    f.add_mime_type("video/mp4")
-    f.add_mime_type("video/mp2t")
-    f.add_mime_type("video/isivideo")
-    f.add_mime_type("video/dv")
-    f.add_mime_type("video/annodex")
-    f.add_mime_type("video/3gpp")
-    f.add_mime_type("video/webm")
-
-    return f
-
-def get_image_sequence_file_filter():
-    f = Gtk.FileFilter()
-    f.set_name("Image files")
-    f.add_mime_type("image/bmp")
-    f.add_mime_type("image/tiff")
-    f.add_mime_type("image/gif")
-    f.add_mime_type("image/x-tga")
-    f.add_mime_type("image/png")
-    f.add_mime_type("image/jpeg")
-
-    return f
-
 def file_extension_is_graphics_file(ext):
     ext = ext.lstrip(".")
     ext = ext.lower()
@@ -482,7 +363,7 @@ def get_img_seq_glob_lookup_name(asset_file_name):
     
     return start + "*" + end
 
-def get_img_seq_resource_name(frame_file, new_style_res_name):
+def get_img_seq_resource_name(frame_file):
     (folder, file_name) = os.path.split(frame_file)
     try:
         number_parts = re.findall("[0-9]+", file_name)
@@ -496,11 +377,7 @@ def get_img_seq_resource_name(frame_file, new_style_res_name):
     path_name_part = file_name[0:number_index]
     end_part = file_name[number_index + len(number_part):len(file_name)]
 
-    # The better version with "?begin=xxx" only available after 0.8.7
-    if new_style_res_name:
-        resource_name_str = path_name_part + "%" + "0" + str(len(number_part)) + "d" + end_part + "?begin=" + number_part
-    else:
-        resource_name_str = path_name_part + "%" + "0" + str(len(number_part)) + "d" + end_part
+    resource_name_str = path_name_part + "%" + "0" + str(len(number_part)) + "d" + end_part
 
     return resource_name_str
 
@@ -594,6 +471,7 @@ _audio_file_extensions = [  "act",
                             "iklax",
                             "m4a",
                             "m4p",
+                            "mka",
                             "mmf",
                             "mp2",
                             "mp3",
@@ -686,8 +564,8 @@ _video_file_extensions = [  "avi",
                             "y4m",
                             "yuv",
                             "xml"]
-                            
-                            
+
+
 def start_timing(msg="start timing"):
     global _start_time
     _start_time = time.time()
@@ -701,32 +579,17 @@ def elapsed_time(msg="elapsed: ", show_in_millis=True):
     else:
         unit = "s"
 
-def get_display_monitors_size_data():
-    monitors_size_data = []
-    
-    display = Gdk.Display.get_default()
-    scr_w = Gdk.Screen.width()
-    scr_h = Gdk.Screen.height()
-    monitors_size_data.append((scr_w, scr_h))
-        
-    num_monitors = display.get_n_monitors() # Get number of monitors.
-    if num_monitors == 1:
-        return monitors_size_data
-    else:
-        for monitor_index in range(0, num_monitors):
-            monitor = display.get_monitor(monitor_index)
-            geom = monitor.get_geometry()
-            monitors_size_data.append((geom.width, geom.height))
-        
-        return monitors_size_data
-
 def unpickle(path):
     try:
         f = open(path, "rb")
-        return pickle.load(f)
+        data = pickle.load(f)
+        f.close()
+        return data
     except:
-        f = open(path, 'rb')
-        return pickle.load(f, encoding='latin1') 
+        f = open(path, "rb")
+        data = pickle.load(f, encoding='latin1') 
+        f.close()
+        return data
 
 def get_flatpak_real_path_for_app_files(app_file):
     # Blender etc. some times need real absolute paths for application script files.
@@ -741,3 +604,32 @@ def get_flatpak_real_path_for_app_files(app_file):
             return app_file_path
     
     return None # Hitting here needs to crash
+
+
+def get_headless_arg_value(args, key_str):
+    for arg in args:
+        parts = arg.split(":")
+        if len(parts) > 1:
+            if parts[0] == key_str:
+                return parts[1]
+    
+    return None
+
+def get_folder_size_recursively(folder):
+    files = os.listdir(folder)
+    size = 0
+    for f in files:
+        if os.path.isdir(folder + "/" + f):
+            size += get_folder_size_recursively(folder + "/" + f)
+        else:
+            size += os.path.getsize(folder +"/" + f)
+    return size
+
+def get_disk_size_str(size):
+    if size > 1000000:
+        return str(int((size + 500000) / 1000000)) + " MB"
+    elif size > 1000:
+        return str(int((size + 500) / 1000)) + " kB"
+    else:
+        return str(int(size)) + " B"
+            
\ No newline at end of file
